/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : keypad_platformservice.h
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the keypad platform service decelerations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _KEYPAD_PLATFORM_SERVICE_H_
#define	_KEYPAD_PLATFORM_SERVICE_H_

/*!
 * Keypad Platform Service configuration
 */
#define KEYPAD_RX_TX_MAX_Q_SIZE		10
#define PS_KPD_TIMEOUT              100

/********* Keypad GPIO *********/
#define KEY_DATA_PIN_1			    GPIO_PIN_12
#define KEY_DATA_PORT_1			    GPIOC
#define KEY_DATA_PIN_2			    GPIO_PIN_2
#define KEY_DATA_PORT_2			    GPIOD
#define KEY_DATA_PIN_3			    GPIO_PIN_3
#define KEY_DATA_PORT_3			    GPIOD
#define KEY_DATA_PIN_4			    GPIO_PIN_13
#define KEY_DATA_PORT_4			    GPIOG

#define KEY_BACKLIGHT_PIN		    GPIO_PIN_6
#define KEY_BACKLIGHT_PORT		    GPIOC

/*!
 * Keypad Mode
 */
#define KEYPAD_INTERRUPT_MODE	    0
#define KEYPAD_POLLING_MODE		    1

/*!
 * Keypad index
 */
#define KPD_FUNC_INDEX	    		0
#define KPD_RESULT_INDEX    		0
#define KPD_STATE_INDEX	    		1

/*!
 * Keypad back light state
 */
#define KEY_BACKLIGHT_ON		    1
#define KEY_BACKLIGHT_OFF		    0

/*!
 * Keypad Function state
 */
#define KP_ACTIVE    		        1
#define KP_INACTIVE    		        2

/*!
 * Key Pressed / Released
 */
#define KEYPAD_RELEASED             0
#define KEYPAD_PRESSED              1

/*!
 * Number of Hardware Keys
 */
#define NUMBER_OF_KEYPAD            4

/*!
 * Function declaration
 */
void Keypad_DB_Init(void);

/*!
 * Keypad SDK Message
 */
typedef enum __Keypad_MSG
{
	KEYPAD1_INTERRUPT_EVENT,
	KEYPAD2_INTERRUPT_EVENT,
	KEYPAD3_INTERRUPT_EVENT,
	KEYPAD4_INTERRUPT_EVENT,
	KEYPAD_MSG_EVENT,
	KEYPAD_SET,
}Keypad_MSG;

/*!
 * Keypad Press event
 */
typedef enum __Keypressevent
{
	KEY_ACTIVE = 1,
	KEY_INACTIVE,
	SHORT_PRESSED,
	LONG_PRESSED,
	CONTINUOUS_PRESSED,
}Keypressevent;

/*!
 * Keypad Events
 */
typedef enum __Keypad_Event
{
	KPD_TMR_EVENT,
	KPD_MSG_EVENT,
}Keypad_Event;

/*!
 * Keypad Configuration
 */
typedef struct _keyconf
{
	uint32_t      kpd_db;
	uint8_t       kpd_prevstatus;
	GPIO_TypeDef  *KPD_PORT;
	uint16_t      KPD_PIN;
	uint32_t      KPD_timeout;
}keyconf;

/*!
 * Function declaration
 */
void HAL_GPIO_EXTI_KP_Callback();
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin);


#endif	//_KEYPAD_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
