/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : DL_Buf_Validation.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Data Layer buffer field validation function

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../Maximatecc/Inc/Data_Layer.h"
#include "../Inc/Common.h"
#include "../Inc/Dl_Buf_Validation.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Global Variables */
int8_t (*dl_buf_func    [(END_BUF512BIT -1) - START_BUF512BIT ])(uint16_t, uint8_t*);

extern pvt_variable pv;
/* USER CODE END Global Variables */

/** @brief DB Validation function
 *
 *  This function initialize the DB Validation function for the each
 *  unsinged 8 bits field.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void DB_Buf_Field_Validation_Reg(void)
{
	/* Clear the Callback memory */
	memset (&dl_buf_func[0], 0x00, ((END_BUF512BIT    -1) - START_BUF512BIT));

#if (SDK_SERVICE_BLE == PS_ENABLE)

    /* BLE_TX_DATA Validation */
	dl_buf_func[BLE_TX_DATA - (START_BUF512BIT + 1)] = validate_ble_data;

    /* BLE_RX_DATA Validation */
	dl_buf_func[BLE_RX_DATA - (START_BUF512BIT + 1)] = validate_ble_data;
#endif

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

    /* CAN1_TX_DATA Validation */
	dl_buf_func[CAN1_TX_DATA - (START_BUF512BIT + 1)] = validate_can_data;

    /* CAN1_TX_DATA Validation */
	dl_buf_func[CAN1_RX_DATA - (START_BUF512BIT + 1)] = validate_can_data;

#endif

}

#if (SDK_SERVICE_BLE == PS_ENABLE)
/** @brief BLE field validation
 *
 *  This function validates the BLE Placeholder and send the data
 *  to platform service.
 *
 *  @param  ble_db_idx  [IN]  : BLE DB Index
 *          ble_data    [IN]  : BLE data
 *
 *  @return  0 : FAILURE
 *  		 1 : SUCCESS
 *
 */
int8_t validate_ble_data(uint16_t ble_db_idx, uint8_t *ble_data)
{
	SetBLE((int8_t *)ble_data, sizeof(ble_data));

	return SUCCESS;
}
#endif	//SDK_SERVICE_BLE

#if (SDK_SERVICE_FDCAN == PS_ENABLE)
/** @brief FDCAN validation
 *
 *  This function validates the FDCAN data and send the data
 *  to platform service.
 *
 *  @param  db_idx  [IN]  : CAN DB Index
 *          data    [IN]  : CAN data
 *
 *  @return  0 : FAILURE
 *  		 1 : SUCCESS
 *
 */
int8_t validate_can_data(uint16_t db_idx, uint8_t *data)
{
	switch(db_idx+1)
	{
		case CAN1_TX_DATA:

			SetCAN(FDCAN_1, data, sizeof(data));
			break;

	}

	return SUCCESS;
}
#endif	//SDK_SERVICE_FDCAN

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
