/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Fdcan_platformservice.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the FDCAN platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0             21/01/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/Fdcan.h"

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

extern pvt_variable pv;

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief CAN Interface deinit
 *
 *  This function will make the CAN PHY Standby pin to High.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void CAN_deinit(void)
{
	/* Make the GPIO to HIGH*/
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_SET);
}

/** @brief CAN Interface reinit
 *
 *  This function will make CAN PHY standby pin to LOW state.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void CAN_reinit(void)
{
	/* Make the GPIO to LOW*/
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_RESET);
}
#endif

/* USER CODE END Includes */
FDCAN_Config_Info_Typedef fdcanFilterConf[MAX_NUM_FDCAN] =
{
		{FDCAN1_ID, FDCAN1_FILTERINDEX, FDCAN1_FILTERTYPE, FDCAN1_FILTERCONFIG,
				FDCAN1_FILTERID1, FDCAN1_FILTERID1,FDCAN1_RXBUFFERIND},
};

FDCAN_Tx_Config_Info_Typedef fdcanTxConf [MAX_NUM_FDCAN] =
{
		{ FDCAN1_IDENTIFIER, FDCAN1_IDTYPE, FDCAN1_TXFRAMETYPE, FDCAN1_DATALENGTH, FDCAN1_ERRORSTATEIND,
				FDCAN1_BITRATESWITCH, FDCAN1_FDFORMATE, FDCAN1_TXEVENTFIFOCONTROL, FDCAN1_MESSAGEMARKER },
};


/** @brief FDCAN Configuration
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void FDCAN_Conf(uint8_t canid)
{
	FDCAN_HandleTypeDef   *hfdcan;

	if (0 == canid)
	{
		hfdcan = pv.hfdcan1;
	}

	/* Validate the Rx FilterConf variable */
	if (!(fdcanFilterConf[canid].FDCAN_id >= ((uint32_t)0x00000000U) ||
			fdcanFilterConf[canid].FDCAN_id <= (uint32_t)0x40000000))
	{
		fdcanFilterConf[canid].FDCAN_id = FDCAN_DEFAULT_ID;
	}

	if (!(fdcanFilterConf[canid].FDCAN_FilterIndex >= 0 &&
			fdcanFilterConf[canid].FDCAN_FilterIndex <= 127))
	{
		fdcanFilterConf[canid].FDCAN_FilterIndex = FDCAN_DEAULT_INDEX;
	}

	if (!(fdcanFilterConf[canid].FDCAN_FilterType >= ((uint32_t)0x00000000U) &&
			fdcanFilterConf[canid].FDCAN_FilterType <= ((uint32_t)0x00000003U)))
	{
		fdcanFilterConf[canid].FDCAN_FilterType = FDCAN_DEFAULT_FILTERTYPE;
	}

	if (!(fdcanFilterConf[canid].FDCAN_FilterConfig >= ((uint32_t)0x00000000U) &&
			fdcanFilterConf[canid].FDCAN_FilterConfig <= ((uint32_t)0x00000007U)))
	{
		fdcanFilterConf[canid].FDCAN_FilterConfig = FDCAN_DEFAULT_FILTERCONFIG;
	}

	if (!(fdcanFilterConf[canid].FDCAN_FilterId1 > ((uint32_t)0x00000000U)))
	{
		fdcanFilterConf[canid].FDCAN_FilterId1 = FDCAN_DEAFULT_FILTERID1;
	}

	if (!(fdcanFilterConf[canid].FDCAN_FilterId2 > ((uint32_t)0x00000000U)))
	{
		fdcanFilterConf[canid].FDCAN_FilterId2 = FDCAN_DEAFULT_FILTERID2;
	}

	if (!(fdcanFilterConf[canid].RxBufferIndex >= 0 &&
			fdcanFilterConf[canid].RxBufferIndex <= 63))
	{
		fdcanFilterConf[canid].RxBufferIndex = FDCAN_DEFAULT_RXBUFFERINDEX;
	}

	/* Set FDCAN MaskFilter */
	FDCAN_SetMaskFilter(hfdcan, fdcanFilterConf[canid].FDCAN_id ,
						fdcanFilterConf[canid].FDCAN_FilterType,
						fdcanFilterConf[canid].FDCAN_FilterConfig,
						fdcanFilterConf[canid].FDCAN_FilterId1,
						fdcanFilterConf[canid].FDCAN_FilterId2, TRUE);

	/* Validate the Tx Config variable */
	if (!(fdcanTxConf[canid].FDCAN_Identifier > 0x0001))
	{
		fdcanTxConf[canid].FDCAN_Identifier = FDCAN_DEFAULT_IDENTIFIER;
	}

	if (fdcanTxConf[canid].FDCAN_IdType != ((uint32_t)0x00000000U) &&
			fdcanTxConf[canid].FDCAN_IdType != ((uint32_t)0x40000000U))
	{
		fdcanTxConf[canid].FDCAN_IdType = FDCAN_DEFAULT_IDTYPE;
	}

	if (fdcanTxConf[canid].FDCAN_TxFrameType != ((uint32_t)0x00000000U) ||
			fdcanTxConf[canid].FDCAN_TxFrameType != ((uint32_t)0x20000000U))
	{
		fdcanTxConf[canid].FDCAN_TxFrameType = FDCAN_DEFAULT_TXFRAME_TYPE;
	}

	if (!((fdcanTxConf[canid].FDCAN_DataLength >= ((uint32_t)0x00000000U)) ||
			(fdcanTxConf[canid].FDCAN_DataLength <= ((uint32_t)0x000F0000U))))
	{
		fdcanTxConf[canid].FDCAN_DataLength = FDCAN_DEFAULT_DATALENGTH;
	}

	if (fdcanTxConf[canid].FDCAN_ErrorStateIndicator != ((uint32_t)0x00000000U) ||
			fdcanTxConf[canid].FDCAN_ErrorStateIndicator != ((uint32_t)0x80000000U))
	{
		fdcanTxConf[canid].FDCAN_ErrorStateIndicator = FDCAN_DEFAULT_ERROR_STAE_IDICATOR;
	}

	if (fdcanTxConf[canid].FDCAN_BitRateSwitch != ((uint32_t)0x00000000U) ||
			fdcanTxConf[canid].FDCAN_BitRateSwitch != ((uint32_t)0x00100000U))
	{
		fdcanTxConf[canid].FDCAN_BitRateSwitch = FDCAN_DEFAULT_BITRATESWITCH;
	}

	if (fdcanTxConf[canid].FDCAN_FDFormate != ((uint32_t)0x00000000U) ||
			fdcanTxConf[canid].FDCAN_FDFormate != ((uint32_t)0x00200000U))
	{
		fdcanTxConf[canid].FDCAN_FDFormate = FDCAN_DEFULT_FDFORMATE;
	}

	if (fdcanTxConf[canid].FDCAN_TxEventFifoControl != ((uint32_t)0x00000000U) ||
			fdcanTxConf[canid].FDCAN_TxEventFifoControl == ((uint32_t)0x00800000U))
	{
		fdcanTxConf[canid].FDCAN_TxEventFifoControl = FDCAN_DEFULT_TXEVENT_FIFO_CONTROL;
	}

	if (fdcanTxConf[canid].FDCAN_MessageMarker != 0)
	{
		fdcanTxConf[canid].FDCAN_MessageMarker = FDCAN_DEFAULT_MESSAGE_MAKER;
	}

	/* Start the FDCAN */
	HAL_FDCAN_Start(hfdcan);
}

/** @brief CAN Standby GPIO Initialization
 *
 *  This function Initialize CAN Standby pin to RESET
 *
 *  @param  None
 *
 *  @return None
 *
 */
void CAN_STANDBY_GPIO_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	__HAL_RCC_GPIOG_CLK_ENABLE();

	/*Configure GPIO pin : PG5 */
	GPIO_InitStruct.Pin = GPIO_PIN_5;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_RESET);
}

/** @brief FDCAN DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t FDCAN_DB_Init(void)
{
	uint8_t val;
	/* Configure the CAN Standby PIN */
	CAN_STANDBY_GPIO_Init();

    /* Initialize the FDCAN1 Interface */
	FDCAN_Conf(0);

	val = CAN_BUS_ON;
	Set_DL(CAN1_CH0_STATES, &val);

	return PS_SUCCESS;
}

/** @brief FDCAN1 Rx Platform service task
 *
 *  This function receives the FDCAN1 data.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void FDCAN1_RxServiceStart(void *pvParameters)
{
	uint8_t *can1_buff;
	uint32_t ret_val = HAL_OK;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(CAN_deinit,NULL,CAN_reinit);
#endif

	/* Receive the CAN data from the CAN interface and
	 * Update into the DB Layer
	 */
	while(1)
	{
		vTaskDelay(FDCAN1_RECEIVE_TASK_DELAY);

		if(HAL_OK == ret_val)
		{
			/* Get the buffer from the Memory */
			can1_buff = (uint8_t *) can1rx__memory_alloc();

			if (NULL == can1_buff)
				continue;
		}

		memset (&pv.rx1Header, 0x00, sizeof(pv.rx1Header));

		/* Receive the data from the FDCAN1 */
		ret_val = FDCAN_RecieiveRxMessage(pv.hfdcan1, FDCAN_RX_FIFO0, &pv.rx1Header, can1_buff);

		if(HAL_OK == ret_val)
		{
			/* Send the data to CAN1 Rx Queue */
			put_can1_data_to_rxbufq(can1_buff);
		}
	}
}

/** @brief FDCAN Platform service task
 *
 *  This function starts the FDCAN Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void FDCAN_ServiceStart(void *pvParameters)
{
	uint8_t               *fdcanbuf;
	uint32_t              cNotifiedValue;
	int8_t                res = 0;
	uint8_t               func;
	FDCAN_HandleTypeDef   *hfdcan;
	uint8_t               canid;
	uint8_t               baud_idx;
	uint32_t              addr;

	while (1)
	{
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( 0x00, ULONG_MAX , &cNotifiedValue, PS_FDCAN_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Receive Message from the FDCAN SDK API */
		res = xQueueReceive(pv.fdtxq, &fdcanbuf , ( TickType_t ) 20 );

		if(res != pdTRUE)
		{
			break;
		}

		func = fdcanbuf[FDCAN_FUNC_INDEX];

		/* Update the result */
		fdcanbuf[FDCAN_RESULT_INDEX] = SUCCESS;

		switch(func)
		{
			case FDCAN1_SET:

				memcpy(&addr, (uint8_t *)&fdcanbuf[FDCAN_DATA_INDEX], sizeof(addr));

				/* FDCAN_ConfigureTx  */
				FDCAN_ConfigureTx(pv.hfdcan1,fdcanTxConf[0].FDCAN_Identifier,
						            fdcanTxConf[0].FDCAN_IdType,
									(uint8_t *)addr,
									FDCAN_TX_BUFFER0);

				/* FDCAN SendMessage To Buffer */
				FDCAN_SendMessageToBuffer(pv.hfdcan1 , FDCAN_TX_BUFFER0 );
				break;

			case FDCAN_ACT:

				/* Get the CAN ID */
				canid = fdcanbuf[FDCAN_FUNC_SUB_INDEX+1];
				if (0 == canid)
				{
					hfdcan = pv.hfdcan1;
				}

				if (SET_FILTER_INDEX_ENABLE == fdcanbuf[FDCAN_FUNC_SUB_INDEX])
				{
					fdcanFilterConf[fdcanbuf[FDCAN_FUNC_SUB_INDEX+1]].FDCAN_FilterEnable =
							fdcanbuf[FDCAN_FUNC_SUB_INDEX+2];
					fdcanFilterConf[fdcanbuf[FDCAN_FUNC_SUB_INDEX+1]].FDCAN_FilterIndex =
							fdcanbuf[FDCAN_FUNC_SUB_INDEX+3];
					fdcanFilterConf[fdcanbuf[FDCAN_FUNC_SUB_INDEX+1]].FDCAN_id =
							fdcanbuf[FDCAN_FUNC_SUB_INDEX+4];

					/* Set FDCAN MaskFilter */
					FDCAN_SetMaskFilter(hfdcan, fdcanFilterConf[canid].FDCAN_id ,
										fdcanFilterConf[canid].FDCAN_FilterType,
										fdcanFilterConf[canid].FDCAN_FilterConfig,
										fdcanFilterConf[canid].FDCAN_FilterId1,
										fdcanFilterConf[canid].FDCAN_FilterId2, TRUE);
				}
				else
				/* Reset the CAN Interface */
				if (SET_FDCAN_RESET == fdcanbuf[FDCAN_FUNC_SUB_INDEX])
				{
					if (TRUE == fdcanbuf[FDCAN_FUNC_SUB_INDEX + 2] )
					{
						HAL_FDCAN_EnterPowerDownMode(hfdcan);
					}
					else
					{
						HAL_FDCAN_ExitPowerDownMode(hfdcan);
					}
				}
				else
				/* ReInitialize the CAN Interface */
				if (SET_FDCAN_DRIVER_RESET == fdcanbuf[FDCAN_FUNC_SUB_INDEX])
				{
					/* UnInitialize the FDCAN device */
					HAL_FDCAN_DeInit(hfdcan);

					/* Initialize the FDCAN device */
					HAL_FDCAN_Init(hfdcan);

				}
				else
				/* ReInitialize the CAN Interface */
				if (SET_FDCAN_BAUDRATE == fdcanbuf[FDCAN_FUNC_SUB_INDEX])
				{
					baud_idx = fdcanbuf[FDCAN_FUNC_SUB_INDEX + 2];

					/* UnInitialize the FDCAN device */
					HAL_FDCAN_DeInit(hfdcan);

					if (0 == canid)
					{
						/* Initialize the FDCAN1 device */
						PS_FDCAN1_Init(baud_idx);
					}
				}
				break;

			default :
				fdcanbuf[FDCAN_RESULT_INDEX] = FAILURE;
				break;
		}
		/* Send the response to the SDK API */
		xQueueSend( pv.fdrxq, ( void * )&fdcanbuf, ( TickType_t ) Q_TICKSTOWAIT );
	}

	/* Clear the Notify Value */
	cNotifiedValue = 0;
}


/** @brief Start FDCAN Platform service
 *
 *  This function initializes the following modules
 *     ===> FDCAN DB
 *     ===> FDCAN platform driver
 *     ===> FDCAN software resources
 *     ===> Start the FDCAN service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t FDCAN_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create FDCAN TxQ for receiving value from the SDK API */
	pv.fdtxq = xQueueCreate( FDCAN_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.fdtxq)
	{
		LOGERROR("Unable to initialize the FDCAN TxQ\r\n");
		return PS_FAILURE;
	}

	/* Create FDCAN RxQ for sending return value to the SDK API */
	pv.fdrxq = xQueueCreate( FDCAN_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.fdrxq)
	{
		LOGERROR("Unable to initialize the FDCAN RxQ\r\n");
		return PS_FAILURE;
	}

	/* Start the FDCAN Platform service thread */
	xReturned = xTaskCreate(FDCAN_ServiceStart, "FDCAN",
			512, NULL, PS_FDCAN_TASK_PRIORITY, &pv.fdcan_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the FDCAN Platform service thread\r\n");
		return PS_FAILURE;
	}

	/* Create CAN RX thread for the FDCAN1 */
	/* Start the FDCAN Platform service thread */
	xReturned = xTaskCreate(FDCAN1_RxServiceStart, "FDCANRx1",
			512, NULL, PS_FDCAN_TASK_PRIORITY, &pv.fdcanrx1_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the FDCAN1 RX Platform service thread\r\n");
		return PS_FAILURE;
	}

	LOGERROR("FDCAN Platform Service task created Successfully \r\n");

	return PS_SUCCESS;

}
#endif
