/*
* main.c - contains program main
*
* Copyright (c) 2014-2019 emotas embedded communication GmbH
*-------------------------------------------------------------------
* $Id: main.c 30444 2020-01-30 13:52:43Z hil $
*
*
*-------------------------------------------------------------------
*
*
*/

/********************************************************************/
/**
* \file
* \brief main routine
*
*/

/* header of standard C - libraries
---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* header of project specific types
---------------------------------------------------------------------------*/

#include "../Inc/Common.h"
#include "../../../../Core/Maximatecc/user_modify_files/gen_define.h"
#include "../../Middlewares/Third_Party/J1939/j1939_Ml/gen_extern_decl.h"
#include "../../../core/Maximatecc/inc/J1939.h"
#include "../../Middlewares/Third_Party/J1939/j1939lib_ml/inc/j1939_j1939.h"
#include "../../Middlewares/Third_Party/J1939/colib_ml/inc/co_drv.h"
#include "../../Middlewares/Third_Party/J1939/j1939lib_ml/inc/j1939_diagnostic.h"
#include "../../Middlewares/Third_Party/J1939/j1939_Ml/gen_extern_decl.h"
#include "../../../core/Maximatecc/inc/Data_Layer.h"
#include "../../../core/Maximatecc/inc/J1939.h"
#include "../../Middlewares/Third_Party/J1939/colib_ml/inc/codrv_FreeRTOS.h"


#if (SDK_SERVICE_J1939 == PS_ENABLE)
/* constant definitions
---------------------------------------------------------------------------*/

/* local defined data types
---------------------------------------------------------------------------*/

/* list of external used functions, if not in headers
--------------------------------------------------------------------------*/
//extern void j1939drvHardwareInit(void);

/* list of global defined functions
---------------------------------------------------------------------------*/

/* list of local defined functions
---------------------------------------------------------------------------*/
static UNSIGNED8 claimAddress(UNSIGNED8 canLine, BOOL_T validAddr, UNSIGNED8 ownClaimAddr);
static void commInd(UNSIGNED8 canLine, CO_COMM_STATE_EVENT_T commEvent);
static void canInd(UNSIGNED8 canLine, CO_CAN_STATE_T canState);
#ifdef __AI280_DM_REQUEST__
static RET_T diagnosticRequest(UNSIGNED8 canLine, UNSIGNED32  pgn, UNSIGNED8 srcNode);
#endif
static void diagnosticReceive(UNSIGNED8 canLine, UNSIGNED32  pgn, UNSIGNED8 srcNode);
static void receivePgn(CO_LINE_TYPE canLine,UNSIGNED32  pgn,UNSIGNED8	node,J1939_RECEIVE_STATE_T state);
extern EXTERN_DECL void codrvWaitForEvent(UNSIGNED32	msecTimeout);
/* external variables
---------------------------------------------------------------------------*/
extern pvt_variable pv;
extern J1939_CONFIG_T	j1939Config;
/*  J1939 PGN configuration */
extern J1939_CONFIG_T  j1939Config_l0;
extern J1939_CONFIG_T  j1939Config_l1;

extern uint8_t   dl_ju8      [(END_J1939U8BIT      -1)   - START_J1939U8BIT   ];
extern uint16_t  dl_ju16     [(END_J1939U16BIT      -1)  - START_J1939U16BIT   ];
extern uint32_t  dl_ju32     [(END_J1939U32BIT      -1)  - START_J1939U32BIT   ];

uint8_t debug_data[250];
/* global variables
---------------------------------------------------------------------------*/
UNSIGNED8	protectLamp = 1;
UNSIGNED8	amberWarningLamp = 3;
UNSIGNED8	redStopLamp = 1;
UNSIGNED8	malFunctionIndicatorLamp = 0;
UNSIGNED8	flashProtectLamp = 1;
UNSIGNED8	flashAmberWarningLamp = 3;
UNSIGNED8	flashRedStopLamp = 1;
UNSIGNED8	flashMalfuncIndicatorLamp = 2;
J1939Interface if_conf[] =
{
	{
		J1939_CAN1_CLAIM_ADDRESS,      /* Claiming Address*/
		J1939_CAN1_ENABLE,  /* FDCAN 1 Channel Value*/
		J1939_CAN1_BITRATE,  /* Bitrate */
		&j1939Config_l0      /* Pointer to the J1939_CONFIG_T structure*/
	},
	{
		85,      /* Claiming Address*/
		0,  /* FDCAN 1 Channel Value */
		250,  /* Bitrate */
		&j1939Config_l1      /* Pointer to the J1939_CONFIG_T structure*/
	}
};
/* local defined variables
---------------------------------------------------------------------------*/


#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief CAN Interface deinit
 *
 *  This function will make the CAN PHY Standby pin to High.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void J1939_deinit(void)
{
	/* Make the GPIO to HIGH*/
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_SET);
}

/** @brief CAN Interface reinit
 *
 *  This function will make CAN PHY standby pin to LOW state.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void J1939_reinit(void)
{
	/* Make the GPIO to LOW*/
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5, GPIO_PIN_RESET);
}
#endif




/** @brief J1939 Control Platform service task
 *
 *  This function starts the J1939 Control Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void J1939_Controlthread(void * pvParameters)
{
	uint8_t  *jbuf;
	uint32_t cNotifiedValue;
	uint8_t  res = 0;
	uint8_t  func;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(J1939_deinit,NULL,J1939_reinit);
#endif
	while(1)
	{
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_J1939_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Process the SDK API message */
		if (cNotifiedValue & (1<<J1939_MSG_EVENT))
		{
			/* Receive Message from the LCD SDK API */
			res = xQueueReceive(pv.jctrltxq, &jbuf , ( TickType_t ) Q_TICKSTOWAIT );

			if(res != pdTRUE)
			{
				break;
			}

			func = jbuf[J1939_FUNC_INDEX];

			/* Update the result */
			jbuf[J1939_FUNC_INDEX] = SUCCESS;
			switch(func)
			{
				case SET_J1939:

					switch(jbuf[J1939_SUB_FUNC_INDEX])
					{
						case CAN1_DEST_ADDRESS:
							memcpy(&pv.jcan1_dest_address,
									&jbuf[J1939_SUB_FUNC_INDEX+1],
									sizeof(pv.jcan1_dest_address));
							break;

						case SEND_CAN1_DM1_MSG:
							pv.send_j1939_can1_dm1 = 1;
							break;

						case SEND_CAN1_DM2_MSG:
							pv.send_j1939_can1_dm2 = 1;
							break;

						case SEND_CAN1_DM3_MSG:
							pv.send_j1939_can1_dm3 = 1;
							break;
					}
					break;

				default:
					jbuf[J1939_RESULT_INDEX] = FAILURE;
					break;
			}

			/* Send the response to the SDK API */
			xQueueSend( pv.jctrlrxq, ( void * )&jbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/***************************************************************************/
/**
* \brief main entry
*
* \param
*	nothing
* \results
*	nothing
*/
void J1939_ServiceStart(void * pvParameters)
{
	/* hardware initialization */
		j1939drvHardwareInit();
#if (J1939_CAN1_ENABLE == PS_ENABLE)
		if (j1939drvCanInit(0, J1939_CAN1_BITRATE) != RET_OK)  {
			Error_Handler();
		}
#endif

//		if (j1939drvTimerSetup(CO_TIMER_INTERVAL) != RET_OK)  {
//			Error_Handler();
//		}
		if (codrvOSConfig() != RET_OK)  {
			Error_Handler();
		}
		/* stack init */
		if (j1939StackInit() != RET_OK)  {
			Error_Handler();
		}
#if (J1939_CAN1_ENABLE == PS_ENABLE)
		/* enable CAN communication */
		if (j1939drvCanEnable(0) != RET_OK)  {
			Error_Handler();
		}
#endif

		/* setup event registration */
		if (j1939EventRegister_COMM_EVENT(commInd) != RET_OK)  {
			Error_Handler();
		}

		if (j1939EventRegister_CAN_STATE(canInd) != RET_OK)  {
			Error_Handler();
		}

#if (J1939_CLAIM_ADDRESS == PS_ENABLE)
		if (j1939EventRegister_CLAIM_ADDRESS(claimAddress) != RET_OK)  {
			Error_Handler();
		}
#endif

		if (j1939EventRegister_RECEIVE_PGN(receivePgn) != RET_OK)  {
			Error_Handler();
		}

		if (j1939EventRegister_DIAGNOSTIC_RECEIVED_PGN(diagnosticReceive) != RET_OK)  {
			Error_Handler();
		}

#ifdef __AI280_DM_REQUEST__
		if (j1939EventRegister_DIAGNOSTIC_REQUESTED_PGN(diagnosticRequest) != RET_OK)  {
			return(RET_EVENT_NO_RESSOURCE);
		}
#endif
		while (1)  {

			/* do j1939 task */
#if (J1939_CAN1_ENABLE == PS_ENABLE)
		while (j1939CommTask(0) == CO_TRUE)
#endif
		{
            /* Check for the DM message to be sent */
			if (1 == pv.send_j1939_can1_dm1)
			{
				j1939RequestPgn(0, J1939_PGN_DM1, pv.jcan1_dest_address);
				pv.send_j1939_can1_dm1 = 0;
			}

			if (1 == pv.send_j1939_can1_dm2)
			{
				j1939RequestPgn(0, J1939_PGN_DM2,  pv.jcan1_dest_address);
				pv.send_j1939_can1_dm2 = 0;
			}

			if (1 == pv.send_j1939_can1_dm3)
			{
				j1939RequestPgn(0, J1939_PGN_DM3,  pv.jcan1_dest_address);
				pv.send_j1939_can1_dm3 = 0;
			}
		}
		codrvWaitForEvent(1);
		}
}

/** @brief Start J1939 Platform service
 *
 *  This function initializes the following modules
 *     ===> Start the J1939 service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t J1939_PS_Start(void)
{
	BaseType_t xReturned;

	/* Start the J1939 Platform service thread */
	xReturned = xTaskCreate(J1939_ServiceStart, "J1939Task",
			512, NULL, PS_J1939_TASK_PRIORITY, &pv.j1939_task);

	if( xReturned != pdPASS )
	{
		LOGERROR("Unable to create the J1939 Task \r\n");
		return PS_FAILURE;
	}

	/* Create J1939 Control TxQ for receiving value from the SDK API */
	pv.jctrltxq = xQueueCreate( J1939_CTRL_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.jctrltxq)
	{
		LOGERROR("Unable to create the J1939 Control TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create J1939 Control RxQ for sending return value to the SDK API */
	pv.jctrlrxq = xQueueCreate( J1939_CTRL_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.jctrlrxq)
	{
		LOGERROR("Unable to create the J1939 Control RxQ \r\n");
		return PS_FAILURE;
	}

	/* Start the J1939 Control thread */
	xReturned = xTaskCreate(J1939_Controlthread, "J1939Control",
			512, NULL, PS_J1939_TASK_PRIORITY, &pv.j1939_ctrltask);

	if( xReturned != pdPASS )
	{
		LOGERROR("Unable to create the J1939 Task \r\n");
		return PS_FAILURE;
	}

	LOGERROR("J1939 Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}


/*********************************************************************/
/*
 * registered function for claim address
 */
static UNSIGNED8 claimAddress(
		UNSIGNED8	canLine,		/* can line */
		BOOL_T valid,				/* claim address valid */
		UNSIGNED8	claimAddr	/* own claim address */
	)
{
	UNSIGNED8 newAddress = claimAddr;
	const UNSIGNED8 iCH_CAN1_Line=0;

	if (valid == CO_TRUE)
	{
		printf("Claim Address %d is valid now...\n", claimAddr);
	}
	else
	{
		printf("Old Address claimed: %d invalid, ", claimAddr);
		if (claimAddr < 220u)
		{
			newAddress = claimAddr + 1u;
		}
		else
		{
			return(255u);
		}

		printf("try new claim Address %d\n", newAddress);

		//Line 1
		if (canLine==iCH_CAN1_Line)
		{
			if(J1939_CAN1_ADDRESS_CLAIM_DYNAMIC)
			{
				if(newAddress<J1939_CAN1_ADDRESS_CLAIM_NEXT_ADDRESS)
					newAddress=J1939_CAN1_ADDRESS_CLAIM_NEXT_ADDRESS;
				return(newAddress); //Try new Address to Claim
			}
			else
			{
				return(255u); //Mute and stay only listening
//	      		return(claimAddr); //Stay talking on collision address
			}
		}
	}
	return(newAddress);
}

extern uint16_t temp_speed;

/*
 * registered function for receive PGNs
 */
static void receivePgn(

		CO_LINE_TYPE canLine,
		UNSIGNED32  pgn,
		UNSIGNED8	node,
		J1939_RECEIVE_STATE_T state
	)
{

#if((BLE_J1939_DEBUG_DATA == PS_ENABLE) && (SDK_SERVICE_BLE == PS_ENABLE))
	/* Send the J1939 Debug data to BLE */
	sendSDKPkt(debug_data, sizeof(debug_data));
#endif
}

/* Convert Data to Real Value */
/** @brief ConvertReadableData
 *
 *  This function will convert raw data from PGN to real value
 *
 *  ivalue  ivalue         : raw data
 *  		mapping_ext	   : array structure with format data
 *  		rowel		   : row element to convert
 *
 *  @return  float         :real value
 *
 */
float ConvertReadableData(uint32_t ivalue, const J1939_MAPPING_T_EXT mapping_ext[], uint8_t rowel)
{
	 float    fResPerBit = mapping_ext[rowel].fResPerBit;
	 float    fOffset = mapping_ext[rowel].Offset;
	 return   (float_t)(ivalue)*(fResPerBit)+(fOffset);
}
/*********************************************************************/
/*
 * registered function for diagnostic requests
 */
static void diagnosticReceive(
		UNSIGNED8	canLine,
		UNSIGNED32  pgn,			/* PGN requested */
		UNSIGNED8	srcNode			/* requested node */
	)
{
RET_T	retVal;

//UNSIGNED32	spn;
#if (EMTOS_J1939_DM1 == PS_ENABLE)
	if (pgn == J1939_PGN_DM1)  {
		printf("DM1 received\n");
		do {
			retVal = j1939DiagnosticGet_DM1(canLine,
				&dl_ju8[DM1_PROTECT_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_AMBER_WARN_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_RED_STOP_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_MAL_FUNC_IND_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_FLASH_PROT_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_FLASH_AMBER_WARN_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_FLASH_RED_STOP_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_FLASH_MAL_FUNC_IND_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju32[DM1_SPN - (START_J1939U32BIT + 1)],
				&dl_ju8[DM1_FMI - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_SPN_CONV_METHOD - (START_J1939U8BIT + 1)],
				&dl_ju8[DM1_OC - (START_J1939U8BIT + 1)]);
		} while (retVal == RET_SERVICE_BUSY);
#if (EMTOS_J1939_DM2 == PS_ENABLE)
		//j1939RequestPgn(canLine, J1939_PGN_DM2, 0x84);
#endif
	}
#endif
#if (EMTOS_J1939_DM2 == PS_ENABLE)
	if (pgn == J1939_PGN_DM2)  {
		printf("DM2 received\n");

		do {
			retVal = j1939DiagnosticGet_DM2(canLine,
				&dl_ju8[DM2_PROTECT_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_AMBER_WARN_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_RED_STOP_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_MAL_FUNC_IND_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_FLASH_PROT_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_FLASH_AMBER_WARN_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_FLASH_RED_STOP_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_FLASH_MAL_FUNC_IND_LAMP - (START_J1939U8BIT + 1)],
				&dl_ju32[DM2_SPN - (START_J1939U32BIT + 1)],
				&dl_ju8[DM2_FMI - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_SPN_CONV_METHOD - (START_J1939U8BIT + 1)],
				&dl_ju8[DM2_OC - (START_J1939U8BIT + 1)]);
		} while (retVal == RET_SERVICE_BUSY);
	}
#endif
}

#ifdef __AI280_DM_REQUEST__

/*********************************************************************/
/*
 * registered function for diagnostic requests
 */
static RET_T diagnosticRequest(
		UNSIGNED8	canLine,		/* can line */
		UNSIGNED32  pgn,			/* PGN requested */
		UNSIGNED8	srcNode			/* requested node */
	)
{

	if (pgn == J1939_PGN_DM1)  {
		/* call DM1 message */
		j1939DiagnosticMap_DM1(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10000, 3, 0, 5
		);
		j1939DiagnosticMap_DM1(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 1, 0, 4
		);
		j1939DiagnosticSend_DM1(canLine);
	}

	if (pgn == J1939_PGN_DM2)  {
		/* call DM2 message */
		j1939DiagnosticMap_DM2(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10000, 3, 0, 5);
		j1939DiagnosticMap_DM2(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 2, 0, 6);
		j1939DiagnosticMap_DM2(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10000, 3, 0, 5);
		j1939DiagnosticSend_DM2(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM3)  {
		/* call DM3 message */
		j1939DiagnosticSend_DM3(canLine, srcNode, 0u);
	}

	if (pgn == J1939_PGN_DM4)  {
		j1939DiagnosticMap_DM4(canLine,
			10000, 3, 0, 5,
			1, 2, 4, 4, 5, 6, (UNSIGNED8 *)"Msg 1", 5);
		j1939DiagnosticMap_DM4(canLine,
			10001, 2, 0, 6,
			1, 2, 4, 4, 5, 6, (UNSIGNED8 *)"Msg 2", 5);

		j1939DiagnosticSend_DM4(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM5)  {
		j1939DiagnosticSend_DM5(canLine, srcNode, 4, 3, 0x55, 0, 0x1234, 0x5678);

	}

	if (pgn == J1939_PGN_DM6)  {
		/* call DM6 message */
		j1939DiagnosticMap_DM6(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 2, 0, 6
		);
		j1939DiagnosticMap_DM6(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10002, 2, 0, 6
		);
		j1939DiagnosticSend_DM6(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM7)  {
		UNSIGNED8	testIdent;
		UNSIGNED32	spn;
		UNSIGNED8	fmi;
		j1939DiagnosticGetPara_DM7(canLine, &testIdent, &spn, &fmi);

		/* send test result */
		j1939DiagnosticMap_DM8(canLine, testIdent, 23, 0, 1, 1);
		j1939DiagnosticSend_DM8(canLine, srcNode);

	}

	if (pgn == J1939_PGN_DM11)  {
		j1939DiagnosticSend_DM11(canLine, srcNode, 0u);
	}

	if (pgn == J1939_PGN_DM13)  {
		UNSIGNED8 currentDataLink, j1587Network;
		UNSIGNED8 j1922Network, j1939Network_1;
		UNSIGNED8 j1939Network_2, iso9141Network;
		UNSIGNED8 j1850Network, manuNetwork;
		UNSIGNED8 j1939Network_3, propNetwork_1;
		UNSIGNED8 propNetwork_2, j1939Network_4;
		UNSIGNED8 holdSignal, suspendSignal;
		UNSIGNED16 suspendDuration;

		printf("DM13 Request:\n");

		j1939DiagnosticGetPara_DM13(canLine,
			&currentDataLink, &j1587Network, &j1922Network, &j1939Network_1,
			&j1939Network_2, &iso9141Network, &j1850Network, &manuNetwork,
			&j1939Network_3, &propNetwork_1, &propNetwork_2, &j1939Network_4,
			&holdSignal, &suspendSignal, &suspendDuration);
		printf("currentDataLink %d, ", currentDataLink);
		printf("j1587Network %d, ", j1587Network);
		printf("j1922Network %d, ", j1922Network);
		printf("j1939Network_1 %d, ", j1939Network_1);
		printf("j1939Network_2 %d, ", j1939Network_2);
		printf("iso9141Network %d, ", iso9141Network);
		printf("j1850Network %d, ", j1850Network);
		printf("manuNetwork %d, ", manuNetwork);
		printf("j1939Network_3 %d, ", j1939Network_3);
		printf("propNetwork_1 %d, ", propNetwork_1);
		printf("propNetwork_2 %d, ", propNetwork_2);
		printf("j1939Network_4 %d, ", j1939Network_4);
		printf("holdSignal %d, ", holdSignal);
		printf("suspendSignal %d, ", suspendSignal);
		printf("suspendDuration %d\n", suspendDuration);
		if (j1939Network_1 == 0)  {
			/* stop broadcast for ET1 */
			j1939PgnDisable(canLine, DIR_TRANSMIT, 65262);
		} else
		if (j1939Network_1 == 1)  {
			/* start broadcast for ET1 */
			j1939PgnEnable(canLine, DIR_TRANSMIT, 65262);
		}
	}

	if (pgn == J1939_PGN_DM14)  {
		UNSIGNED16	lenNoReq;		/**< length/number requested */
		UNSIGNED8	type;			/**< pointer type */
		UNSIGNED8	command;		/**< command */
		UNSIGNED32	mem;			/**< memory pointer (24bit) */
		UNSIGNED8	ptrExtension;	/**< pointer extension */
		UNSIGNED16	key;			/**< key/user level */
		j1939DiagnosticGetPara_DM14(canLine,
			&lenNoReq,		/**< length/number requested */
			&type,			/**< pointer type */
			&command,		/**< command */
			&mem,			/**< memory pointer (24bit) */
			&ptrExtension,	/**< pointer extension */
			&key			/**< key/user level */
		);
		printf("DM14 Request:\n");
		printf("len: %d, type: %d, cmd: %d, ptr: %d, key: %d\n",
			lenNoReq, type,	command, mem, key);


		/* send answer */
		j1939DiagnosticSend_DM15(canLine, srcNode,
			lenNoReq, 1, 0x123456, 0x78, 0x55aa);

		j1939DiagnosticSend_DM16(canLine, srcNode, (UNSIGNED8 *)"12345678", 8);
	}


	if (pgn == J1939_PGN_DM16)  {
		UNSIGNED32	len;
		# define MAX_LEN	20
		UNSIGNED8	buf[MAX_LEN];

		/* data transfer received */
		j1939DiagnosticGetPara_DM16(canLine, &len, &buf[0], MAX_LEN);

	}

	if (pgn == J1939_PGN_DM17)  {
		UNSIGNED8	buf[8];

		j1939DiagnosticGetPara_DM17(canLine, &buf[0]);
	}



	if (pgn == J1939_PGN_DM21)  {
		j1939DiagnosticSend_DM21(canLine, srcNode, 0x123, 0x456, 0x1234, 0x5678);
	}

	if (pgn == J1939_PGN_DM22)  {
		j1939DiagnosticSend_DM22(canLine, srcNode, 0x1, 0x2, 0x1234, 3);
	}

	if (pgn == J1939_PGN_DM23)  {
		j1939DiagnosticMap_DM23(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10002, 2, 0, 6
		);
		j1939DiagnosticSend_DM23(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM24)  {
		j1939DiagnosticMap_DM24(canLine, 10001, 4, 1, 1, 0);
		j1939DiagnosticMap_DM24(canLine, 10002, 4, 1, 1, 0);
		j1939DiagnosticMap_DM24(canLine, 10003, 4, 1, 1, 0);
		j1939DiagnosticSend_DM24(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM25)  {
		j1939DiagnosticMap_DM25(canLine,
			10000, 3, 0, 5,
			(UNSIGNED8 *)"Data 1", 6);
		j1939DiagnosticMap_DM25(canLine,
			10001, 2, 0, 6,
			(UNSIGNED8 *)"Data 2", 6);

		j1939DiagnosticSend_DM25(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM26)  {
		j1939DiagnosticSend_DM26(canLine, srcNode, 12345, 0, 1, 0x1234);
	}

	if (pgn == J1939_PGN_DM27)  {
		/* call DM6 message */
		j1939DiagnosticMap_DM27(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 2, 0, 6
		);
		j1939DiagnosticMap_DM27(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10002, 2, 0, 6
		);
		j1939DiagnosticSend_DM27(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM28)  {
		/* call DM6 message */
		j1939DiagnosticMap_DM28(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 2, 0, 6
		);
		j1939DiagnosticMap_DM28(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10002, 2, 0, 6
		);
		j1939DiagnosticSend_DM28(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM29)  {
		j1939DiagnosticSend_DM29(canLine, srcNode, 1, 2, 3, 4, 5);
	}

	if (pgn == J1939_PGN_DM31)  {
		/* call DM31 message */
		j1939DiagnosticMap_DM31(canLine,
			protectLamp, amberWarningLamp,
			redStopLamp, malFunctionIndicatorLamp,
			flashProtectLamp, flashAmberWarningLamp,
			flashRedStopLamp, flashMalfuncIndicatorLamp,
			10001, 2, 0, 6
		);
		j1939DiagnosticSend_DM31(canLine, srcNode);
	}

	if (pgn == J1939_PGN_DM32)  {
		/* call DM31 message */
		j1939DiagnosticMap_DM32(canLine,
			0x1234, 3, 1000, 1000, 2);
		j1939DiagnosticSend_DM32(canLine, srcNode);
	}
	return(RET_OK);
}
#endif

/*********************************************************************/
/*
 * registered function for CAN events
 */
static void canInd(
		UNSIGNED8	canLine,		/* can line */
		CO_CAN_STATE_T  canState
	)
{
	switch (canState)  {
	case CO_CAN_STATE_BUS_OFF:
		printf("CAN: Bus Off\n");
		break;
	case CO_CAN_STATE_BUS_ON:
		printf("CAN: Bus On\n");
		break;
	case CO_CAN_STATE_PASSIVE:
		printf("CAN: Passive\n");
		break;
	default:
		break;
	}
}


/*********************************************************************/
/*
 * registered function for Communication events
 */
static void commInd(
		UNSIGNED8	canLine,		/* can line */
		CO_COMM_STATE_EVENT_T   commEvent
	)
{
	switch (commEvent)  {
	case CO_COMM_STATE_EVENT_CAN_OVERRUN:
		printf("COMM-Event: CAN Overrun\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
		printf("COMM-Event: Rec Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
		printf("COMM-Event: Rec Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_EMPTY:
		printf("COMM-Event: Rec Queue Empty\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
		printf("COMM-Event: Tr Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
		printf("COMM-Event: Tr Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
		printf("COMM-Event: Tr Queue Empty\n");
		break;
	default:
		printf("COMM-Event: %d\n", commEvent);
		break;
	}

}
/*********************************************************************/
void errorHandler(int error)
{
	while (1)
	{

	}
}

#endif
