/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : keypad_platformservice.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Keypad platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */

/* USER CODE END Macros */

/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
keyconf  kpd_conf[NUMBER_OF_KEYPAD] = {

#if (CONF_KEYPAD_01_STATE == PS_ENABLE)
	{KEY_STATUS_01, KEYPAD_RELEASED, KEY_DATA_PORT_1, KEY_DATA_PIN_1, 0},
#else
	{KEY_STATUS_01_DUMMY, KEYPAD_RELEASED, KEY_DATA_PORT_1, KEY_DATA_PIN_1, 0},
#endif

#if (CONF_KEYPAD_02_STATE == PS_ENABLE)
	{KEY_STATUS_02, KEYPAD_RELEASED, KEY_DATA_PORT_2, KEY_DATA_PIN_2, 0},
#else
	{KEY_STATUS_02_DUMMY, KEYPAD_RELEASED, KEY_DATA_PORT_2, KEY_DATA_PIN_2, 0},
#endif

#if (CONF_KEYPAD_03_STATE == PS_ENABLE)
	{KEY_STATUS_03, KEYPAD_RELEASED, KEY_DATA_PORT_3, KEY_DATA_PIN_3, 0},
#else
	{KEY_STATUS_03_DUMMY, KEYPAD_RELEASED, KEY_DATA_PORT_3, KEY_DATA_PIN_3, 0},
#endif

#if (CONF_KEYPAD_04_STATE == PS_ENABLE)
	{KEY_STATUS_04, KEYPAD_RELEASED, KEY_DATA_PORT_4, KEY_DATA_PIN_4, 0}
#else
	{KEY_STATUS_04_DUMMY, KEYPAD_RELEASED, KEY_DATA_PORT_4, KEY_DATA_PIN_4, 0}
#endif

};

extern pvt_variable pv;
/* USER CODE END Global Variables */

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief keypad deinit
 *
 *  This function will make the keypad pin to deinit.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void keypad_deinit(void)
{
	uint8_t state = 0;
	/* deinitialize the keypad GPIO pins */
	/* key_1 - PC-12 */
	HAL_GPIO_DeInit(KEY_DATA_PORT_1, KEY_DATA_PIN_1);

	/* If the Keypad backlight is ON, OFF the Keypad backlight */
	Get_DL(KEYPAD_BACKLIGHT, &state);

	if(KEY_BACKLIGHT_ON == state)
	{
		/* KEY_backlightOff */
		HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_RESET);
	}

	/* key_2 - PD-2,
	 * key_3 - PD-3
	 **/
	HAL_GPIO_DeInit(GPIOD, KEY_DATA_PIN_2 | KEY_DATA_PIN_3);

	/* key_4 - PG-13,
	 **/
	HAL_GPIO_DeInit(KEY_DATA_PORT_4, KEY_DATA_PIN_4);

	HAL_NVIC_DisableIRQ(EXTI15_10_IRQn);
	HAL_NVIC_DisableIRQ(EXTI2_IRQn);
	HAL_NVIC_DisableIRQ(EXTI3_IRQn);
}

/** @brief keypad reinit
 *
 *  This function will make the keypad pin to reinitialize.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void keypad_reinit(void)
{
	uint8_t state =0;
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/*Configure GPIO pin : KEY_DATA_PIN_1 */
#if(CONF_KEYPAD_01_STATE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_1;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_1, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI15_10_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

#endif

	/*Configure GPIO pins : KEY_DATA_PIN_2 */
#if(CONF_KEYPAD_02_STATE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_2;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_2, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI2_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI2_IRQn);

#endif

	/*Configure GPIO pins : KEY_DATA_PIN_3 */
#if(CONF_KEYPAD_03_STATE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_3;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_3, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI3_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI3_IRQn);

#endif

	/*Configure GPIO pins : KEY_DATA_PIN_4 */
#if(CONF_KEYPAD_04_STATE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_4, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI15_10_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

#endif

	/* If the Keypad backlight is ON, ON the Keypad backlight */
	Get_DL(KEYPAD_BACKLIGHT, &state);

	if(KEY_BACKLIGHT_ON == state)
	{
		/* KEY_backlightON */
		HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_SET);
	}
}

/** @brief keypad wakeup source
 *
 *  This function will make the keypad wakeup source
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void keypad_Wakeup_Source(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/* KEYPAD01_WAKEUP_SOURCE */
#if(KEYPAD01_WAKEUP_SOURCE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_1;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_1, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI15_10_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

#endif

	/* KEYPAD02_WAKEUP_SOURCE */
#if(KEYPAD02_WAKEUP_SOURCE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_2;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_2, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI2_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI2_IRQn);

#endif

	/* KEYPAD03_WAKEUP_SOURCE */
#if(KEYPAD03_WAKEUP_SOURCE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_3;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_3, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI3_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI3_IRQn);

#endif

	/* KEYPAD04_WAKEUP_SOURCE */
#if(KEYPAD04_WAKEUP_SOURCE == PS_ENABLE)

	GPIO_InitStruct.Pin = KEY_DATA_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	HAL_GPIO_Init(KEY_DATA_PORT_4, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI15_10_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

#endif

}
#endif

/** @brief Keypad DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void Keypad_DB_Init(void)
{
	uint8_t state;

	/* Initialize all the DB variables */
	state = KEYPAD_BACKLIGHT_CFG_STATE;

	if(KEY_BACKLIGHT_ON == state)
	{
		/* KEY_backlightOn */
		HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_SET);
	}
	else if(KEY_BACKLIGHT_OFF == state)
	{
		/* KEY_backlightOff */
		HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_RESET);
	}

    Set_DL(KEYPAD_BACKLIGHT, &state);

#if (CONF_KEYPAD_01_STATE == PS_ENABLE)
    state = KP_INACTIVE;
    Set_DL(KEY_STATUS_01, &state);
#endif

#if (CONF_KEYPAD_02_STATE == PS_ENABLE)
    state = KP_INACTIVE;
    Set_DL(KEY_STATUS_02, &state);
#endif

#if (CONF_KEYPAD_03_STATE == PS_ENABLE)
    state = KP_INACTIVE;
    Set_DL(KEY_STATUS_03, &state);
#endif

#if (CONF_KEYPAD_04_STATE == PS_ENABLE)
    state = KP_INACTIVE;
    Set_DL(KEY_STATUS_04, &state);
#endif

}


/** @brief Keypad Timer Callback
 *
 *  This function is used for the keypad state
 *  functionality
 *
 *  @param  xTimer : Keypad timer handle
 *
 *  @return none
 *
 */
void KPDTimerCallback( TimerHandle_t xTimer )
{
	/* Notify the timer callback to Keypad task */
	xTaskNotify( pv.kpd_task, (1<<KPD_TMR_EVENT), eSetBits );
}

/** @brief GPIO ISR From the keypad Button Callback
 *
 *  This function process the Keypad P4 pin
 *
 *  @param
 *
 *  @return none
 *
 */
void HAL_GPIO_EXTI_KP_Callback()
{
	/* Send Notify to the Keypad task to update the Key Status */
	xTaskNotifyFromISR( pv.kpd_task, (1<<KEYPAD4_INTERRUPT_EVENT) , eSetBits, NULL );
}

/** @brief GPIO ISR From the keypad Button Callback
 *
 *  This function process the KP1, KP2 and KP3 pins
 *
 *  @param GPIO_Pin : GPIO Pin
 *
 *  @return none
 *
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	/* Prevent unused argument(s) compilation warning */
   UNUSED(GPIO_Pin);

   /* Check for the Keypad Interrupt */
   if (GPIO_Pin == KEY_DATA_PIN_1)
   {
		/* Send Notify to the Keypad task to update the Key Status */
		xTaskNotifyFromISR( pv.kpd_task, (1<<KEYPAD1_INTERRUPT_EVENT) , eSetBits, NULL );
   }

   /* Check for the Keypad Interrupt */
   if (GPIO_Pin == KEY_DATA_PIN_2)
   {
		/* Send Notify to the Keypad task to update the Key Status */
		xTaskNotifyFromISR( pv.kpd_task, (1<<KEYPAD2_INTERRUPT_EVENT) , eSetBits, NULL );
   }

   /* Check for the Keypad Interrupt */
   if (GPIO_Pin == KEY_DATA_PIN_3)
   {
		/* Send Notify to the Keypad task to update the Key Status */
		xTaskNotifyFromISR( pv.kpd_task, (1<<KEYPAD3_INTERRUPT_EVENT) , eSetBits, NULL );
   }

}


/** @brief Process Keypad event
 *
 *  This function update the keystate to
 *  SHORT_PRESSED/LONG_PRESSED to data layer data base
 *
 *  @param  none
 *
 *  @return 1 : KEYPAD_PRESSED
 *          0 : KEYPAD_RELEASED
 *
 */
uint8_t process_kpd_event(void)
{
    uint8_t index;
    uint8_t port_state = 0;
    uint8_t keypressed = 0;

    for (index = 0; index < NUMBER_OF_KEYPAD; index++)
    {
    	/* Read Keypad IO Pin*/
    	port_state = HAL_GPIO_ReadPin(kpd_conf[index].KPD_PORT, kpd_conf[index].KPD_PIN);

    	/* Update the Status in the DB */
    	if ((KEYPAD_PRESSED == kpd_conf[index].kpd_prevstatus) &&
    			(KEYPAD_RELEASED == (!port_state)))
        {
    		if (kpd_conf[index].KPD_timeout < SHORT_PRESS_TIMEOUT)
    		{
    			port_state = SHORT_PRESSED;
    		}
    		else
    		{
    		    /* Update the short press key in the DB */
    			port_state = LONG_PRESSED;
    		}
		    /* Update the short press key in the DB */
			Set_DL(kpd_conf[index].kpd_db, &port_state);

			kpd_conf[index].kpd_prevstatus = KEYPAD_RELEASED;
			kpd_conf[index].KPD_timeout = 0;
        }
        else
        if (KEYPAD_PRESSED == (!port_state))
		{
			keypressed = KEYPAD_PRESSED;
			kpd_conf[index].KPD_timeout++;
			if (kpd_conf[index].KPD_timeout > SHORT_PRESS_TIMEOUT)
			{
				port_state = CONTINUOUS_PRESSED;
				Set_DL(kpd_conf[index].kpd_db, &port_state);
			}
		}
    }

    return keypressed;
}

/** @brief Keypad Platform service task
 *
 *  This function starts the Keypad Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void Keypad_ServiceStart(void * pvParameters)
{
	uint8_t  *keypadbuf;
	uint32_t cNotifiedValue;
	uint8_t  res = 0;
	int32_t  kpd_timeout = -1;
	uint8_t  func;

	while(1)
	{
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, kpd_timeout);

		if(res != pdTRUE)
		{
			kpd_timeout = -1;

			/* Press the Key release event */
			res = process_kpd_event();
			if (KEYPAD_PRESSED == res)
				kpd_timeout = PS_KPD_TIMEOUT;

			continue;
		}

		/* Process the Keypad Interrupt */
		if (cNotifiedValue & (1<<KEYPAD1_INTERRUPT_EVENT))
		{
			kpd_conf[KEYPAD1_INTERRUPT_EVENT].kpd_prevstatus = KEYPAD_PRESSED;
			kpd_timeout = PS_KPD_TIMEOUT;

#if (CONF_KEYPAD_01_STATE == PS_ENABLE)
			res = KP_ACTIVE;
			Set_DL(KEY_STATUS_01, &res);
#endif
		}

		/* Process the Keypad Interrupt */
		if (cNotifiedValue & (1<<KEYPAD2_INTERRUPT_EVENT))
		{
			kpd_conf[KEYPAD2_INTERRUPT_EVENT].kpd_prevstatus = KEYPAD_PRESSED;
			kpd_timeout = PS_KPD_TIMEOUT;

#if (CONF_KEYPAD_02_STATE == PS_ENABLE)
			res = KP_ACTIVE;
			Set_DL(KEY_STATUS_02, &res);
#endif
		}

		/* Process the Keypad Interrupt */
		if (cNotifiedValue & (1<<KEYPAD3_INTERRUPT_EVENT))
		{
			kpd_conf[KEYPAD3_INTERRUPT_EVENT].kpd_prevstatus = KEYPAD_PRESSED;
			kpd_timeout = PS_KPD_TIMEOUT;

#if (CONF_KEYPAD_03_STATE == PS_ENABLE)
			res = KP_ACTIVE;
			Set_DL(KEY_STATUS_03, &res);
#endif
		}

		/* Process the Keypad Interrupt */
		if (cNotifiedValue & (1<<KEYPAD4_INTERRUPT_EVENT))
		{
			kpd_conf[KEYPAD4_INTERRUPT_EVENT].kpd_prevstatus = KEYPAD_PRESSED;
			kpd_timeout = PS_KPD_TIMEOUT;

#if (CONF_KEYPAD_04_STATE == PS_ENABLE)
			res = KP_ACTIVE;
			Set_DL(KEY_STATUS_04, &res);
#endif
		}

		/* Process the SDK API message */
		if (cNotifiedValue & (1<<KEYPAD_MSG_EVENT))
		{
			/* Receive Message from the keypad SDK API */
			res = xQueueReceive(pv.kpdtxq, &keypadbuf , ( TickType_t ) Q_TICKSTOWAIT );

			if(res != pdTRUE)
			{
				break;
			}

			func = keypadbuf[KPD_FUNC_INDEX];

			/* Update the result */
			keypadbuf[KPD_RESULT_INDEX] = SUCCESS;

			switch(func)
			{
				case KEYPAD_SET:

					/* call the HAL API to SET or RESET the back light pin state */
					if(keypadbuf[KPD_STATE_INDEX] == KEY_BACKLIGHT_ON)
					{
						/* KEY_backlightOn */
						HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_SET);
					}
					else if(keypadbuf[KPD_STATE_INDEX] == KEY_BACKLIGHT_OFF)
					{
						/* KEY_backlightOff */
						HAL_GPIO_WritePin(KEY_BACKLIGHT_PORT, KEY_BACKLIGHT_PIN, GPIO_PIN_RESET);
					}
					break;
			}
			/* Send the response to the SDK API */
			xQueueSend( pv.kpdrxq, ( void * )&keypadbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start Keypad Platform service
 *
 *  This function initializes the following modules
 *     ===> Keypad DB
 *     ===> Keypad platform driver
 *     ===> Keypad software resources
 *     ===> Start the Keypad service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t Keypad_PS_Start(void)
{
	BaseType_t xReturned;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(keypad_deinit,keypad_Wakeup_Source,keypad_reinit);
#endif

	/* Create the Keypad timer */
	pv.xTimers[ PS_KPD_TIMER ] = xTimerCreate
	                   ( "KPD_Timer",
	                     PS_KPD_TIMEOUT,
	                     pdTRUE,
	                     ( void * ) 0,
	                     KPDTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_KPD_TIMER ])
	{
		LOGERROR("Unable to initialize the Keypad timer\r\n");
		return PS_FAILURE;
	}

	/* Create Keypad TxQ for receiving value from the SDK API */
	pv.kpdtxq = xQueueCreate( KEYPAD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.kpdtxq)
	{
		LOGERROR("Unable to create the Keypad TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create Keypad RxQ for sending return value to the SDK API */
	pv.kpdrxq = xQueueCreate( KEYPAD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.kpdrxq)
	{
		LOGERROR("Unable to create the Keypad RxQ \r\n");
		return PS_FAILURE;
	}

	/* Start the Keypad Platform service thread */
	xReturned = xTaskCreate(Keypad_ServiceStart, "Keypad",
			512, NULL, PS_KPD_TASK_PRIORITY, &pv.kpd_task);

	if( xReturned != pdPASS )
	{
		LOGERROR("Unable to create the keypad Task \r\n");
		return PS_FAILURE;
	}

	LOGERROR("keypad Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
