/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : platform_service.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Platform service initialization functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* Include Header files */
#include "../Inc/Common.h"

/* USER CODE BEGIN PV */
pvt_variable pv;
/* USER CODE END PV */

#if ((SDK_SERVICE_LED == PS_ENABLE) ||        \
		(SDK_SERVICE_CFG_INPUT == PS_ENABLE))
/** @brief Create the Mutex
 *
 *  This function create the I2C Mutex
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t create_I2C_mutex(void)
{
	/*	Create mutexes for the I2C	*/
	pv.i2c_mutex = xSemaphoreCreateBinary();

	 if(NULL == pv.i2c_mutex)
	 {
	     /* There was insufficient FreeRTOS heap available
	      * for the semaphore to
	      * be created successfully.
	      */
		 return FAILURE;
	 }

	 /* Give the Semaphore */
	 xSemaphoreGive(pv.i2c_mutex);

	 return SUCCESS;
}

/** @brief Get the Mutex Lock
 *
 *  This function gets the I2C Mutex Lock
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t TakeI2CBusLock()
{
	int8_t ret = SUCCESS;
    if (pdTRUE != xSemaphoreTake(pv.i2c_mutex, portMAX_DELAY))
    {
    	ret = FAILURE;
    }

    return ret;
}


/** @brief Release the Mutex Lock
 *
 *  This function releases the I2C Mutex Lock
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t GiveI2CBusLock()
{
	int8_t ret = SUCCESS;

	/* Get the Semaphore */
    if (pdTRUE != xSemaphoreGive(pv.i2c_mutex))
    {
    	ret = FAILURE;
    }

    return ret;
}
#endif

#if ((SDK_SERVICE_EEPROM == PS_ENABLE) || \
		(SDK_SERVICE_BLE == PS_ENABLE))
/** @brief Create the Mutex
 *
 *  This function create the SPI Mutex
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t create_SPI_mutex(void)
{
	/*	Create mutexes for the SPI	*/
	pv.spi_mutex = xSemaphoreCreateBinary();

	 if(NULL == pv.spi_mutex)
	 {
	     /* There was insufficient FreeRTOS heap available
	      * for the semaphore to
	      * be created successfully.
	      */
		 return FAILURE;
	 }

	 /* Give the Semaphore */
	 xSemaphoreGive(pv.spi_mutex);

	 return SUCCESS;
}

/** @brief Get the Mutex Lock
 *
 *  This function gets the SPI Mutex Lock
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t TakeSPIBusLock()
{
	int8_t ret = SUCCESS;
    if (pdTRUE != xSemaphoreTake(pv.spi_mutex, portMAX_DELAY))
    {
    	ret = FAILURE;
    }

    return ret;
}


/** @brief Release the Mutex Lock
 *
 *  This function releases the SPI Mutex Lock
 *
 *  @param  None
 *
 *  @return 0 : SUCCESS
 *          1 : FAILURE
 */
int8_t GiveSPIBusLock()
{
	int8_t ret = SUCCESS;

	/* Get the Semaphore */
    if (pdTRUE != xSemaphoreGive(pv.spi_mutex))
    {
    	ret = FAILURE;
    }

    return ret;
}
#endif


/**
  * @brief This function handles EXTI line[9:5] interrupts.
  */
void EXTI9_5_IRQHandler(void)
{
#if (SDK_SERVICE_BLE == PS_ENABLE)
	uint32_t pending = EXTI->PR1;

	if(pending & (1 << 6))
	{
		__HAL_GPIO_EXTI_CLEAR_IT(GPIO_PIN_6);
		__HAL_GPIO_EXTI_CLEAR_FLAG(GPIO_PIN_6);
		HAL_GPIO_EXTI_BLE_Callback();
	}
#endif
}


/**
  * @brief This function handles EXTI line[15:10] interrupts.
  */
void EXTI15_10_IRQHandler(void)
{
#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
	uint32_t pending = EXTI->PR1;

	/* CAN1 RX Interrupt */
	if(pending & (1 << 11))
	{
		HAL_GPIO_EXTI_IRQHandler(GPIO_PIN_11);
		__HAL_GPIO_EXTI_CLEAR_IT(GPIO_PIN_11);
		__HAL_GPIO_EXTI_CLEAR_FLAG(GPIO_PIN_11);
	}

	/* Keypad1 Interrupt */
	if(pending & (1 << 12))
	{
		HAL_GPIO_EXTI_IRQHandler(KEY_DATA_PIN_1);
		__HAL_GPIO_EXTI_CLEAR_IT(KEY_DATA_PIN_1);
		__HAL_GPIO_EXTI_CLEAR_FLAG(KEY_DATA_PIN_1);
	}

	/* Keypad4 Interrupt */
	if(pending & (1 << 13))
	{
		HAL_GPIO_EXTI_IRQHandler(KEY_DATA_PIN_4);
		__HAL_GPIO_EXTI_CLEAR_IT(KEY_DATA_PIN_4);
		__HAL_GPIO_EXTI_CLEAR_FLAG(KEY_DATA_PIN_4);
		HAL_GPIO_EXTI_KP_Callback();
	}
#endif
}

#if (SDK_SERVICE_QSPI == PS_ENABLE)
	extern QSPI_HandleTypeDef hqspi;
#endif

/** @brief Initialize all the platform service
 *
 *
 *  @param  None
 *  @return None
 *
 */
void Platform_service_init(void)
{
#if (SDK_SERVICE_QSPI == PS_ENABLE)
	pv.hqspi = &hqspi;
#endif

	/* Configure the Memory Management Module */
	start_memory_management();

#if ((SDK_SERVICE_EEPROM == PS_ENABLE) || \
		(SDK_SERVICE_BLE == PS_ENABLE))

	create_SPI_mutex();
#endif

#if (SDK_SERVICE_FDCAN == PS_ENABLE)
	/* Create FDCAN1 buffer queue */
	can1_rx_memory_management();
#endif	//SDK_SERVICE_FDCAN

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Clear Power Mode Callback function */
    PM_CB_Clear();
#endif

#if ((SDK_SERVICE_LED == PS_ENABLE) ||        \
		(SDK_SERVICE_CFG_INPUT == PS_ENABLE))

	create_I2C_mutex();
#endif

	/* Configure the USER defined configuration */
    Conf_PS_Start();

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
	/* Start EEPROM Platform Service */
	EE_PS_Start();
#endif

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
	/* Start keypad Platform Service */
	Keypad_PS_Start();
#endif

#if (SDK_SERVICE_RTC == PS_ENABLE)
	/* Start RTC Platform Service */
	RTC_PS_Start();
#endif

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)
	/* Start Digital output Platform Service */
	DIO_PS_Start();
#endif

#if (SDK_SERVICE_LED == PS_ENABLE)
	/* Start LED Platform Service */
	LED_PS_Start();
#endif

#if (SDK_SERVICE_LCD == PS_ENABLE)
	/* Start LCD Platform Service */
	LCD_PS_Start();
#endif

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Start Power Mode Platform Service */
	PM_PS_Start();
#endif

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
	/* Start Watchdog Platform Service */
	Watchdog_PS_Start();
#endif

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)
	/* Start Power Monitor Platform Service */
	PowerMonitor_PS_Start();
#endif

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
	/* Start SW Timer Platform Service */
	SWT_PS_Start();
#endif

#if (SDK_SERVICE_BLE == PS_ENABLE)
	/* Start BLE Platform Service */
	BLE_PS_Start();
#endif

#if (SDK_SERVICE_FDCAN == PS_ENABLE)
	/* Start FDCAN Platform Service */
	FDCAN_PS_Start();
#endif

#if (SDK_SERVICE_J1939 == PS_ENABLE)
	/* Start J1939 Platform Service */
	J1939_PS_Start();
#endif

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)
	/* Start ConfigurableInput Platform Service */
	CI_PS_Start();
#endif

#if (THROUGH_PUT_SERVICE == PS_ENABLE)
	Throughput_service_Start();
#endif

#if (USER_TASK_STATE == PS_ENABLE)
	User_Task_Start();
#endif
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
