/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Qspi.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Qspi API.

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0             21/01/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

#if (SDK_SERVICE_QSPI == PS_ENABLE)
extern pvt_variable pv;

//--------------------------------------------------------------------------------------------------------------
/*	This function send a Write Enable and wait it is effective.
	hqspi: QSPI handle	*/
uint8_t QSPI_WriteEnable(QSPI_HandleTypeDef *hqspi)
{
	QSPI_AutoPollingTypeDef s_config;
	QSPI_CommandTypeDef		QSPI_COMMAND_WRITE_ENABLE = {
			// Instruction
			W25Q32JV_WRITE_ENABLE_INSTRUCTION,
			// Address
			0,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			0,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_NONE,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_NONE,
			// NbData
			0,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};

	if(HAL_QSPI_Command(hqspi, &QSPI_COMMAND_WRITE_ENABLE, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK){
		return QSPI_ERROR;
	}

	/*	Configure automatic polling mode to wait for write enabling	*/
	s_config.Match = 2;
	s_config.Mask = 2;
	s_config.MatchMode = QSPI_MATCH_MODE_AND;
	s_config.StatusBytesSize = 1;
	s_config.Interval = 0x10;
	s_config.AutomaticStop = QSPI_AUTOMATIC_STOP_ENABLE;

	QSPI_CommandTypeDef		QSPI_COMMAND_AUTOPOLLING = {
			// Instruction
			W25Q32JV_READ_STATUS_REGISTER1_INSTRUCTION,
			// Address
			0,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			0,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_NONE,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_1_LINE,
			// NbData
			0,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};

	if(HAL_QSPI_AutoPolling(hqspi, &QSPI_COMMAND_AUTOPOLLING, &s_config, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	return QSPI_OK;
}

uint8_t QSPI_ResetMemory(QSPI_HandleTypeDef *hqspi)
{
	QSPI_CommandTypeDef s_command;

	/* Initialize the reset enable command */
	s_command.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
	s_command.Instruction       = RESET_ENABLE_CMD;
	s_command.AddressMode       = QSPI_ADDRESS_NONE;
	s_command.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
	s_command.DataMode          = QSPI_DATA_NONE;
	s_command.DummyCycles       = 0;
	s_command.DdrMode           = QSPI_DDR_MODE_DISABLE;
	s_command.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
	s_command.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

	if(HAL_QSPI_Command(hqspi, &/*QSPI_COMMAND_RESET_ENABLE*/s_command, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	/* Send the reset memory command */
	s_command.Instruction = RESET_MEMORY_CMD;

	if(HAL_QSPI_Command(hqspi, &/*QSPI_COMMAND_RESET*/s_command, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	/*	polling mode to wait the memory is ready	*/
	if(QSPI_AutoPollingMemReady(hqspi, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != QSPI_OK)
	{
		return QSPI_ERROR;
	}

	return QSPI_OK;
}

//--------------------------------------------------------------------------------------------------------------
/*	This function read the SR of the memory and wait the EOP	*/
uint8_t QSPI_AutoPollingMemReady(QSPI_HandleTypeDef *hqspi, uint32_t Timeout)
{
	QSPI_AutoPollingTypeDef s_config;
	QSPI_CommandTypeDef		QSPI_COMMAND_AUTOPOLLING = {
			// Instruction
			W25Q32JV_READ_STATUS_REGISTER1_INSTRUCTION,
			// Address
			0,
			// AlternateBytes
			0,
			// AddressSize
			QSPI_ADDRESS_24_BITS,
			// AlternateBytesSize
			0,
			// DummyCycles
			0,
			// InstructionMode
			QSPI_INSTRUCTION_1_LINE,
			// AddressMode
			QSPI_ADDRESS_NONE,
			// AlternateByteMode
			QSPI_ALTERNATE_BYTES_NONE,
			// DataMode
			QSPI_DATA_1_LINE,
			// NbData
			0,
			// DdrMode
			QSPI_DDR_MODE_DISABLE,
			// DdrHoldHalfCycle
			QSPI_DDR_HHC_ANALOG_DELAY,
			// SIOOMode
			QSPI_SIOO_INST_EVERY_CMD,
	};

	s_config.Match = 0;
	s_config.Mask = 1;
	s_config.MatchMode = QSPI_MATCH_MODE_AND;
	s_config.StatusBytesSize = 1;
	s_config.Interval = 0x10;
	s_config.AutomaticStop = QSPI_AUTOMATIC_STOP_ENABLE;

	if(HAL_QSPI_AutoPolling(hqspi, &QSPI_COMMAND_AUTOPOLLING, &s_config, Timeout) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	return QSPI_OK;
}

//--------------------------------------------------------------------------------------------------------------
QSPI_CommandTypeDef		QSPI_COMMAND_READ = {
	// Instruction
	W25Q32JV_FAST_READ_QUAD_I_O_INSTRUCTION,
	// Address
	0,
	// AlternateBytes
	0,
	// AddressSize
	QSPI_ADDRESS_24_BITS,
	// AlternateBytesSize
	0,
	// DummyCycles
	6,
	// InstructionMode
	QSPI_INSTRUCTION_1_LINE,
	// AddressMode
	QSPI_ADDRESS_4_LINES,
	// AlternateByteMode
	QSPI_ALTERNATE_BYTES_NONE,
	// DataMode
	QSPI_DATA_4_LINES,
	// NbData
	0,
	// DdrMode
	QSPI_DDR_MODE_DISABLE,
	// DdrHoldHalfCycle
	QSPI_DDR_HHC_ANALOG_DELAY,
	// SIOOMode
	QSPI_SIOO_INST_EVERY_CMD,
};

//--------------------------------------------------------------------------------------------------------------
uint8_t QSPI_selfTest(QSPI_HandleTypeDef *hqspi, unsigned long int size, unsigned long int location)
{
	unsigned long int  index = 0;
	unsigned long int ret = 0;
	unsigned char TxBuffer[100];//Replace SIZE with size for debug
	unsigned char RxBuffer[100];//Replace SIZE with size for debug

	for (index = (location/256); index <= (size/256); index+=256 )
	{
		if( QSPI_Erase_Block(index) )
		{
			return(1);
		}
	}

	/* Put values	*/
	for (index = 0; index < size; index++ )
	{
		TxBuffer[index] = index;
		RxBuffer[index] = 0;
	}

	if( QSPI_Read(&RxBuffer[0], location, size) )
	{
		return(3);
	}

	if( QSPI_Write(&TxBuffer[0], location, size) )
	{
		return(2);
	}

	if( QSPI_Read(&RxBuffer[0], location, size) )
	{
		return(3);
	}

	for (index = 0; index < size; index++)
	{
		if (RxBuffer[index] != TxBuffer[index])
		{
			ret++;
		}
	}
	return(ret);
}

//--------------------------------------------------------------------------------------------------------------
//	Configure the QSPI in memory-mapped mode
//	QSPI memory status
uint8_t QSPI_EnableMemoryMappedMode(QSPI_HandleTypeDef *hqspi)
{
	QSPI_MemoryMappedTypeDef s_mem_mapped_cfg;

	//	Configure the memory mapped mode
	s_mem_mapped_cfg.TimeOutActivation = QSPI_TIMEOUT_COUNTER_DISABLE;
	s_mem_mapped_cfg.TimeOutPeriod = 0;

	if(HAL_QSPI_MemoryMapped(hqspi, &QSPI_COMMAND_READ, &s_mem_mapped_cfg) != HAL_OK)
	{
		return QSPI_ERROR;
	}

	return QSPI_OK;
}

void QSPI_QuadEnableCfg(QSPI_HandleTypeDef *hqspi)
{
	QSPI_CommandTypeDef sCommand;
	uint8_t ucWriteData;
	uint8_t ucRegister2;

	//QSPI_AutoPollingTypeDef sConfig;
	/* Read Volatile Configuration register --------------------------- */

	sCommand.InstructionMode  = QSPI_INSTRUCTION_1_LINE;
	sCommand.Instruction      = 0x35/* Status Register-2 Read */;
	sCommand.AddressMode      = QSPI_ADDRESS_NONE;
	sCommand.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
	sCommand.DataMode         = QSPI_DATA_1_LINE;
	sCommand.DummyCycles      = 0;
	sCommand.DdrMode          = QSPI_DDR_MODE_DISABLE;
	sCommand.DdrHoldHalfCycle = QSPI_DDR_HHC_ANALOG_DELAY;
	sCommand.SIOOMode         = QSPI_SIOO_INST_EVERY_CMD;
	sCommand.NbData           = 1;

	/* Read Volatile Configuration register --------------------------- */
	if (HAL_QSPI_Command(hqspi, &sCommand, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		Error_Handler();
	}

	if (HAL_QSPI_Receive(hqspi, &ucRegister2, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
	{
		Error_Handler();
	}

	/* Enable write operations */
	if (!(ucRegister2 & 0x02))
	{
		QSPI_WriteEnable(hqspi);
		sCommand.Instruction = 0x31;
		ucWriteData = ucRegister2 | 0x02;// QUAD ENABLE

		if (HAL_QSPI_Command(hqspi, &sCommand, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
		{
			Error_Handler();
		}

		if (HAL_QSPI_Transmit(hqspi, &ucWriteData, HAL_QPSI_TIMEOUT_DEFAULT_VALUE) != HAL_OK)
		{
			Error_Handler();
		}
	}
}

#endif
/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
