/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : TerminalCommand.c
   Author             :
   Version            : V0.0.0.1
   Origination Date   : 14/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Terminal commands process functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_BLE == PS_ENABLE)
//#include "stm32h7xx_hal_pcd.h"


extern pvt_variable pv;


/** @brief Checksum
 *
 *  This function calculate the checksum.
 *
 *  @param  BufferToCal - data buffer
 *          len         - data buffer length
 *
 *  @return none
 *
 */
int8_t CalCheckSum(uint8_t *BufferToCal, uint8_t len)
{
	uint8_t  bindex = 0;
    uint32_t isum = 0;
    uint8_t  bmodule = 0;
    for (; bindex < len; bindex++)
    {
        isum += BufferToCal[bindex];
    }

	bmodule = (uint8_t)(isum % 256);
    return bmodule;
}

/** @brief start BLE communication
 *
 *  This function starts the BLE communication between
 *  the device and PC. And it sends the response to the
 *  PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return SUCCESS
 *          FAILURE
 *
 */

int8_t SetstartCOM(uint8_t *rxpkt, uint8_t *rxlen)
{
	uint16_t version;
	int8_t   res = FAILURE;

    /* Validate the packet Size */
	if ((*rxlen != START_REQ_PKT_SIZE) ||
	     (rxpkt[3] != START_REQ_PKT_DATA_SIZE))
	{
		rxpkt[4] = (NEGATIVE_COM_RESP);
	}
	else
	{
		rxpkt[4] = (rxpkt[4] + POSITIVE_COM_RESP);
		res = SUCCESS;
		pv.start_com = TRUE;
	}

	/* Copy the Version number */
	version = COMM_VERSION;
	memcpy(&rxpkt[5], &version, sizeof(version));

	/* update the length */
	rxpkt[3] = START_RESP_PKT_SIZE;

	/* Update the checksum */
	rxpkt[7] = CalCheckSum(rxpkt, START_RESP_TOTAL_PKT_SIZE - 1);

	*rxlen = START_RESP_TOTAL_PKT_SIZE;

	return res;
}

/** @brief stop BLE communication
 *
 *  This function stops the BLE communication between
 *  the device and PC. And it sends the response to the
 *  PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return SUCCESS
 *          FAILURE
 *
 *
 */
int8_t SetstopCOM(uint8_t *rxpkt, uint8_t *rxlen)
{
	int8_t   res = FAILURE;

    /* Validate the packet Size */
	if ((*rxlen != STOP_REQ_PKT_SIZE) ||
	     (rxpkt[3] != STOP_REQ_PKT_DATA_SIZE))
	{
		rxpkt[4] = NEGATIVE_COM_RESP;
	}
	else
	{
		rxpkt[4] = (rxpkt[4] + POSITIVE_COM_RESP);
		res = SUCCESS;
	}

	/* update the length */
	rxpkt[3] = STOP_RESP_PKT_SIZE;

	/* Update the checksum */
	rxpkt[5] = CalCheckSum(rxpkt, 5);

	*rxlen = STOP_RESP_TOTAL_PKT_SIZE;

	return res;
}

/** @brief ECU Reset
 *
 *  This function resets the ECU. And it sends the response to the
 *  PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return none
 *
 */
void SetecuReset(uint8_t *rxpkt, uint8_t *rxlen)
{
	//uint8_t  val;

    /* Validate the packet Size */
	if ((*rxlen != ECURESET_REQ_PKT_SIZE) ||
	     (rxpkt[3] != ECURESET_REQ_PKT_DATA_SIZE))
	{
		rxpkt[4] = NEGATIVE_COM_RESP;
	}
	else
		rxpkt[4] = (rxpkt[4] + POSITIVE_COM_RESP);

    /* Need more information from the Maximatech */
    /* For processing this command */
	NVIC_SystemReset();


	/* update the length */
	rxpkt[3] = ECURESET_RESP_PKT_SIZE;

	/* Update the checksum */
	rxpkt[5] = CalCheckSum(rxpkt, 5);

	*rxlen = ECURESET_RESP_TOTAL_PKT_SIZE;
}

/** @brief readEcuIdentification
 *
 *  This function reads the ECU identification. And it sends the response to the
 *  PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return none
 *
 */
void GetreadEcuIdentification(uint8_t *rxpkt, uint8_t *rxlen)
{
    uint8_t  pktsize = CMD_PKT_HEADER_SIZE;

	rxpkt[pktsize] = (rxpkt[pktsize] + POSITIVE_COM_RESP);
	pktsize++;

	switch(rxpkt[pktsize])
	{
		case VEHICLE_MANU_ECU_SW_NUM:
			break;
		case VEHICLE_MANU_ECU_SW_VER:
			break;
		case ECU_MANU_DATE:
			break;
		case ECU_SERIAL_NUM:
			break;
		case PROGRAMMING_DATE:
			break;
		default:
			rxpkt[4] = NEGATIVE_COM_RESP;
			rxpkt[5] = readECUIdentification;
			pktsize++;
			break;
	}

	/* update the length */
	rxpkt[3] = pktsize;

	/* Update the checksum */
	rxpkt[pktsize] = CalCheckSum(rxpkt, pktsize);

	*rxlen = pktsize;

#if 0
	//uint8_t  val;
    uint8_t  pktsize = CMD_PKT_HEADER_SIZE;


	pktsize++;
	rxpkt[4] = (rxpkt[3] + POSITIVE_COM_RESP);


	switch(rxpkt[4])
	{
		case VEHICLE_MANU_ECU_SW_NUM:
			break;
		case VEHICLE_MANU_ECU_SW_VER:
			break;
		case ECU_MANU_DATE:
			break;
		case ECU_SERIAL_NUM:
			break;
		case PROGRAMMING_DATE:
			break;
		default:
			rxpkt[4] = NEGATIVE_COM_RESP;
			rxpkt[5] = readECUIdentification;
			pktsize++;
			break;
	}

	/* update the length */
	rxpkt[3] = pktsize;

	/* Update the checksum */
	rxpkt[5] = CalCheckSum(rxpkt, pktsize);

	*rxlen = pktsize;
#endif
}

/** @brief readDataByCommonIdentifier
 *
 *  This function reads the data common identifer.
 *  And it sends the response to the PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return none
 *
 */
extern uint8_t Get_data_type(uint16_t dl_index);

void GetreadDataByCommonIdentifier(uint8_t *rxpkt, uint8_t *rxlen)
{
	uint16_t dl_index;
	uint8_t data_type = NOT_SUPPRTED;
    uint8_t  pktsize = CMD_PKT_HEADER_SIZE;
	uint8_t  size = 0;
	uint8_t  buf[DB_MAX_VAL_SIZE];

	rxpkt[pktsize] = (rxpkt[pktsize] + POSITIVE_COM_RESP);

    /* Get the record common Identifier */
	dl_index = (rxpkt[5] << 8) | (rxpkt[6]);

    Get_DL(dl_index, &buf[0]);

    data_type = Get_data_type(dl_index);

	switch(data_type)
	{
	case DLU8:
	case DL_JU8:
		memcpy(&rxpkt[7], &buf[0], sizeof(uint8_t));
		size = 4;
		pktsize = CMD_PKT_HEADER_SIZE + size;
		break;

	case DLU16:
	case DL_JU16:
		memcpy(&rxpkt[7], &buf[0], sizeof(uint16_t));
		size = 5;
		pktsize = CMD_PKT_HEADER_SIZE + size;
		break;

	case DLU32:
	case DL_JU32:
		memcpy(&rxpkt[7], &buf[0], sizeof(uint32_t));
		size = 7;
		pktsize = CMD_PKT_HEADER_SIZE + size;
		break;

	case DL_EEPROM:
		memcpy(&rxpkt[7], &buf[0], sizeof(uint32_t));
		size = 7;
		pktsize = CMD_PKT_HEADER_SIZE + size;
		break;
	case DL_BUF512BIT:
	case NOT_SUPPRTED:
	    rxpkt[pktsize++] = NEGATIVE_COM_RESP;
	    rxpkt[pktsize] = readDataByCommonIdentifier;
	    size = 1;
	    pktsize = CMD_PKT_HEADER_SIZE  + 1;
		break;
	}

	/* update the length */
	rxpkt[3] = size;

	/* Update the checksum */
	rxpkt[pktsize] = CalCheckSum(rxpkt, pktsize);

	*rxlen = pktsize+1;
}


/** @brief readMemoryByAddress
 *
 *  This function reads the data from the specified memory.
 *  And it sends the data/response to the PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return none
 *
 */
void GetreadMemoryByAddress(uint8_t *rxpkt, uint8_t *rxlen)
{
	uint32_t  val;
	uint8_t   size;
	uint8_t   checksum;
    uint8_t   pktsize = CMD_PKT_HEADER_SIZE;

	rxpkt[pktsize] = (rxpkt[pktsize] + POSITIVE_COM_RESP);
	pktsize ++;
    /* Get the Memory Address */
    val = (rxpkt[5] << 24) | (rxpkt[6] << 16) | (rxpkt[7] << 8) | (rxpkt[8]);

	/* Get the Size */
    size = rxpkt[9];

    /* Check whether the address is valid */
    if (((FLASH_START_ADDR   <= val) && (FLASH_END_ADDR   > val)) ||
       ((DRCMRAM_START_ADDR <= val) && (DRCMRAM_END_ADDR > val)) ||
       ((RAMD1_START_ADDR   <= val) && (RAMD1_END_ADDR   > val)) ||
       ((RAMD2_START_ADDR   <= val) && (RAMD2_END_ADDR   > val)) ||
       ((RAMD3_START_ADDR   <= val) && (RAMD3_END_ADDR   > val)) ||
       ((ITCMRAM_START_ADDR <= val) && (ITCMRAM_END_ADDR > val)) ||
       ((SDRAM_START_ADDR   <= val) && (SDRAM_END_ADDR   > val)))
	{
		/* Copy the data to the buffer */
		memcpy(&rxpkt[pktsize], (uint8_t *)val, size);

		pktsize += size;
	}
	else
	{
		rxpkt[4] = NEGATIVE_COM_RESP;
		rxpkt[pktsize++] = readMemoryByAddress;
	}

	/* update the length */
	rxpkt[3] = pktsize - CMD_PKT_HEADER_SIZE;

	/* Update the checksum */
	checksum = CalCheckSum(rxpkt, pktsize);
	rxpkt[pktsize++] = checksum;

	*rxlen = pktsize;
}

/** @brief writeDataByCommonIdentifier
 *
 *  This function writes value into the data common identifier.
 *  And it sends the response to the PC.
 *
 *  @param  rxpkt - received packet from the PC
 *          rxlen - received packet length
 *
 *  @return none
 *
 */

void SetwriteDataByCommonIdentifier(uint8_t *rxpkt, uint8_t *rxlen)
{
	uint16_t dl_index;
	uint8_t  data_type = NOT_SUPPRTED;
    uint8_t  pktsize = CMD_PKT_HEADER_SIZE;
	uint8_t  val = 0;
	uint8_t  buf[DB_MAX_VAL_SIZE];

	rxpkt[pktsize] = (rxpkt[pktsize] + POSITIVE_COM_RESP);
	pktsize++;

    /* Get the record common Identifier */
	dl_index = (rxpkt[5] << 8) | (rxpkt[6]);

    data_type = Get_data_type(dl_index);

    #define DLU8_PKT_SIZE 0x04

	switch(data_type)
	{
	case DLU8:
		if (DLU8_PKT_SIZE != rxpkt[3])
			break;

		val = 1;
		memcpy(&buf[0], &rxpkt[7], val);
		break;

	case DLU16:
		if (DLU8_PKT_SIZE != rxpkt[3])
			break;

		val = 2;
		memcpy(&buf[0], &rxpkt[7], val);
		break;

	case DLU32:
		if (DLU8_PKT_SIZE != rxpkt[3])
			break;

		val = 3;
		memcpy(&buf[0], &rxpkt[7], val);
		break;

	case DL_EEPROM:
	case DL_BUF512BIT:
	case NOT_SUPPRTED:
	    rxpkt[pktsize++] = NEGATIVE_COM_RESP;
	    rxpkt[pktsize] = writeDataByCommonIdentifier;
	    val = 0;
	    pktsize = CMD_PKT_HEADER_SIZE  + 1;
		break;
	}

	if (val)
	{
	    val = Set_DL(dl_index, &buf[0]);

	    if(FAILURE == val)
	    {
		    rxpkt[pktsize++] = NEGATIVE_COM_RESP;
		    rxpkt[pktsize] = writeDataByCommonIdentifier;
		    val = 0;
		    pktsize = CMD_PKT_HEADER_SIZE  + 1;
	    }
	}


	/* update the length */
	rxpkt[3] = pktsize - CMD_PKT_HEADER_SIZE;

	pktsize +=2;
	/* Update the checksum */
	rxpkt[pktsize] = CalCheckSum(rxpkt, pktsize);

	pktsize++;

	*rxlen = pktsize;
}

#endif	//SDK_SERVICE_BLE

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
