/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : AI280_Config.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the AI280 board configuration macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1              21/01/2022          Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

#ifndef _AI280_CONFIG_H_
#define	_AI280_CONFIG_H_

/* ########################## Module Selection ############################## */
/**
  * @brief USER has to change only these for enabling and disabling
  *        Platform services software components. And also the USER can
  *        configure the Platform service driver configuration.
  */

/*****************************************************************************
 *
 *                      Keypad Module Configuration
 *
 *****************************************************************************/
/*!
 * Keypad Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_KEYPAD           			 PS_ENABLE

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)

/*!
 * Keypad Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_KPD_TASK_PRIORITY   					 osPriorityIdle
/*!
 * MACRO Supported
 *
 * Keypad backlight configuration state
 * 1: KEY_BACKLIGHT_ON /
 * 0: KEY_BACKLIGHT_OFF
 */
#define KEYPAD_BACKLIGHT_CFG_STATE				 KEY_BACKLIGHT_OFF
/*!
 * SHORT Press timeout (Millisecond)
 */
#define SHORT_PRESS_TIMEOUT         			 10
/*!
 * MACOR Supported
 *
 * Keypad configuration state
 * 1: PS_ENABLE /
 * 0: PS_DISABLE
 */
#define CONF_KEYPAD_01_STATE            		 PS_ENABLE
#define CONF_KEYPAD_02_STATE            		 PS_ENABLE
#define CONF_KEYPAD_03_STATE            		 PS_ENABLE
#define CONF_KEYPAD_04_STATE            		 PS_ENABLE

#endif	//SDK_SERVICE_KEYPAD

/*****************************************************************************
 *
 *                      RTC Module Configuration
 *
 *****************************************************************************/

/*!
 * RTC Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_RTC           				 PS_ENABLE

#if (SDK_SERVICE_RTC == PS_ENABLE)

/*!
 * RTC Task Periodicity	100ms
 */
#define PS_RTC_TASK_TIMEOUT         			 100

/*!
 * RTC Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_RTC_TASK_PRIORITY   					 osPriorityIdle

#endif	//SDK_SERVICE_RTC


/*****************************************************************************
 *
 *                      Digital Output Module Configuration
 *
 *****************************************************************************/

/*!
 * DIO Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_DIGITAL_OUTPUT           	 PS_ENABLE

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)

/*!
 * DIO Task Periodicity	100ms
 */
#define PS_DIO_TASK_TIMEOUT         			 100

/*!
 * DIO Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_DIO_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Select the DIGITAL_OUTPUT_CFG 00 : CONF_LOW_SIDE_DRIVER
 *                               01 : CONF_HIGH_SIDE_DRIVER
 *                               02 : CONF_OPEN_DRIVE_DRIVER
 */
#define CF_DIGITAL_OUTPUT_01_CFG                       CONF_OPEN_DRIVE_DRIVER
#define CF_DIGITAL_OUTPUT_02_CFG                       CONF_LOW_SIDE_DRIVER
#define CF_DIGITAL_OUTPUT_03_CFG                       CONF_HIGH_SIDE_DRIVER
#define CF_DIGITAL_OUTPUT_04_CFG                       CONF_HIGH_SIDE_DRIVER

#endif	//SDK_SERVICE_DIGITAL_OUTPUT

/*****************************************************************************
 *
 *                      LED Configuration
 *
 *****************************************************************************/

/*!
 * LED Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_LED      			 		 PS_ENABLE

#if (SDK_SERVICE_LED == PS_ENABLE)

/*!
 * LED Task Periodicity		100ms
 */
#define PS_LED_TASK_TIMEOUT        				 100

/*!
 * LED Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_LED_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Maximum Number of LED required for this application
 *
 * MACOR Supported
 *
 * MAX_LED_NUM  : This hardware support maximum of 2 LED's
 *
 * CONF_xx_LED_PS_STATE			            PS_ENABLE
 *                                          PS_DISABLE
 *
 * CONF_xx_LED_STATE      					LED_CONF_OFF
 *                                          LED_CONF_ON
 *
 * CONF_xx_LED_BLINKING_MS        			<0-65535>
 *
 */
#define MAX_LED_NUM                    			 2

#define CONF_RED_LED_PS_STATE		             PS_ENABLE
#define CONF_RED_LED_STATE             			 LED_CONF_OFF
#define CONF_RED_LED_BLINKING_MS       			 1

#define CONF_AMB_LED_PS_STATE		             PS_ENABLE
#define CONF_AMB_LED_STATE             			 LED_CONF_OFF
#define CONF_AMB_LED_BLINKING_MS       			 1

#endif	//SDK_SERVICE_LED

/*****************************************************************************
 *
 *                      Configurable Input Module Configuration
 *
 *****************************************************************************/

/*!
 * Config input Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_CFG_INPUT      				 PS_ENABLE

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

/*!
 * Config_input Task Periodicity		100ms
 */
#define PS_CFG_INPUT_TASK_TIMEOUT        		 100

/*!
 * CFG Input Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_CFG_INPUT_TASK_PRIORITY   			 osPriorityIdle

#define CONF_INPUT_TYPE_01       				 CI_INPUT_VOLTAGE_LOW_6V
#define CONF_INPUT_TYPE_02       				 CI_INPUT_VOLTAGE_LOW_6V
#define CONF_INPUT_TYPE_03       				 CI_INPUT_VOLTAGE_LOW_6V
#define CONF_INPUT_TYPE_04       				 CI_INPUT_VOLTAGE_LOW_6V
#define CONF_INPUT_TYPE_05       				 CI_INPUT_VOLTAGE_LOW_6V

#define CONF_INPUT_01_NUMB_SAMPLES   			 1
#define CONF_INPUT_02_NUMB_SAMPLES   			 1
#define CONF_INPUT_03_NUMB_SAMPLES   			 1
#define CONF_INPUT_04_NUMB_SAMPLES   			 1
#define CONF_INPUT_05_NUMB_SAMPLES   			 1

#endif	//SDK_SERVICE_CFG_INPUT

/*****************************************************************************
 *
 *                      LCD Module Configuration
 *
 *****************************************************************************/

/*!
 * LCD Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_LCD           			 	 PS_ENABLE

#if (SDK_SERVICE_LCD == PS_ENABLE)

/*!
 * LCD Task Periodicity	100ms
 */
#define PS_LCD_TASK_TIMEOUT					 	 100

/*!
 * LCD Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_LCD_TASK_PRIORITY   					 osPriorityIdle

/*!
 * MACOR Supported
 *
 * CONF_LCD_STATE		         LCD_CONF_ON/
 * 								 LCD_CONF_OFF
 *
 * CONF_LCD_BRIGHTNESS           <0 - 100>
 *
 */
#define CONF_LCD_STATE             				 LCD_CONF_ON
#define CONF_LCD_BRIGHTNESS             		 30

#endif	//SDK_SERVICE_LCD

/*****************************************************************************
 *
 *                      Power Management Module Configuration
 *
 *****************************************************************************/

/*!
 * Power Management Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_PM                         	 PS_ENABLE

#if (SDK_SERVICE_PM == PS_ENABLE)

/*!
 * Power Management Task Periodicity	100ms
 */
#define PS_PM_TASK_TIMEOUT                     	 100

/*!
 * PM Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_PM_TASK_PRIORITY   					 osPriorityIdle

/*!
 * GPIO Wake up Source
 */
#define KEYPAD01_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD02_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD03_WAKEUP_SOURCE                 	 PS_ENABLE
#define KEYPAD04_WAKEUP_SOURCE                 	 PS_ENABLE

/*!
 * RTC Wake up Source
 */
#define RTC_WAKEUP_SOURCE_STATE                	 PS_DISABLE
#define RTC_WAKEUP_SOURCE_TIMEOUT	           	 10000

/*!
 * IGN Wake up Source
 */
#define IGN_WAKEUP_SOURCE					   	 PS_ENABLE

/*!
 * CAN Wake up Source
 */
#define CAN_WAKEUP_SOURCE					   	 PS_ENABLE

#endif //SDK_SERVICE_PM

/*****************************************************************************
 *
 *                      Watchdog Module Configuration
 *
 *****************************************************************************/

/*!
 * Watchdog Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_WATCHDOG      			 	 PS_DISABLE

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)

/*!
 * Watchdog Task Periodicity		100ms
 */
#define PS_WD_TASK_TIMEOUT        				 100

/*!
 * Watchdog Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_WD_TASK_PRIORITY   					 osPriorityIdle

/*!
 * Watchdog Reset timer
 */
#define PS_WD_RESET_TIMER           			 800

/*
 * IWDG_PRESCALER_4
 * IWDG_PRESCALER_8
 * IWDG_PRESCALER_16
 * IWDG_PRESCALER_32
 * IWDG_PRESCALER_64
 * IWDG_PRESCALER_128
 * IWDG_PRESCALER_256
 */
#define WATCHDOG_FEED_TIME					 	 IWDG_PRESCALER_256
/*!
 * Watchdog external task ping_id
 *
 * MAX Supported USER Watchdog is 10
 */
#define USER_TASK_WD0                            PS_ENABLE
#define USER_TASK_WD1                            PS_ENABLE
#define USER_TASK_WD2                            PS_ENABLE
#define USER_TASK_WD3                            PS_ENABLE
#define USER_TASK_WD4                            PS_ENABLE
#define USER_TASK_WD5                            PS_ENABLE
#define USER_TASK_WD6                            PS_ENABLE
#define USER_TASK_WD7                            PS_ENABLE
#define USER_TASK_WD8                            PS_ENABLE
#define USER_TASK_WD9                            PS_ENABLE


#endif	//SDK_SERVICE_WATCHDOG

/*****************************************************************************
 *
 *                      Power Monitor Module Configuration
 *
 *****************************************************************************/

/*!
 * Power Monitor Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_POWER_MONITOR      			 PS_ENABLE

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

#if ((SDK_SERVICE_POWER_MONITOR == PS_ENABLE) && (SDK_SERVICE_CFG_INPUT == PS_DISABLE))
#undef SDK_SERVICE_POWER_MONITOR
#define SDK_SERVICE_POWER_MONITOR				PS_DISABLE
#endif

/*!
 * Power Monitor Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_POWER_MONITOR_TASK_PRIORITY   		 osPriorityIdle

/*!
 * Power Monitor Task Periodicity	100ms
 */
#define PS_POWER_MONITOR_TASK_TIMEOUT            100


#endif	//SDK_SERVICE_POWER_MONITOR

/*****************************************************************************
 *
 *                      EEPROM Module Configuration
 *
 *****************************************************************************/

/*!
 * EEPROM Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_EEPROM      			 	 PS_ENABLE

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
/*!
 * EEPROM Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_EE_TASK_PRIORITY   					 osPriorityIdle

/*!
 * EEPROM Task Periodicity	100ms
 */
#define PS_EE_TASK_TIMEOUT					 	 100

/*!
 * EEPROM Place holder size
 */
#define SIZE_OF_PLACEHOLDER                      100
#endif	//SDK_SERVICE_EEPROM

/*****************************************************************************
 *
 *                      SWTIMER Module Configuration
 *
 *****************************************************************************/

/*!
 * SWTIMER Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_SWTIMER      			 	 PS_ENABLE

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
/*!
 * SWTIMER Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_SWT_TASK_PRIORITY   					 osPriorityIdle

/*!
 * SWTimer Task Periodicity	100ms
 */
#define PS_SWT_TASK_TIMEOUT					 	 100

#endif	//SDK_SERVICE_SWTIMER

/*****************************************************************************
 *
 *                      BLE Module Configuration
 *
 *****************************************************************************/

/*!
 * BLE Platform service Enable(PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_BLE      			 	 	 PS_ENABLE

#if (SDK_SERVICE_BLE == PS_ENABLE)
/*!
 * BLE Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_BLE_TASK_PRIORITY   			 		 osPriorityBelowNormal7

/*!
 * BLE Task Periodicity	100ms
 */
#define PS_BLE_TASK_TIMEOUT				 		 100

/*!
 * BLE Device Name
 */
#define BLE_DEVICE_NAME 						 "Max12345678"

#define VEHICLE_MANU_ECU_SW_NUM			 		 0x88
#define VEHICLE_MANU_ECU_SW_VER          		 0x89
#define ECU_MANU_DATE                    		 0x8B
#define ECU_SERIAL_NUM                   		 0x8C
#define PROGRAMMING_DATE                 		 0x99
#define PS_USB_USER_PACKET                       0x50

#endif	//SDK_SERVICE_BLE

/*****************************************************************************
 *
 *                      QSPI Module
 *
 *****************************************************************************/

/*!
 * QSPI SDK API (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_QSPI      			 		 PS_ENABLE


/*****************************************************************************
 *
 *                      FDCAN Module
 *
 *****************************************************************************/

/*!
 * FDCAN Platform service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_FDCAN      			 		 PS_ENABLE

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

/*!
 * FDCAN Task Periodicity		100ms
 */
#define PS_FDCAN_TASK_TIMEOUT        			 100

/*!
 * FDCAN Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_FDCAN_TASK_PRIORITY   				 osPriorityIdle

/*
 * BAUDRATE_50K
 * BAUDRATE_100K
 * BAUDRATE_125K
 * BAUDRATE_250K
 * BAUDRATE_500K
 * BAUDRATE_1000K
 */
#define FDCAN1_BAUDRATE							BAUDRATE_50K

#define FDCAN1_IDENTIFIER						0x19FEFCFE //0x111
#define FDCAN1_IDTYPE							FDCAN_EXTENDED_ID
#define FDCAN1_TXFRAMETYPE						FDCAN_DATA_FRAME
#define FDCAN1_DATALENGTH						FDCAN_DLC_BYTES_8
#define FDCAN1_ERRORSTATEIND					FDCAN_ESI_ACTIVE
#define FDCAN1_BITRATESWITCH					FDCAN_BRS_ON
#define FDCAN1_FDFORMATE						FDCAN_FD_CAN
#define FDCAN1_TXEVENTFIFOCONTROL				FDCAN_STORE_TX_EVENTS
#define FDCAN1_MESSAGEMARKER					0

#define FDCAN1_ID								FDCAN_STANDARD_ID
#define FDCAN1_FILTERINDEX						0
#define FDCAN1_FILTERTYPE						FDCAN_FILTER_DUAL
#define FDCAN1_FILTERCONFIG						FDCAN_FILTER_TO_RXBUFFER
#define FDCAN1_FILTERID1						FDCAN_DEAFULT_FILTERID1
#define FDCAN1_FILTERID2						FDCAN_DEAFULT_FILTERID2
#define FDCAN1_RXBUFFERIND						1

/* CAN1 Receive Task Delay */
#define FDCAN1_RECEIVE_TASK_DELAY				100

#endif	//SDK_SERVICE_FDCAN

/*****************************************************************************
 *
 *                      J1939 Module
 *
 *****************************************************************************/

/*!
 * J1939 Platform service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define SDK_SERVICE_J1939      			 		 PS_ENABLE

#if ((SDK_SERVICE_J1939 == PS_ENABLE) && (SDK_SERVICE_FDCAN == PS_ENABLE))
#undef SDK_SERVICE_FDCAN
#define SDK_SERVICE_FDCAN      			 		 PS_DISABLE
#endif

#if (SDK_SERVICE_J1939 == PS_ENABLE)

/*!
 * J1939 Task Periodicity		100ms
 */
#define PS_J1939_TASK_TIMEOUT        			 100

/*!
 * J1939 Task Priority
 * osPriorityNone          =  0,
 * osPriorityIdle          =  1,
 * osPriorityLow           =  8,
 * osPriorityLow1          =  8+1,
 * 		,,					  ,,
 * 		,,					  ,,
 * osPriorityISR           = 56,
 * osPriorityError         = -1,
 * osPriorityReserved      = 0x7FFFFFFF
 */
#define PS_J1939_TASK_PRIORITY   				 osPriorityIdle

/*!
 * J1939_CLAIM_ADDRESS
 */
#define J1939_CLAIM_ADDRESS                      PS_ENABLE

/*
 * State BLE J1939 Debug data
 */
#define BLE_J1939_DEBUG_DATA					 PS_ENABLE

/*
 * CAN1 State
 */
#define J1939_CAN1_ENABLE                        PS_ENABLE

/*!
 * J1939_CLAIM_ADDRESS will be between 0 to 255
 */
#define J1939_CAN1_CLAIM_ADDRESS                 23

/*!
 * J1939_ADDRESS_CLAIM_DYNAMIC will search address when set to 1, Fixed = 0
 */
#define J1939_CAN1_ADDRESS_CLAIM_DYNAMIC	     1

/*!
 * J1939_ADDRESS_CLAIM_NEXT_ADDRESS When dynamic set to 1, search will starting from this value and up
 */
#define J1939_CAN1_ADDRESS_CLAIM_NEXT_ADDRESS	 80

/*!
 * J1939_CAN1_BITRATE will be between 0/250/500/1000
 */
#define J1939_CAN1_BITRATE                       250

/*!
 * J1939 ENABLE/Disable DMx
 */
#define EMTOS_J1939_DM1							 PS_ENABLE
#define EMTOS_J1939_DM2							 PS_ENABLE
#define EMTOS_J1939_DM3							 PS_ENABLE

/*!
 * J1939 Set number of PGN'S to receive and transmit
 */
#define CAN1_NUMBER_PGNS_RX                      18u
#define CAN1_NUMBER_PGNS_TX                      1u

/*!
 * J1939 Set COB handlers
 */
#define CAN1_CO_COB_COUNTS                       36u
#endif	//SDK_SERVICE_J1939

/*****************************************************************************
 *
 *                      Through Put Service
 *
 *****************************************************************************/

/*!
 * Through Put service (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define THROUGH_PUT_SERVICE							PS_ENABLE

#if (THROUGH_PUT_SERVICE == PS_ENABLE)

#endif	//THROUGH_PUT_SERVICE


/*****************************************************************************
 *
 *                      		USER Task
 *
 *****************************************************************************/

/*!
 * User Task (PS_ENABLE) / Disable(PS_DISABLE) Macros
 */
#define USER_TASK_STATE								PS_ENABLE

#if (USER_TASK_STATE == PS_ENABLE)

#define USER_TASK1									PS_ENABLE
#define USER_TASK2									PS_ENABLE
#define USER_TASK3									PS_ENABLE

#endif	//USER_TASK_STATE

#endif	//_AI280_CONFIG_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
