﻿/*
* j1939_datatype.h - contains defines for data types
*
* Copyright (c) 2014-2021 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_datatype.h 38182 2021-10-04 14:25:29Z boe $
*-------------------------------------------------------------------
*
*
*
*/

/**
* \brief data types
*
* \file j1939_datatype.h - contains defines for data types
*
*/

#ifndef J1939_DATATYPE_H
#define J1939_DATATYPE_H 1


/* define of masks */
#define J1939_PGN_PRIORITY_MASK ~(0x3fffffful)

#define J1939_PGN_PRIORITY_0 0x00000000ul /**< bitmask for priority 0 */
#define J1939_PGN_PRIORITY_1 0x04000000ul /**< bitmask for priority 1 */
#define J1939_PGN_PRIORITY_2 0x08000000ul /**< bitmask for priority 2 */
#define J1939_PGN_PRIORITY_3 0x0c000000ul /**< bitmask for priority 3 */
#define J1939_PGN_PRIORITY_4 0x10000000ul /**< bitmask for priority 4 */
#define J1939_PGN_PRIORITY_5 0x14000000ul /**< bitmask for priority 5 */
#define J1939_PGN_PRIORITY_6 0x18000000ul /**< bitmask for priority 6 */
#define J1939_PGN_PRIORITY_7 0x1c000000ul /**< bitmask for priority 7 */


#ifndef ALIGNMENT
# define ALIGNMENT  2
#endif /* ALIGNMENT */

#ifndef U_DSP_T
# if ALIGNMENT == 2
typedef unsigned char U_DSP_T;
# endif /* ALIGNMENT == 2 */
# if ALIGNMENT == 4
typedef unsigned int U_DSP_T;
# endif /* ALIGNMENT == 4 */
#endif


/**
 * J1939 data types
 */
typedef enum {
	J1939_DTYPE_U1, J1939_DTYPE_U2, J1939_DTYPE_U3, J1939_DTYPE_U4,
	J1939_DTYPE_U5, J1939_DTYPE_U6, J1939_DTYPE_U7, J1939_DTYPE_U8,
	J1939_DTYPE_U12, J1939_DTYPE_U16, J1939_DTYPE_U24, J1939_DTYPE_U32,
	J1939_DTYPE_U40, J1939_DTYPE_U48, J1939_DTYPE_U64,
	J1939_DTYPE_NON_NUMERIC = 256,
	J1939_DTYPE_STRING_ASTERIKS,
	J1939_DTYPE_STRING_NULL
} J1939_DATA_TYPE_T;


/**
 * Device Name
 */
typedef struct {
	UNSIGNED32	identifyNumber;		/**< identify number */
	UNSIGNED16	manufacturerCode;	/**< manufacturer code */
	UNSIGNED8	ecuInstance;		/**< ecu instance */
	UNSIGNED8	functionInstance;	/**< function instance */
	UNSIGNED8	function;			/**< function */
	UNSIGNED8	vehicleSystem;		/**< vehicle system */
	UNSIGNED8	vehicleInstance;	/**< vehicle instance */
	UNSIGNED8	industrieGroup;		/**< industrie group */
	UNSIGNED8	adressCapable;		/**< address capable */
} J1939_DEVICE_NAME;


/**
 * Direction
 */
typedef enum {
	DIR_RECEIVE,
	DIR_TRANSMIT
} DIRECTION_T;




/**
 * mapping entries
 */
typedef struct {
	void			*pVar;			/**< Pointer auf Variable */
	UNSIGNED16		nrOfBits;		/**< number of bits for this variable */
	UNSIGNED32		spn;			/**< spn number */
	J1939_DATA_TYPE_T	dType;		/**< data type */
#ifdef J1939_DYNAMIC_PGN
	UNSIGNED8		dynVar;			/**< dynamic variable */
#endif /*J1939_DYNAMIC_PGN*/
} J1939_MAPPING_T;

// Extended descriptor structure to convert SPNs
typedef struct {
	char          sName[32];
	UNSIGNED16    uSourceAddress;
	float         fScaleLoLimit;
	float         fScaleHiLimit;
	float         fResPerBit;
	float         Offset;
	char          Units[8];
} J1939_MAPPING_T_EXT;
/**
 * PGN definition 
 */
typedef struct {
	UNSIGNED32	pgn;				/**< PGN number */
#ifdef J1939_DYNAMIC_PGN
	J1939_MAPPING_T *pMap;			/**< pointer to mapping */
#else /*J1939_DYNAMIC_PGN*/
	const J1939_MAPPING_T *pMap;	/**< pointer to mapping */
#endif /*J1939_DYNAMIC_PGN*/
	UNSIGNED16	cycleTime;			/**< cycle time in msec (0 for acyclic) */
	UNSIGNED16	inhibitTime;		/**< inhibit in msec */
	UNSIGNED8	priority;			/**< PGN priority */
	UNSIGNED16	nrOfMappings;		/**< nr of mappings */
} J1939_PGN_CONFIG_T;


/**
 * Receive State
 * (used for receive indication)
 */
typedef enum {
	J1939_RECEIVE_STATE_ACTIVE,		/**< receive PGN is active */
	J1939_RECEIVE_STATE_FAILURE		/**< receive PGN has been failed */
} J1939_RECEIVE_STATE_T;


/**
 * stack initialisation
 */
typedef struct {
	const J1939_PGN_CONFIG_T	*trPgnConfig;	/**< pointer to transmit PGNs */
	const UNSIGNED16			trPgnCnt;		/**< number of transmit PGNs */
	const J1939_PGN_CONFIG_T	*recPgnConfig;	/**< pointer to receive PGNs */
	const UNSIGNED16			recPgnCnt;		/**< number of receive PGNs */
	const J1939_PGN_CONFIG_T	*devNameConfig;	/**< device name */
	UNSIGNED8					nodeAddress;	/**< first claim address */
} J1939_CONFIG_T;

#endif /* J1939_DATATYPE_H */

typedef enum
{
	ASR_Engine_Control_Active,
	ASR_Brake_Control_Active,
	Anti_Lock_Braking_ABS_Active,
    EBS_Brake_Switch,
	Brake_Pedal_Position,
	ABS_Off_road_Switch,
	ASR_Off_road_Switch,
	ASR_Hill_Holder_Switch,
	Traction_Control_Override_Sw,
	Accelerator_Interlock_Switch,
	Engine_Derate_Switch,
	Engine_Auxiliary_Shutdown_Sw,
    Remote_Accelerator_Enable_Sw,
	Engine_Retarder_Selection,
	ABS_Fully_Operational,
	EBS_Red_Warning_Signal,
    ABS_EBS_Amber_War_Signal_PowV,
	ATC_ASR_Information_Signal,
	Source_Add_of_Cont_Dev_Brak_Con,
	Railroad_Mode_Switch,
	Halt_brake_switch,
	Trailer_ABS_Status,
	Tractor_Mounted_Trailer_ABS_WS
} ePGN_61441;


static const J1939_MAPPING_T_EXT rx_mapping_61441_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "ASR Engine Control Active     ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* ASR Engine Control Active */
	{ "ASR Brake Control Active      ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* ASR Brake Control Active */
	{ "Anti-Lock Braking (ABS) Active", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Anti-Lock Braking (ABS) Active */
	{ "EBS Brake Switch              ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },  	/* EBS Brake Switch */
	{ "Brake Pedal Position          ", 0u,          0.0,       100.0,         0.4,       0.0, "%    " }, 	/* Brake Pedal Position */
	{ "ABS Off-road Switch           ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* ABS Off-road Switch */
	{ "ASR Off-road Switch           ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* ASR Off-road Switch */
	{ """ASR ""Hill Holder"" Switch     ", 0u,       0.0,       3.0,           1.0,      0.0, "s2bit" },	/* "ASR ""Hill Holder"" Switch" */
	{ "Traction Control Override Sw  ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Traction Control Override Switch */
	{ "Accelerator Interlock Switch  ", 0u,          0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Accelerator Interlock Switch */
	{ "Engine Derate Switch           ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Engine Derate Switch */
	{ "Engine Auxiliary Shutdown Sw   ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Engine Auxiliary Shutdown Sw */
	{ "Remote Accelerator Enable Sw   ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Remote Accelerator Enable Switch */
	{ "Engine Retarder Selection      ", 0u,         0.0,       100.0,         0.4,      0.0, "%    " },	/* Engine Retarder Selection */
	{ "ABS Fully Operational          ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit"  },	/* ABS Fully Operational */
	{ "EBS Red Warning Signal         ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit"  },	/* EBS Red Warning Signal */
	{ "ABS/EBS Amber War Signal (PowV)", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit"  },	/* ABS/EBS Amber Warning Signal (Powered Vehicle) */
	{ "ATC/ASR Information Signal     ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit"  },	/* ATC/ASR Information Signal */
	{ "Source Add of Cont Dev Brak Con", 0u,         0.0,       253.0,         1.0,      0.0, "SA   " },	/* Source Address of Controlling Device for Brake Control */
	{ "Railroad Mode Switch           ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Railroad Mode Switch */
	{ "Halt brake switch              ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Halt brake switch */
	{ "Trailer ABS Status             ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Trailer ABS Status */
	{ "Tractor-Mounted Trailer ABS WS ", 0u,         0.0,       3.0,           1.0,      0.0, "s2bit" },	/* Tractor-Mounted Trailer ABS Warning Signal */
};

typedef enum
{
	engine_torque_mode,
	Actual_Engine_P_Torque_Frac,
	Drivers_Demand_Engine_P_Torque,
	Actual_Engine_Percent_Torque,
	Engine_Speed,
	SA_Controlling_Device_for_ECU,
	Engine_Starter_Mode,
	PGN_61444_Dummy_7,
	Engine_Demand_Percent_Torque,
} ePGN_61444;

static const J1939_MAPPING_T_EXT rx_mapping_61444_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Engine Torque Mode            ", 0u,          0.0,       15.0,            16.0,       0.0,"s4bit" },	/* Engine Torque Mode */
	{ "Actual Engine-% Torque (Frac) ", 0u,          0.0,       0.875,           0.125,      0.0, "%    " },	/* Actual Engine - Percent Torque (Fractional) */
	{ "Drivers Demand Engine-% Torque", 0u,       -125.0,       125.0,           1.0,     -125.0, "%    " },	/* Drivers Demand Engine - Percent Torque */
	{ "Actual Engine - Percent Torque", 0u,       -125.0,       125.0,           1.0,     -125.0, "%    " },  	/* Actual Engine - Percent Torque */
	{ "Engine Speed                  ", 0u,          0.0,       8031.875,        0.125,      0.0, "RPM  " }, 	/* Engine Speed */
	{ "SA Controlling Device for ECU ", 0u,          0.0,        253.0,           1.0,       0.0, "SA   "  },	/* Source Address of Controlling Device for Engine Control */
	{ "Engine Starter Mode           ", 0u,          0.0,        15.0,           1.0,        0.0, "s4bit" },	/* Engine Starter Mode */
	{ "                              ", 0u,          0.0,        255.0,          1.0,        0.0, "     " },	    /*  */
	{ "Engine Demand Percent Torque  ", 0u,       -125.0,       125.0,           1.0,     -125.0, "%    " },	/* Engine Demand Percent Torque */
};

static const J1939_MAPPING_T_EXT rx_mapping_64774_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Vehicle Batt Volt Low Lamp Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Vehicle Battery Voltage Low Lamp Command */
	{ "Vehicle Fuel Lev Low Lamp Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Vehicle Fuel Level Low Lamp Command */
	{ "Vehicle Air Pres Low Lamp Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	    /* Vehicle Air Pressure Low Lamp Command */
	{ "Vehicle HVAC Recircu Lamp Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Vehicle HVAC Recirculation Lamp Command */
	{ "Vehicle Battery Char Lamp Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" }, 	/* Vehicle Battery Charging Lamp Command */
	{ "Hill Holder Lamp Command      ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" }, 	/* Hill Holder Lamp Command  */
	{ "Vehicle Lim Perfo Mod Lamp Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit"  },	/* Vehicle Limited Performance Mode Lamp Command */
};

static const J1939_MAPPING_T_EXT rx_mapping_64775_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Engine Protect Lamp Command   ", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },  	/* Engine Protect Lamp Command */
	{ "Engine Amber Warning Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },	/* Engine Amber Warning Lamp Command */
	{ "Engine Red Stop Lamp Command  ", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },	/* Engine Red Stop Lamp Command */
	{ "OBD Malfunction Indi Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },  	/* OBD Malfunction Indicator Lamp Command */
	{ "Engine Brake Active Lamp Comm ", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Brake Active Lamp Command */
	{ "Comp Brake En Sw Ind Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Compression Brake Enable Switch Indicator Lamp Command  */
	{ "Eng Oil Pressure Low Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Oil Pressure Low Lamp Command  */
	{ "Eng Coolant Temp High Lamp Com", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Coolant Temperature High Lamp Command  */
	{ "Eng Coolant Level Low Lamp Com", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Coolant Level Low Lamp Command  */
	{ "Eng Idle Mana Active Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Idle Management Active Lamp Command  */
	{ "Eng Air Filter Restr Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Air Filter Restriction Lamp Command  */
	{ "Eng Fuel Filte Restr Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Fuel Filter Restricted Lamp Command  */
	{ "Eng Ct Mod1 Ready Use Lamp Com", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Control Module 1 Ready for Use Lamp Command  */
	{ "Eng Ct Mod2 Ready Use Lamp Com", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Control Module 2 Ready for Use Lamp Command  */
	{ "Eng Ct Mod3 Ready Use Lamp Com", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Control Module 3 Ready for Use Lamp Command  */
	{ "Engine Speed High Lamp Command", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Speed High Lamp Command  */
	{ "Eng Speed Very High Lamp Comm ", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Speed Very High Lamp Command  */
	{ "Vehicle Acc Rate Lim Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Vehicle Acceleration Rate Limit Lamp Command  */
	{ "Eng Idle Man Pend Ev Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Idle Management Pending Event Lamp Command  */
	{ "Eng Stop-Start Auto ALamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Stop-Start Automatic Stop Active Lamp Command  */
	{ "Eng Stop-Start Auto FLamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Stop-Start Automatic Start Failed Lamp Command  */
	{ "Eng Stop-Start Enabl Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Engine Stop-Start Enabled Lamp Command  */
};

static const J1939_MAPPING_T_EXT rx_mapping_64892_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Diesel Partic Filter Lamp Comm", 0u,          0.0,          7.0,            1.0,      0.0, "s3bit" },  	/* Diesel Particulate Filter Lamp Command */
	{ "Diesel Partic Filter Act RegSt", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },	/* Diesel Particulate Filter Active Regeneration Availability Status */
	{ "DUMMY                         ", 0u,          0.0,          7.0,            1.0,      0.0, "s3bit" },	/* Dummy */
	{ "Aftert Diesel Partic Fil PRSta", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },	/* Aftertreatment Diesel Particulate Filter Passive Regeneration Status */
	{ "Aftert Diesel Partic Fil ARSta", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" },  	/* Aftertreatment Diesel Particulate Filter Active Regeneration Status */
	{ "Aftert Diesel Partic Fil Stat ", 0u ,         0.0,          7.0,            1.0,      0.0, "s3bit" }, 	/* Aftertreatment Diesel Particulate Filter Status */
	{ "Diesel Partic Fil Active RInSt", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Status */
	{ "Diesel Partic Fil Active RIDIS", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Inhibit Switch */
	{ "Diesel Partic Fil Active RIDCD", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Clutch Disengaged */
	{ "Diesel Partic Fil Activ RIDSBA", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Service Brake Active */
	{ "Diesel Partic Fil Acti RIDPTOA", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to PTO Active */
	{ "Diesel Partic Fil Acti RIDAPOI", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Accelerator Pedal Off Idle */
	{ "Diesel Partic Fil Acti RIDONeu", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Out of Neutral */
	{ "Diesel Partic Fil Act RIDVSAAS", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Vehicle Speed Above Allowed Speed */
	{ "Diesel Partic Fil Act RIDPBNSt", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Parking Brake Not Set */
	{ "Diesel Partic Fil Act RIDLETem", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Low Exhaust Temperature */
	{ "Diesel Partic Fil Act RIDSFAct", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to System Fault Active */
	{ "Diesel Partic Fil Act RIDSTime", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to System Timeout */
	{ "Diesel Partic Fil Act RIDTSLoc", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Temporary System Lockout */
	{ "Diesel Partic Fil Act RIDPSLoc", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Permanent System Lockout */
	{ "Diesel Partic Fil Act RIDENWUp", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Engine Not Warmed Up */
	{ "Diesel Partic Fil Act RIDVSBAS", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Vehicle Speed Below Allowed Speed */
	{ "Diesel Partic Fil Auto ARIConf", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Automatic Active Regeneration Initiation Configuration */
	{ "Exhaus Sys High Temp Lamp Comm", 0u,          0.0,          7.0,            1.0,      0.0, "s3bit" }, 	/* Exhaust System High Temperature Lamp Command */
	{ "Diesel Partic Fil Act RFStatus", 0u,          0.0,          7.0,            1.0,      0.0, "s3bit"  }, 	/* Diesel Particulate Filter Active Regeneration Forced Status */
	{ "Hydrocarbon Doser Purging Enab", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Hydrocarbon Doser Purging Enable */
	{ "Diesel Partic Filt ARDLEPressu", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Low Exhaust Pressure */
	{ "Aftert 1 DPF Cond NotM Act Reg", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Aftertreatment 1 Diesel Particulate Filter Conditions Not Met for Active Regeneration */
	{ "Eng Stop-Start Enabl Lamp Comm", 0u,          0.0,          3.0,            1.0,      0.0, "s2bit" }, 	/* Diesel Particulate Filter Active Regeneration Inhibited Due to Thresher */
};

typedef enum
{
	Hydraulic_Brake_Pres_Circuit_1,
    Hydraulic_Brake_Pres_Circuit_2,
    Hydraulic_Brake_Pres_Warn_StC1,
    Hydraulic_Brake_Pres_Warn_StC2,
    Hydraulic_Brake_Pres_Supp_StC1,
	Hydraulic_Brake_Pres_Supp_StC2,
    Hydraulic_Brake_Sys_Aud_W_Comm,
    Hydraulic_Brake_Fluid_Level_Sw
} ePGN_64998;

static const J1939_MAPPING_T_EXT rx_mapping_64998_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Hydraulic Brake Pres Circuit 1", 0u,          0.0,     25000.0,           100.0,      0.0, "KPa  " },    /* Hydraulic Brake Pressure Circuit 1 */
	{ "Hydraulic Brake Pres Circuit 2", 0u,          0.0,     25000.0,           100.0,      0.0, "KPa  " },    /* Hydraulic Brake Pressure Circuit 2 */
	{ "Hydraulic Brake Pres Warn StC1", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Hydraulic Brake Pressure Warning State Circuit 1 */
	{ "Hydraulic Brake Pres Warn StC2", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Hydraulic Brake Pressure Warning State Circuit 2 */
	{ "Hydraulic Brake Pres Supp StC1", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Hydraulic Brake Pressure Supply State Circuit 1 */
	{ "Hydraulic Brake Pres Supp StC2", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Hydraulic Brake Pressure Supply State Circuit 2 */
	{ "Hydraulic Brake Sys Aud W Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Hydraulic Brake System Audible Warning Command */
	{ "Hydraulic Brake Fluid Level Sw", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Hydraulic Brake Fluid Level Switch */
};

static const J1939_MAPPING_T_EXT rx_mapping_65089_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Running Light Command         ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },    /* Running Light Command */
	{ "Alternate Beam Head Light Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" }, /* Alternate Beam Head Light Command */
	{ "Low Beam Head Light Command   ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Low Beam Head Light Command */
	{ "High Beam Head Light Command  ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* High Beam Head Light Command */
	{ "Tractor Front Fog Lights Comma", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Front Fog Lights Command */
	{ "Rotating Beacon Light Command ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Rotating Beacon Light Command */
	{ "Right Turn Signal Lights Comma", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Right Turn Signal Lights Command */
	{ "Left Turn Signal Lights Comman", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Left Turn Signal Lights Command */
	{ "Back Up Light/Alarm Horn Comma", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Back Up Light and Alarm Horn Command */
	{ "Center Stop Light Command     ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Center Stop Light Command */
	{ "Right Stop Light Command      ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Right Stop Light Command */
	{ "Left Stop Light Command       ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Left Stop Light Command */
	{ "Implement Clearance Light Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Clearance Light Command */
	{ "Tractor Clearance Light Comman", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Clearance Light Command */
	{ "Implement Marker Light Command", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Marker Light Command */
	{ "Tractor Marker Light Command  ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Marker Light Command */
	{ "Rear Fog Light Command        ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Rear Fog Light Command */
	{ "Tra Under Moun Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Underside Mounted Work Lights Command */
	{ "Tra Rear Low M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Rear Low Mounted Work Lights Command */
	{ "Tra Rear Hig M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Rear High Mounted Work Lights Command */
	{ "Tra Side Low M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Side Low Mounted Work Lights Command */
	{ "Tra Side Hig M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Side High Mounted Work Lights Command */
	{ "Tra Fron Low M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Front Low Mounted Work Lights Command  */
	{ "Tra Fron Hig M Work Lights Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Tractor Front High Mounted Work Lights Command  */
	{ "Implement OEM Opt2 Light Comma", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement OEM Option 2 Light Command  */
	{ "Implement OEM Opt1 Light Comma", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement OEM Option 1 Light Command  */
	{ "Implement Rig F Work Light Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Right Facing Work Light Command  */
	{ "Implement Lef FW Work Light Cm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Left Forward Work Light Command  */
	{ "Lighting Data Request Command ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Lighting Data Request Command  */
	{ "Implement Rig FW Work Light Cm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Right Forward Work Light Command */
	{ "Implement Rig F Work Light Com", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Left Facing Work Light Command */
	{ "Implement Rear Work Light Comm", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Implement Rear Work Light Command */
};

typedef enum
{
	Aftert1_Diesel_ExFluid_TankVol,
	Aftert1_Diesel_ExFluid_TankTm1,
    Aftert1_Diesel_ExFluid_TankLev,
    Aftert1_Diesel_ExFluid_TankFMI,
	Aftert1_Diesel_ExFluid_TankLoI,
    Aftert1_Diesel_ExFluid_TempFMI,
    Aftert1_SCR_Opera_Ind_Severity,
	Aftert1_Diesel_ExFluid_TankHet,
	Aftert1_Dsl_ExFlu_Tank1HeatFMI,
} ePGN_65110;

static const J1939_MAPPING_T_EXT rx_mapping_65110_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Aftert1 Diesel ExFluid TankVol", 0u,          0.0,       100.0,             0.4,      0.0, "%    " },  	/* Aftertreatment_1_diesel_exhaust_fluid_tank_volume */
	{ "Aftert1 Diesel ExFluid TankTm1", 0u,        -40.0,       210.0,             1.0,    -40.0, "C    " },	/* Aftertreatment_1_diesel_exhaust_fluid_tank_temperature_1 */
	{ "Aftert1 Diesel ExFluid TankLev", 0u,          0.0,       6425.5,            0.1,      0.0, "mm   " },	/* Aftertreatment_1_diesel_exhaust_fluid_tank_level */
	{ "Aftert1 Diesel ExFluid TankFMI", 0u,          0.0,         31.0,           1.0,      0.0,  "binar" },  	/* Aftertreatment_1_diesel_exhaust_fluid_tank_level_volume_preliminary_fmi */
	{ "Aftert1 Diesel ExFluid TankLoI", 0u,          0.0,          7.0,           8.0,      0.0,  "s3bit" }, 	/* Aftertreatment_diesel_exhaust_fluid_tank_low_level_indicator */
	{ "Aftert1 Diesel ExFluid_TempFMI", 0u,          0.0,         31.0,           1.0,      0.0,  "binar"  },	/* Aftertreatment_1_diesel_exhaust_fluid_tank_1_temperature_preliminary_fmi */
	{ "Aftert1 SCR Opera Ind Severity", 0u,          0.0,          7.0,           8.0,      0.0,  "s3bit"  },    /* Aftertreatment_scr_operator_inducement_severity */
	{ "Aftert1 Diesel ExFluid TankHet", 0u,          0.0,        100.0,           1.0,      0.0,  "%    " },	/* Aftertreatment_1_diesel_exhaust_fluid_tank_heater */
	{ "Aftert1 Dsl ExFlu Tank1HeatFMI", 0u,          0.0,         31.0,           1.0,      0.0,  "binar" },	/* Aftertreatment_1_diesel_exhaust_fluid_tank_1_heater_preliminary_fmi */
};

typedef enum
{
	Hydraulic_temperature,
	Hydraulic_oil_filter_restri_SW,
	Winch_oil_pressure_switch,
	PGN_65128_Dummy_3,
	Hydraulic_oil_level
} ePGN_65128;

static const J1939_MAPPING_T_EXT rx_mapping_65128_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Hydraulic temperature         ", 0u,          -40.0,       210.0,           1.0,      -40.0, "C    " },  	        /* Hydraulic_temperature */
	{ "Hydraulic oil filter restri SW", 0u,          0.0,         3.0,             1.0,      0.0,   "s2bit" },	/* Hydraulic_oil_filter_restriction_switch */
	{ "Winch oil pressure switch     ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Winch_oil_pressure_switch */
	{ "DUMMY                         ", 0u,          0.0,         15.0,            1.0,      0.0, "binar" },  	/*  */
	{ "Hydraulic oil level           ", 0u,          0.0,         100.0,           0.4,      0.0, "%    " }, 	/* Hydraulic_oil_level */
};

typedef enum
{
	Engine_fan_1_estimated_prc_speed,
	Fan_drive_state,
	PGN_65213_Dummy_2,
	Fan_speed,
	Hydraulic_fan_motor_pressure,
	Fan_drive_bypass_command_stat
} ePGN_65213;

static const J1939_MAPPING_T_EXT rx_mapping_65213_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Engine fan 1 estimated % speed", 0u,          0.0,         100.0,           1.0,      0.0, "%    " },  	/* Engine_fan_1_estimated_percent_speed */
	{ "Fan drive state               ", 0u,          0.0,         15.0,            1.0,      0.0, "s4bit" },	/* Fan_drive_state */
	{ "DUMMY                         ", 0u,          0.0,         15.0,            1.0,      0.0, "s4bit" },	/* dummy4 */
	{ "Fan speed                     ", 0u,          0.0,         8031.875,        0.125,    0.0, "RPM  " },  	/* Fan_speed */
	{ "Hydraulic fan motor pressure  ", 0u,          0.0,         32127.5,         0.5,      0.0, "KPa  " }, 	/* Hydraulic_fan_motor_pressure */
	{ "Fan drive bypass command stat ", 0u,          0.0,         100.0,           0.4,      0.0, "%    " }, 	/* Fan_drive_bypass_command_status  */
};

typedef enum
{
	Alternator_Speed,
	Alternator_1_Status,
    Alternator_2_Status,
	Alternator_3_Status,
	Alternator_4_Status
} ePGN_65237;

static const J1939_MAPPING_T_EXT rx_mapping_65237_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Alternator Speed              ", 0u,          0.0,     32127.5,             0.5,      0.0, "RPM  " },  /* Alternator Speed */
	{ "Alternator 1 Status           ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Alternator 1 Status */
	{ "Alternator 2 Status           ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Alternator 2 Status */
	{ "Alternator 3 Status           ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Alternator 3 Status */
	{ "Alternator 4 Status           ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Alternator 4 Status */
};

static const J1939_MAPPING_T_EXT rx_mapping_65252_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Eng idle shutdw has shutdw eng", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },    /* Engine_idle_shutdown_has_shutdown_engine */
	{ "Eng idle shutdw driv alert mod", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_idle_shutdown_driver_alert_mode  */
	{ "Eng idle shutdw timer override", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_idle_shutdown_timer_override */
	{ "Eng idle shutdown timer state ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },  	/* Engine_idle_shutdown_timer_state */
	{ "DUMMY                         ", 0u,          0.0,        63.0,             1.0,      0.0, "binar" },    /* */
	{ "Eng idle shutdown timer functi", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_idle_shutdown_timer_function */
	{ "AC high pressure fan switch   ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* AC_high_pressure_fan_switch */
	{ "Refrigerant low pressure switc", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Refrigerant_low_pressure_switch */
	{ "Refrigerant high pressure swit", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Refrigerant_high_pressure_switch */
	{ "DUMMY                         ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* */
	{ "Engine_wait_to_start_lamp     ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_wait_to_start_lamp */
	{ "DUMMY                         ", 0u,          0.0,        63.0,             1.0,      0.0, "binar" },    /* */
	{ "Eng protection syst shutdw eng", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_has_shutdown_engine */
	{ "Eng protection syst appr shutd", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_approaching_shutdown */
	{ "Eng protection syst timer over", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_timer_override */
	{ "Eng protection syst timer over", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_timer_override */
	{ "Eng protection syst timer stat", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_timer_state */
	{ "DUMMY                         ", 0u,          0.0,        63.0,             1.0,      0.0, "binar" },    /* */
	{ "Eng protection syst configurat", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_protection_system_configuration */
	{ "Eng alarm output command stats", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_alarm_output_command_status */
	{ "Eng air shutoff command status", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_air_shutoff_command_status */
	{ "Engine overspeed test         ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_overspeed_test */
	{ "Engine air shutoff status     ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_air_shutoff_status */
	{ "PTO shutdown has shutdown eng ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* PTO_shutdown_has_shutdown_engine */
	{ "Coolant levl eng prot shutd st", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Coolant_level_engine_protection_shutdown_status */
	{ "Engine oil pressure switch    ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Engine_oil_pressure_switch */
};

typedef enum
{
	Engine_Coolant_Temp,
	Engine_Fuel_1_Temperature_1,
    Engine_Oil_Temperature,
	Engine_Turbocharger_1_Oil_Temp,
	Engine_Intercooler_Temperature,
	Eng_Char_Air_Cooler_Therm_Open
} ePGN_65262;

static const J1939_MAPPING_T_EXT mapping_65262_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Engine Coolant Temp           ", 0u,          -40.0,       210.0,       1.0,      -40.0, "C    " }	/* Engine Coolant Temperature */,
	{ "Engine Fuel 1 Temperature 1   ", 0u,          -40.0,       210.0,       1.0,      -40.0, "C    " }	/* Engine Fuel 1 Temperature 1 */,
	{ "Engine Oil Temperature        ", 0u,          -40.0,       210.0,       1.0,      -40.0, "C    " }	/* Engine Oil Temperature 1 */,
	{ "Engine Turbocharger 1 Oil Temp", 0u,          -40.0,       210.0,       1.0,      -40.0, "C    " }	/* Engine Turbocharger Oil Temperature */,
	{ "Engine Intercooler Temperature", 0u,          -40.0,       210.0,       1.0,      -40.0, "C    " }	/* Engine Intercooler Temperature */,
	{ "Eng Char Air Cooler Therm Open", 0u,            0.0,       100.0,       0.4,        0.0, "%    " }	/* Engine Charge Air Cooler Thermostat Opening */
};

typedef enum
{
	Transmission_clutch_1_pressure,
	Transmission_oil_level_1,
	Transmission_filt_diff_pressur,
    Transmission_1_oil_pressure,
	Transmission_oil_temperature_1,
	Transmiss_oil_level_1_high_low,
	Transmiss_oil_level_1_Cntdw_Tm,
	Transmiss_oil_level_1_measu_st
} ePGN_65272;

static const J1939_MAPPING_T_EXT rx_mapping_65272_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Transmission clutch 1 pressure", 0u,          0.0,      4000.0,            16.0,      0.0, "KPa  " },  	/* Transmission_clutch_1_pressure */
	{ "Transmission oil level 1      ", 0u,          0.0,       100.0,             0.4,      0.0, "%    " },	/* Transmission_oil_level_1 */
	{ "Transmission filt diff pressur", 0u,          0.0,       500.0,            2.0,      0.0, "KPa  " },	    /* Transmission_filter_differential_pressure */
	{ "Transmission 1 oil pressure   ", 0u,          0.0,       4000.0,           16.0,      0.0, "KPa  " },  	/* Transmission_1_oil_pressure */
	{ "Transmission oil temperature 1", 0u,       -273.0,  1734.96875,        0.03125,     -273.0,"C    " }, 	/* Transmission_oil_temperature_1 */
	{ "Transmiss oil level 1 high low", 0u,        -62.5,        62.5,        0.5,          -62.5,"L    " }, 	/* Transmission_oil_level_1_high_low */
	{ "Transmiss oil level 1 Cntdw Tm", 0u,          0.0,        13.0,           16.0,      0.0,  "s4bit"  },	/* Transmission_oil_level_1_countdown_timer */
	{ "Transmiss oil level 1 measu st", 0u,          0.0,        15.0,           16.0,      0.0,  "s4bit"  },   /* Transmission_oil_level_1_measurement_status */
};

typedef enum
{
	Brake_application_pressure,
	Brake_primary_pressure,
	Brake_secondary_pressure,
	Parking_brake_actuator,
	Parking_brake_red_warning_sign,
	Park_brake_release_inhibit_sta
} ePGN_65274;

static const J1939_MAPPING_T_EXT rx_mapping_65274_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Brake application pressure    ", 0u,          0.0,     1000.0,              4.0,      0.0, "KPa  " },    /* Brake_application_pressure */
	{ "Brake primary pressure        ", 0u,          0.0,     1000.0,              4.0,      0.0, "KPa  " },    /* Brake_primary_pressure */
	{ "Brake secondary pressure      ", 0u,          0.0,     1000.0,              4.0,      0.0, "KPa  " },	/* Brake_secondary_pressure */
	{ "Parking brake actuator        ", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Parking_brake_actuator */
	{ "Parking brake red warning sign", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Parking_brake_red_warning_signal */
	{ "Park brake release inhibit sta", 0u,          0.0,         3.0,             1.0,      0.0, "s2bit" },	/* Park_brake_release_inhibit_status */
};

typedef enum
{
	Washer_fluid_level,
	Fuel_level_1,
	Engine_fuel_filter_diff_pressu,
	Engine_oil_filter_diff_pressur,
	Cargo_ambient_temperature,
	Fuel_level_2,
	Eng_oil_filter_diff_press_extr
} ePGN_65276;

static const J1939_MAPPING_T_EXT rx_mapping_65276_ext[] = {
   //  char[30]Name                   ,SourceAddress,ScaleLoLimit,ScaleHiLimit,ResPerBit,Offset,Units */
	{ "Washer fluid level            ", 0u,          0.0,       100.0,             0.4,      0.0, "%    " },  	/* Washer_fluid_level */
	{ "Fuel level 1                  ", 0u,          0.0,       100.0,             0.4,      0.0, "%    " },	/* Fuel_level_1 */
	{ "Engine fuel filter diff pressu", 0u,          0.0,       500.0,            2.0,      0.0, "KPa  " },	    /* Engine_fuel_filter_differential_pressure */
	{ "Engine oil filter diff pressur", 0u,          0.0,       125.0,            0.5,      0.0,  "binar" },  	/* Engine_oil_filter_differential_pressure */
	{ "Cargo ambient temperature     ", 0u,       -273.0,  1734.96875,        0.03125,     -273.0,"C    " }, 	/* Cargo_ambient_temperature */
	{ "Fuel level 2                  ", 0u,          0.0,       100.0,            0.4,      0.0,  "%    "  },	/* Fuel_level_2 */
	{ "Eng oil filter diff press extr", 0u,          0.0,       125.0,            0.5,      0.0,  "KPa  "  },    /* Engine_oil_filter_differential_pressure_extended_range */
};
