/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Through_Put_service.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Through put service initialization functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* Include Header files */
#include "../Maximatecc/Inc/Common.h"
/* END Header files */
#if (USER_TASK_STATE == PS_ENABLE)
/* USER CODE BEGIN PV */
#if(USER_TASK1 == PS_ENABLE)
//#define __TEST_USER_TASK1_CODE__
TaskHandle_t	user_task1;
#endif

#if(USER_TASK2 == PS_ENABLE)
//#define __TEST_USER_TASK2_CODE__
TaskHandle_t	user_task2;
#endif

#if(USER_TASK3 == PS_ENABLE)
//#define __TEST_USER_TASK3_CODE__
TaskHandle_t	user_task3;
#endif
/* USER CODE END PV */


#if(USER_TASK1 == PS_ENABLE)

/** @brief User_Task1
 *
 *  User can define their own functionality in this function.
 *
 *  @param  pvParameters : Task Private structure
 *
 *  @return none
 *
 */
void User_Task1_Start(void *pvParameters)
{
	/* Enable this test task by uncommenting "#define __TEST_USER_TASK1_CODE__" */
#ifdef __TEST_USER_TASK1_CODE__
	uint8_t ps_state;
	int8_t led_state;

	while(1)
	{
		GET_DB(IS_PS_INIT_DONE, &ps_state);

		if(TRUE == ps_state)
			break;

		vTaskDelay(500);
	}

	led_state = LED_ON;

	SET_DB(LED_RED_STATE, (uint8_t *)&led_state);

#endif

	while(1)
	{
		vTaskDelay(1000);
	}

}
#endif

#if(USER_TASK2 == PS_ENABLE)
void User_Task2_Start(void *pvParameters)
{

	/* Enable this test task by uncommenting "#define __TEST_USER_TASK2_CODE__" */
#ifdef __TEST_USER_TASK2_CODE__
	uint8_t ps_state;
	int8_t state;

	while(1)
	{
		GET_DB(IS_PS_INIT_DONE, &ps_state);
		if(TRUE == ps_state)
			break;

		vTaskDelay(500);
	}

	state = KEY_BACKLIGHT_ON;

	Set_DL(KEYPAD_BACKLIGHT , (uint8_t *)&state);
#endif

	while(1)
	{
		vTaskDelay(1000);
	}

}
#endif

#if(USER_TASK3 == PS_ENABLE)
void User_Task3_Start(void *pvParameters)
{
	/* Enable this test task by uncommenting "#define __TEST_USER_TASK3_CODE__" */
#ifdef __TEST_USER_TASK3_CODE__
	uint8_t ps_state;
	int8_t led_state;

	while(1)
	{
		GET_DB(IS_PS_INIT_DONE, &ps_state);

		if(TRUE == ps_state)
			break;

		vTaskDelay(500);

	}

	led_state = LED_ON;

	SET_DB(LED_AMB_STATE, (uint8_t *)&led_state);
#endif

	while(1)
	{
		vTaskDelay(1000);
	}

}
#endif

/** @brief Start Through Put service
 *
 *  This function initializes the following modules
 *     ===> Start the Throughput service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t User_Task_Start(void)
{
#if((USER_TASK1 == PS_ENABLE) || (USER_TASK2 == PS_ENABLE) || (USER_TASK3 == PS_ENABLE))
	BaseType_t xReturned;
#endif

#if(USER_TASK1 == PS_ENABLE)
	/* Start the User_Task1_Start service thread */
	xReturned = xTaskCreate(User_Task1_Start, "User_Task1",
			512, NULL, osPriorityIdle,   &user_task1);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the User_Task1 \r\n");
		return PS_FAILURE;
	}
#endif

#if(USER_TASK2 == PS_ENABLE)
	/* Start the User_Task2_Start service thread */
	xReturned = xTaskCreate(User_Task2_Start, "User_Task2",
			512, NULL, osPriorityIdle,   &user_task2);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the User_Task2 \r\n");
		return PS_FAILURE;
	}
#endif

#if(USER_TASK3 == PS_ENABLE)
	/* Start the User_Task3_Start service thread */
	xReturned = xTaskCreate(User_Task3_Start, "User_Task3",
			512, NULL, osPriorityIdle,   &user_task3);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the User_Task3 \r\n");
		return PS_FAILURE;
	}
#endif

	return PS_SUCCESS;
}

#endif
/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
