/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Fdcan.h
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the FDCAN structures and macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _FDCAN_H_
#define _FDCAN_H_


#define Tseg1_min					1	/* Time segement 1 = prop_seg + phase_seg1 */
#define Tseg1_max					16
#define Tseg2_min					1	/* Time segement 2 = phase_seg2 */
#define Tseg2_max					8
#define Sjw_max						4	/* Synchronisation jump width */
#define Brp_min						1	/* Bit-rate prescaler */
#define Brp_max						64
#define Brp_inc						1

#define Ref_clk						10000000	/* CAN system clock frequency in Hz */
#define DEFAULT_BITRATE				1000000
#define BITRATE						0

#define abs(x) ({       \
    long __x = (x);     \
    (__x < 0) ? -__x : __x;   \
  })

typedef enum
{
	TOBUFFER,
    TOFIFO
}fdcantype;

typedef enum
{
	Transfer,
	Receive
}fdcanFstateEnum;


/*!
 * FDCAN Bit Configuration
 */
typedef struct __FDCAN_Bit_Cfg
{
	uint32_t bit_Rate; 		/**< bitrate in kbit/s */
	uint32_t prescaler; 	/**< prescaler */
	uint32_t segment1; 		/**< segment 1 */
	uint32_t segment2; 		/**< segment 2 */

}FDCAN_Bit_Cfg;

#define CAN_CALC_MAX_ERROR   50	  /* in one-tenth of a percent */

/*!
 * Function Declaration
 */
uint8_t  FDCAN_ConfigureTx(FDCAN_HandleTypeDef *hdlrFdcan, uint32_t ident, uint32_t idType,
					uint8_t *TxData_, uint32_t TxBuffer);
uint8_t  FDCAN_SetMaskFilter(FDCAN_HandleTypeDef *hdlrFdcan,	uint32_t idType,
		                    uint32_t filterType, uint32_t filterConfig,
							uint32_t filterID_1, uint32_t filterID_2, uint8_t tobuffer);
uint8_t  FDCAN_SendMessageToBuffer(FDCAN_HandleTypeDef *hdlrFdcan, uint32_t bufferIndex);
uint8_t  FDCAN_SendMessageToTxFioQ(FDCAN_HandleTypeDef *hdlrFdcan,
								  FDCAN_TxHeaderTypeDef *pTxHeader,
								  uint8_t *pTxData);
uint32_t FDCAN_RecieiveRxMessage(FDCAN_HandleTypeDef *hfdcan, uint32_t RxLocation,
							 FDCAN_RxHeaderTypeDef *pRxHeader, uint8_t *pRxData);
uint8_t  FDCAN_Remove_Filter(FDCAN_HandleTypeDef *hdlrFdcan, FDCAN_FilterTypeDef FilterConfig,
							uint32_t idType, uint8_t filterType, uint32_t filterConfig,
							uint32_t filterID_1, uint32_t filterID_2);
uint8_t  FDCAN_GetStatus(FDCAN_HandleTypeDef *hdlrFdcan, uint32_t bufferIndex, uint8_t function);
uint32_t get_cia_sample_point(uint32_t bitrate);

#endif /* _FDCAN_H_ */

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
