/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Digitalio.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Digital Output HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"

/* USER CODE END Includes */

/** @brief DIO Initialize
 *
 *  This function initializes the DIO Expander
 *  IO pins as Output.
 *
 *  @param  None
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t DIO_Init(void)
{

    /* RESET the DIO H1 & DIO L1 pin */
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_0, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_1, GPIO_PIN_RESET);

	/* RESET the DIO H2 & DIO L2 pin */
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_15, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_14, GPIO_PIN_RESET);

	/* RESET the DIO H3 & DIO L3 pin */
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_3, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_2, GPIO_PIN_RESET);

	/* RESET the DIO H4 & DIO L4 pin */
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_9, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_10, GPIO_PIN_RESET);

	return PS_SUCCESS;

}

/** @brief Set Output High
 *
 *  This function sets the Output High in the IO Expander Pins
 *  for the Digital Output High configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Set_OutputHigh(uint8_t dio_num)
{
	switch(dio_num)
	{
		case 1:
			/* SET the DIO H1 & RESET the DIO L1 pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_0, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_1, GPIO_PIN_RESET);
			break;

		case 2:
			/* SET the DIO H2 & RESET the DIO L2 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_15, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_14, GPIO_PIN_RESET);
			break;

		case 3:
			/* SET the DIO H3 & RESET the DIO L3 pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_3, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_2, GPIO_PIN_RESET);
			break;

		case 4:
			/* SET the DIO H4 & RESET the DIO L4 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_9, GPIO_PIN_SET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_10, GPIO_PIN_RESET);
			break;

		default:
			break;
	}

	return PS_SUCCESS;
}

/** @brief Set Output LOW
 *
 *  This function sets the Output Low in the IO Expander Pins
 *  for the Digital Output LOW configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Set_OutputLow(uint8_t dio_num)
{
	switch(dio_num)
	{
		case 1:
			/* RESET the DIO H1 & SET the DIO L1 pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_0, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_1, GPIO_PIN_SET);
			break;

		case 2:
			/* RESET the DIO H2 & SET the DIO L2 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_15, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_14, GPIO_PIN_SET);
			break;

		case 3:
			/* RESET the DIO H3 & SET the DIO L3 pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_3, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_2, GPIO_PIN_SET);
			break;

		case 4:
			/* RESET the DIO H4 & SET the DIO L4 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_9, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_10, GPIO_PIN_SET);
			break;

		default:
			break;
	}

	return PS_SUCCESS;
}

/** @brief Set Output Open
 *
 *  This function sets the Output Open in the IO Expander Pins
 *  for the Digital Output Open configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Set_OutputOpen(uint8_t dio_num)
{
	switch(dio_num)
	{
		case 1:
			/* RESET the DIO H1 & DIO L1 pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_0, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_1, GPIO_PIN_RESET);
			break;

		case 2:
			/* RESET the DIO H2 & DIO L2 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_15, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_14, GPIO_PIN_RESET);
			break;

		case 3:
			/* RESET the DIO H3 & DIO L3 pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_3, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_2, GPIO_PIN_RESET);
			break;

		case 4:
			/* RESET the DIO H4 & DIO L4 pin */
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_9, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(GPIOH, GPIO_PIN_10, GPIO_PIN_RESET);
			break;

		default:
			break;
	}

	return PS_SUCCESS;
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
