/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : LCD.c
   Author             :
   Version            : V0.0.0.1
   Origination Date   : 18/03/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LCD initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             18/03/2022

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
#include "../inc/LCD.h"

#define TIME_COLOR 100

/** @addtogroup BSP
  * @{
  */

/** @addtogroup Components
  * @{
  */

/** @defgroup ST7789V
  * @brief     This file provides a set of functions needed to drive the
  *            FRIDA FRD154BP2901 LCD.
  * @{
  */

/** @defgroup ST7789H2_Private_TypesDefinitions ST7789H2 Private TypesDefinitions
  * @{
  */
typedef struct
{
	uint8_t red;
	uint8_t green;
	uint8_t blue;
} ST7789H2_Rgb888;
/* The below table handle the different values to be set to Memory Data Access Control
   depending on the orientation and pbm image writing where the data order is inverted
*/
static uint32_t OrientationTab[4] =
{
  0x00U , /* Portrait orientation choice of LCD screen               */
  0xC0U , /* Portrait rotated 180� orientation choice of LCD screen  */
  0xA0U , /* Landscape orientation choice of LCD screen              */
  0x60U , /* Landscape rotated 180� orientation choice of LCD screen */
};
/**
  * @}
  */
//void LCD_White_Screen(void);
void     Set_Background_Screen(uint16_t rgb);
void     LCD_Command(uint16_t command, uint16_t * parameters, uint16_t data_interface, uint16_t number_parameters);
uint16_t V_RGB565(uint8_t reverse, uint32_t color24);
uint32_t V_RGB666(uint8_t reverse, uint32_t color24);
void     ReadDisplayStatus(void);
void     ST7789V_DrawBitmap(uint16_t Xpos, uint16_t Ypos, uint8_t *pbmp);
extern   unsigned short _acLenna[] ;
extern   unsigned short _acglobo[] ;
extern   const  uint8_t dragonBitmap[] ;
uint32_t ReadFrameData(uint8_t reverse, uint16_t bgr);
uint32_t ReadFrameData666(uint8_t reverse, uint16_t bgr);
uint8_t  GetB(uint16_t rgb);
uint8_t  GetR(uint16_t rgb);
uint8_t  GetG(uint16_t rgb);
/** @defgroup ST7789V_Private_Defines ST7789H2 Private Defines
  * @{
  */

/**
  * @}
  */

/** @defgroup ST7789V_Private_Macros ST7789H2 Private Macros
  * @{
  */

/**
  * @}
  */

/** @defgroup ST7789V_Private_Variables ST7789H2 Private Variables
  * @{
  */
LCD_DrvTypeDef   ST7789V_drv =
{
  ST7789V_Init,
  ST7789V_ReadID,
  ST7789V_DisplayOn,
  ST7789V_DisplayOff,
  ST7789V_SetCursor,
  ST7789V_WritePixel,
  ST7789V_ReadPixel,
  ST7789V_SetDisplayWindow,
  ST7789V_DrawHLine,
  ST7789V_DrawVLine,
  ST7789V_GetLcdPixelWidth,
  ST7789V_GetLcdPixelHeight,
  ST7789V2_DrawBitmap,
  ST7789V2_DrawRGBImage,
};

static uint16_t WindowsXstart = 0;
static uint16_t WindowsYstart = 0;
static uint16_t WindowsXend = ST7789V_LCD_PIXEL_WIDTH-1;
static uint16_t WindowsYend = ST7789V_LCD_PIXEL_HEIGHT-1;
/**
  * @}
  */

/** @defgroup ST7789H2_Private_FunctionPrototypes ST7789H2 Private FunctionPrototypes
  * @{
  */
static ST7789H2_Rgb888 ST7789V_ReadPixel_rgb888(uint16_t Xpos, uint16_t Ypos);
static void ST7789V_DrawRGBHLine(uint16_t Xpos, uint16_t Ypos, uint16_t Xsize, uint8_t *pdata);

/**
  * @}
  */

/** @defgroup ST7789H2_Private_Functions ST7789H2 Private Functions
  * @{
  */

/**
  * @brief  Initialize the st7789h2 LCD Component.
  * @param  None
  * @retval None
  */
uint8_t ST7789V_Init(void)
{
	volatile uint32_t id = 0;
	id = ST7789V_ReadID();
	if(id != ST7789V_ID)
	{
		return FAIL;
	}

	volatile uint16_t cmd_parameters[64];
    /* Initialize st7789h2 low level bus layer ----------------------------------*/
	LCD_IO_Init();
	/* Sleep In Command */

	ST7789V_WriteReg(ST7789V_SLEEP_IN, (uint16_t*)NULL, 0);
	/* Wait for 10ms */
	LCD_IO_Delay(10);
	/* SW Reset Command */
	ST7789V_WriteReg(ST7789V_SOFTWARE_RESET,(uint16_t*)NULL,  0);
	/* Wait for 200ms */
	LCD_IO_Delay(200);
	/* Sleep Out Command */
	ST7789V_WriteReg(ST7789V_SLEEP_OUT,(uint16_t*)NULL,  0);
    /* Wait for 120ms */
	LCD_IO_Delay(120);
	ST7789V_WriteReg(ST7789V_NORON,(uint16_t *)NULL,  0);

	/* Normal display for Driver Down side */
	cmd_parameters[0] = OrientationTab[ST7789V_ORIENTATION_PORTRAIT];

	ST7789V_WriteReg(ST7789V_NORMAL_DISPLAY,(uint16_t *)&cmd_parameters[0],  1);

	cmd_parameters[0] = 0x00;

#ifdef _16_BITS_
	cmd_parameters[1] = 0xF2;/*RIM  = 0  MDT = 1 b1b0 EPF =0 b5b4*/
#endif

#ifdef _8BITS_
	cmd_parameters[1] = 0xF0;/*RIM  = 0  MDT = 0 b1b0 EPF =0 b5b4*/
#endif

	ST7789V_WriteReg(ST7789V_RAM_CTRL,(uint16_t *)&cmd_parameters[0], 2);

	/* Color mode 16bits/pixel  = '101' = 65k'101'= 16 bit.pixel*/
	/* Color mode 16bits/pixel  35= '110' = 65k'101'= 16 bit.pixel*/

#ifdef _DATA_16_BTS_
	cmd_parameters[0] = 0x55;
#endif

#ifdef _DATA_18_BTS_
	cmd_parameters[0] = 0x66;
#endif

	ST7789V_WriteReg(ST7789V_COLOR_MODE,(uint16_t *)&cmd_parameters[0], 1);
    /* Display inversion On */
	ST7789V_WriteReg(ST7789V_DISPLAY_INVERSION,(uint16_t*)NULL,  0);

	ST7789V_SetCursor(0,0);

#if 0
	cmd_parameters[0] = 0x0000;
	cmd_parameters[1] = 0x000f;
	cmd_parameters[2] = 0x000f;
	ST7789V_WriteReg(ST7789V_FRCTRL1,(uint16_t*)cmd_parameters[0],  3);
#endif

	/*--------------- ST7789V Frame rate setting -------------------------------*/
	/* PORCH control setting */
	cmd_parameters[0] = 0x000C;
	cmd_parameters[1] = 0x000C;
	cmd_parameters[2] = 0x0000;
	cmd_parameters[3] = 0x0033;
	cmd_parameters[4] = 0x0033;
	ST7789V_WriteReg(ST7789V_PORCH_CTRL,(uint16_t *)&cmd_parameters[0],  5);
    /* GATE control setting */
	cmd_parameters[0] = 0x35;
	ST7789V_WriteReg(ST7789V_GATE_CTRL,(uint16_t *)&cmd_parameters[0],  1);
	/*--------------- ST7789V Power setting ------------------------------------*/
	/* VCOM setting */
	cmd_parameters[0] = 0x2b;
	ST7789V_WriteReg(ST7789V_VCOM_SET,(uint16_t *)&cmd_parameters[0], 1);
    /* LCM Control setting */
	cmd_parameters[0] = 0x2C;
	ST7789V_WriteReg(ST7789V_LCM_CTRL,(uint16_t *)&cmd_parameters[0],  1);
	/* VDV and VRH Command Enable */
	cmd_parameters[0] = 0x0001;
	cmd_parameters[1] = 0x00ff;
	ST7789V_WriteReg(ST7789V_VDV_VRH_EN,(uint16_t *)&cmd_parameters[0],  2);
    /*VRHS: VRH Set*/
	cmd_parameters[0] = 0x0011;
	ST7789V_WriteReg(ST7789V_VRH_SET,(uint16_t *)&cmd_parameters[0],  1);
    /* VDV Set =0V */
	cmd_parameters[0] = 0x20;
	ST7789V_WriteReg(ST7789V_VDV_SET,(uint16_t *)&cmd_parameters[0],  1);

	/* Frame Rate Control in normal mode :60fps*/
	cmd_parameters[0] = 0x0F;
	ST7789V_WriteReg(ST7789V_FR_CTRL,(uint16_t *)&cmd_parameters[0],  1);
    /* Power Control */
	cmd_parameters[0] = 0x00A4;
	cmd_parameters[1] = 0x00A1;
	ST7789V_WriteReg(ST7789V_POWER_CTRL,(uint16_t *)&cmd_parameters[0],  2);
    /*--------------- ST7789H2 Gamma setting ------------------------------------*/
	cmd_parameters[0] = 0x0001;//Gamma Curve 1 (G2.2)
	ST7789V_WriteReg(ST7789V_GAMSET,(uint16_t *)&cmd_parameters[0],  1);

	/* Positive Voltage Gamma Control */

	/*Default*/
	cmd_parameters[0] = 0x00d0; /*VP63P3 V63P2 V63P1 V63P0 V0P3 V0P2 V0P1 V0P0*/
	cmd_parameters[1] = 0x0000; /*0       0     V1P5 V1P4  V1P3 V1P2 V1P1 V1P0*/
	cmd_parameters[2] = 0x0002;  /*0      0     V2P5 V2P4  V2P3 V2P2 V2P1 V2P0*/
	cmd_parameters[3] = 0x0004;  /*0      0     0    V4P4  V4P3 V4P2 V4P1 V4P0*/
	cmd_parameters[4] = 0x000B;  /*0      0     0    V6P4  V6P3 V6P2 V6P1 V6P0*/
	cmd_parameters[5] = 0x001A;  /*0      0     J0P1 J0P0  V13P3 V13P2 V13P1 V13P0*/
	cmd_parameters[6] = 0x0031;  /*0    V20P6  V20P5 V20P4 V20P3 V20P2 V20P1 V20P0*/
	cmd_parameters[7] = 0x0005;  /*0    V36P6  V36P5 V36P4 V36P3 V36P2 V36P1 V36P0*/
	cmd_parameters[8] = 0x0040;  /*0 V43P6 V43P5 V43P4 V43P3 V43P2 V43P1 V43P0*/
	cmd_parameters[9] = 0x0029; /*0 0 J1P1 J1P0 V50P3 V50P2 V50P1 V50P0*/
	cmd_parameters[10] = 0x0012; /*0 0 0 V57P4 V57P3 V57P2 V57P1 V57P0*/
	cmd_parameters[11] = 0x0012; /*0 0 0 V59P4 V59P3 V59P2 V59P1 V59P0*/
	cmd_parameters[12] = 0x0012; /*0 0 V61P5 V61P4 V61P3 V61P2 V61P1 V61P0*/
	cmd_parameters[13] = 0x0017; /*0 0 V62P5 V62P4 V62P3 V62P2 V62P1 V62P0*/

	ST7789V_WriteReg(ST7789V_PV_GAMMA_CTRL,(uint16_t *)&cmd_parameters[0],  14);
    /* Negative Voltage Gamma Control */

	/*Default*/
	cmd_parameters[0] = 0x00d0; /*V63N3 V63N2 V63N1 V63N0 V0N3 V0N2 V0N1 V0N0*/
	cmd_parameters[1] = 0x0000; /*0 0 V1N5 V1N4 V1N3 V1N2 V1N1 V1N0*/
	cmd_parameters[2] = 0x0002;  /*0 0 V2N5 V2N4 V2N3 V2N2 V2N1 V2N0*/
	cmd_parameters[3] = 0x0007;  /*0 0 0 V4N4 V4N3 V4N2 V4N1 V4N0*/
	cmd_parameters[4] = 0x0005;  /*0 0 0 V6N4 V6N3 V6N2 V6N1 V6N0*/
	cmd_parameters[5] = 0x0025;  /*0 0 J0N1 J0N0 V13N3 V13N2 V13N1 V13N0*/
	cmd_parameters[6] = 0x002d;  /*0 V20N6 V20N5 V20N4 V20N3 V20N2 V20N1 V20N0*/
	cmd_parameters[7] = 0x0044;  /*0 V36N2 V36N1 V36N0 0 V27N2 V27N1 V27N0*/
	cmd_parameters[8] = 0x0044;  /*0 V43N6 V43N5 V43N4 V43N3 V43N2 V43N1 V43N0*/
	cmd_parameters[9] = 0x001c;  /*0 0 J1N1 J1N0 V50N3 V50N2 V50N1 V50N0*/
	cmd_parameters[10] = 0x0018;  /*0 0 0 V57N4 V57N3 V57N2 V57N1 V57N0*/
	cmd_parameters[11] = 0x0016;   /*0 0 0 V59N4 V59N3 V59N2 V59N1 V59N0*/
	cmd_parameters[12] = 0x001c;   /*0 0 V61N5 V61N4 V61N3 V61N2 V61N1 V61N0*/
	cmd_parameters[13] = 0x001d;   /*0 0 V62N5 V62N4 V62N3 V62N2 V62N1 V62N0*/

	ST7789V_WriteReg(ST7789V_NV_GAMMA_CTRL,(uint16_t *)&cmd_parameters[0],  13);
#if 0
	cmd_parameters[0] = 0x04;
	ST7789V_WriteReg(ST7789V_DGMEN,(uint16_t *)cmd_parameters[0],  1);
	/*Digital Gamma Look-up Table for Red */
	for(int i = 0;i<64;i++)
	{
		cmd_parameters[i] = i*4;
	}

	ST7789V_WriteReg(ST7789V_DGMLUTR,(uint16_t*)cmd_parameters[0],  64);
	/*Digital Gamma Look-up Table for Blue*/
	for(int i = 0;i<64;i++)
	{
		cmd_parameters[i] = i*4;
	}

	ST7789V_WriteReg(ST7789V_DGMLUTB,(uint16_t*)cmd_parameters[0],  64);
#endif

	ST7789V_WriteReg(ST7789V_SLEEP_OUT,(uint16_t*)NULL,  0);
#if 0
	LCD_IO_Delay(120);

	ST7789V_WriteReg(ST7789V_IDLE_MODE_ON,(uint16_t*)NULL,  0);

	/*Write Content Adaptive Brightness Control and Color Enhancement*/
	//'11' - Moving Image '01' Medum enhancement '1' Color Enhancement On
	cmd_parameters[0] = 0x93;
	ST7789V_WriteReg(ST7789V_WRCACE,(uint16_t *)&cmd_parameters[0],  1);

    /*Write CABC Minimum Brightness*/
	//  In principle relationship is that 00h value means the lowest brightness
	//  for CABC and FFh value means the brightness for CABC.
	cmd_parameters[0] = 0xFF;
	ST7789V_WriteReg(ST7789V_WRCABCMB,(uint16_t *)&cmd_parameters[0],  1);
#endif

#if 0
	LCD_IO_Delay(120);
	/*Write Control Display*/
	/*Default 0
   	   Set 0x24 0010 0100
   	   D7 = 0
   	   d6 =0
   	   d5 = BCTRL = 1   Brightness Control Block ;0-off, 1 on
   	   DD = 0  Display Diming  Only for manual brightness setting 0 off 1 on
   	   BL = 1  Backlight Control ;0 off 1 on
	 */
	cmd_parameters[0] = 0x24;
	ST7789V_WriteReg(ST7789V_WCRTLDISP,(uint16_t *)&cmd_parameters[0],  1);
	LCD_IO_Delay(200);
	/*Write Display Brightness */
	//  In principle relationship is that 00h value means the lowest brightness
	//  and FFh value means the highest brightness.
	cmd_parameters[0] = 0xFF;
	ST7789V_WriteReg(ST7789V_WBRIGHTNESS,(uint16_t *)&cmd_parameters[0],  1);
	LCD_IO_Delay(600);
#endif

	//ST7789V_DisplayOn();

#if 0
    /* Tearing Effect Line On: Option (00h:VSYNC Interface OFF, 01h:VSYNC Interface ON) */
	cmd_parameters[0] = 0x00;
	ST7789V_WriteReg(ST7789V_TEARING_EFFECT, (uint16_t *)&cmd_parameters[0], 1);
#endif

#if 0
	Set_Background_Screen(V_RGB565(INVERSION,RGB_RED));
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_DARK_GREEN));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_GREEN));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_DARK_CYAN));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_MAROON));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_PURPLE));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_OLIVE));//fail

	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_LIGHTGREY));//fail
	LCD_IO_Delay(TIME_COLOR);

	Set_Background_Screen(V_RGB565(INVERSION,RGB_DARKGREY));//fail

	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_CYAN));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_BLUE));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_CYAN));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_BLACK));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_WHITE));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_NAVY));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_MAGENTA));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_YELLOW));//ok
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_ORANGE));//fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_GREEN_YELLOW));//ok/fail
	LCD_IO_Delay(TIME_COLOR);
	Set_Background_Screen(V_RGB565(INVERSION,RGB_PINK));//fail
	LCD_IO_Delay(TIME_COLOR);

	Set_Background_Screen( V_RGB565(1,0xabab5e));
	//  ST7789V_DrawBitmap(0,0,(uint8_t *)&_acLenna);
	// ReadDisplayStatus();
#endif
	return OK;
}

void ReadDisplayStatus(void)
{
	uint16_t data[5] = {0, 0, 0, 0 };

	ST7789V_WriteReg(ST7789V_RDDST,(uint16_t *)NULL,  0);

	for(int i =0; i<5;i++)
	{
		data[i] = ST7789V_ReadReg(ST7789V_LCD_ID);
	}

	volatile uint8_t BSTON = (data[1]>>7)&0x01;
	volatile uint8_t MY = (data[1] >>6)&0x01;
	volatile uint8_t MX = (data[1] >>5)&0x01;
	volatile uint8_t MV = (data[1] >>4)&0x01;
	volatile uint8_t ML = (data[1] >>3)&0x01;
	volatile uint8_t RGB = (data[1] >>2)&0x01;
	volatile uint8_t MH = (data[1] >>1)&0x01;

	volatile uint8_t IFPF2 = (data[2]>>6)&0x01;
	volatile uint8_t IFPF1 = (data[2]>>5)&0x01;
	volatile uint8_t IFPF0 = (data[2]>>4)&0x01;
	volatile uint8_t IDMON = (data[2]>>3)&0x01;
	volatile uint8_t PTLON = (data[2]>>2)&0x01;
	volatile uint8_t SLOUT = (data[2]>>1)&0x01;
	volatile uint8_t NORON = (data[2]>>0)&0x01;

	volatile uint8_t ST15 = (data[3]>>7)&0x01;
	//  uint8_t IFPF1 = (data[3]>>6)&0x01;
	volatile uint8_t INVON = (data[3]>>5)&0x01;
	//  uint8_t DISON = (data[3]>>4)&0x01;
	//  uint8_t PTLON = (data[3]>>3)&0x01;
	volatile uint8_t DISON = (data[3]>>2)&0x01;
	volatile uint8_t TEON = (data[3]>>1)&0x01;
	volatile uint8_t GCS2 = (data[3]>>0)&0x01;

	volatile uint8_t GCS1 = (data[4]>>7)&0x01;
	volatile uint8_t GCS0 = (data[4]>>6)&0x01;
	volatile uint8_t TEM = (data[4]>>5)&0x01;
	uint8_t id =0;
	if(BSTON == 1)
	{
		id =1;/*Booster ON*/
	}
	if(MY == 1)
	{
		id =1;/*�0� =Increment, (Top to Bottom, when MADCTL (36h) D7=�0�)*/
	}
	if(MX == 1)
	{
		id =1;/*�0� =Increment, (Left to Right, when MADCTL (36h) D6=�0�)*/
	}
	if(MV == 1)
	{
		id =1;/*�0� = Normal, (when MADCTL (36h) D5=�0�*/
	}
	if(ML == 1)
	{
		id =1;/*�0� =Decrement, LCD refresh Top to Bottom, when MADCTL (36h) D4=�0�) */
	}
	if(RGB == 1)
	{
		id =1;/*�1� =BGR, (When MADCTL (36h) D3=�1�) */
	}
	if(MH == 1)
	{
		id =1;/*�0� =Decrement,LCD refresh Left to Right, when MADCTL (36h) D2=�0�*/
	}
	if(IFPF2 == 1)
	{
		id =1;
	}
	if(IFPF1 == 1)
	{
		id =1;
	}

	if(IFPF0 == 1)
	{
		id =1;/*�101� = 16-bit / pixel,*/
	}

	if(IDMON == 1)
	{
		id =1;/*�1� = On*/
	}

	if(PTLON == 1)
	{
		id =1;/*�0� = Off*/
	}

	if(SLOUT == 1)
	{
		id =1;/*�1� = Out*/
	}

	if(NORON == 1)
	{
		id =1;/*�1� = Normal Display*/
	}

	if(ST15 == 1)
	{
		id =1;/*�0� = Scroll off*/
	}

	if(INVON == 1)
	{
		id =1;/*�1� = On*/
	}

	if(DISON == 1)
	{
		id =1;/*�1� = On*/
	}
    if(TEON == 1)
    {
    	id =1;/*�1� = On*/
    }
    if(GCS2 == 1)
    {
    	id =1;
    }
    if(GCS1 == 1)
    {
    	id =1;
    }
    if(GCS0 == 1)
    {
    	id =1;/*�000� = GC0 */
    }
    if(TEM == 1)
    {
    	id =1;/*�0� = mode1*/
    }
    ST7789V_WriteReg(ST7789V_RDDCOLMOD,(uint16_t *)NULL,  0);

    for(int i =0; i<2;i++)
    {
    	data[i] = ST7789V_ReadReg(ST7789V_LCD_ID);
    }

    volatile uint8_t display_format = data[1];

    if(display_format ==1)
    {
    	id =1;//RGB interface color format 0, Control interface color format 16 bit/pixel
    }

    if(id == 1)
    {
    }
}


/**
  * @brief  Enables the Display.
  * @param  None
  * @retval None
  */
void ST7789V_DisplayOn(void)
{
	/* Display ON command */
	ST7789V_WriteReg(ST7789V_DISPLAY_ON, (uint16_t*)NULL, 0);

	/* Sleep Out command */
	ST7789V_WriteReg(ST7789V_SLEEP_OUT, (uint16_t*)NULL, 0);
}

/**
  * @brief  Disables the Display.
  * @param  None
  * @retval None
  */
void ST7789V_DisplayOff(void)
{
	uint16_t   parameter[1];
	parameter[0] = 0xFE;
	/* Display OFF command */
	ST7789V_WriteReg(ST7789V_DISPLAY_OFF, parameter, 1);
	/* Sleep In Command */
	ST7789V_WriteReg(ST7789V_SLEEP_IN, (uint16_t*)NULL, 0);
	/* Wait for 10ms */
	LCD_IO_Delay(10);
}

/**
  * @brief  Get the LCD pixel Width.
  * @param  None
  * @retval The Lcd Pixel Width
  */
uint16_t ST7789V_GetLcdPixelWidth(void)
{
	return (uint16_t)ST7789V_LCD_PIXEL_WIDTH;
}

/**
  * @brief  Get the LCD pixel Height.
  * @param  None
  * @retval The Lcd Pixel Height
  */
uint16_t ST7789V_GetLcdPixelHeight(void)
{
	return (uint16_t)ST7789V_LCD_PIXEL_HEIGHT;
}

/**
  * @brief  Get the st7789h2 ID.
  * @param  None
  * @retval The st7789h2 ID
  */
uint32_t ST7789V_ReadID(void)
{
	volatile uint32_t id =0;
	LCD_IO_Init();
	/*dummy, id1,id2,id3*/
	uint16_t data[4] = {0, 0, 0, 0 };

	ST7789V_WriteReg(ST7789V_LCD_ID,(uint16_t *)NULL,  0);

	for(int i =0; i<4;i++)
	{
		data[i] = ST7789V_ReadReg(ST7789V_LCD_ID);
	}

	id = ((data[1] << 16) |
			(data[2] << 8) | data[3]);

	return id;
}

/**
  * @brief  Set Cursor position.
  * @param  Xpos: specifies the X position.
  * @param  Ypos: specifies the Y position.
  * @retval None
  */
void ST7789V_SetCursor(uint16_t Xpos, uint16_t Ypos)
{
	uint16_t   parameter[4];
	/* CASET: Comumn Addrses Set */
	parameter[0] = 0x00;
	parameter[1] = 0x00 + Xpos;
	parameter[2] = 0x00;
	parameter[3] = 0xEF + Xpos;
	ST7789V_WriteReg( ST7789V_CASET , parameter, 4);
	/* RASET: Row Addrses Set */
	parameter[0]  =0x00 + (Ypos>>8) ;
	parameter[1] = 0x00 + (Ypos&0xff);;
	parameter[2] = 0x01 + (Ypos>>8) ;
	parameter[3] = 0x3f + (Ypos&0xff);
	ST7789V_WriteReg(ST7789V_RASET, parameter, 4);
}

/**
  * @brief  Write pixel.
  * @param  Xpos: specifies the X position.
  * @param  Ypos: specifies the Y position.
  * @param  RGBCode: the RGB pixel color in RGB565 format
  * @retval None
  */
void ST7789V_WritePixel(uint16_t Xpos, uint16_t Ypos, uint16_t RGBCode)
{
	/* Set Cursor */
	ST7789V_SetCursor(Xpos, Ypos);

	/* Prepare to write to LCD RAM */
	ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t*)NULL, 0);   /* RAM write data command */

	/* Write RAM data */
#ifdef _8BITS_
	LCD_IO_WriteData(RGBCode>>8);
	LCD_IO_WriteData(RGBCode&0xff);
#endif

#ifdef _16_BITS_
	LCD_IO_WriteData(RGBCode);
#endif
}

/**
  * @brief  Read pixel.
  * @param  Xpos: specifies the X position.
  * @param  Ypos: specifies the Y position.
  * @retval The RGB pixel color in RGB565 format
  */
uint16_t ST7789V_ReadPixel(uint16_t Xpos, uint16_t Ypos)
{
	ST7789H2_Rgb888 rgb888;
	uint8_t          r, g, b;
	uint16_t         rgb565;

	/* Set Cursor */
	ST7789V_SetCursor(Xpos, Ypos);

	/* Read RGB888 data from LCD RAM */
	rgb888 = ST7789V_ReadPixel_rgb888(Xpos, Ypos);

	/* Convert RGB888 to RGB565 */
	r = ((rgb888.red & 0xF8) >> 3);    /* Extract the red component 5 most significant bits */
	g = ((rgb888.green & 0xFC) >> 2);  /* Extract the green component 6 most significant bits */
	b = ((rgb888.blue & 0xF8) >> 3);   /* Extract the blue component 5 most significant bits */

	rgb565 = ((uint16_t)(r) << 11) + ((uint16_t)(g) << 5) + ((uint16_t)(b) << 0);

	return (rgb565);
}



/**
  * @brief  Reads the selected LCD Register.
  * @param  Command: command value (or register address as named in st7789h2 doc).
  * @retval Register Value.
  */
uint16_t ST7789V_ReadReg(uint8_t Command)
{
#ifdef _8BITS_
	/* Read register value */
	return ((LCD_IO_ReadData()>>8)&0xff);
#endif

#ifdef _16_BITS_
	return LCD_IO_ReadData();
#endif
}

/**
  * @brief  Sets a display window
  * @param  Xpos:   specifies the X bottom left position.
  * @param  Ypos:   specifies the Y bottom left position.
  * @param  Height: display window height.
  * @param  Width:  display window width.
  * @retval None
  */

void ST7789V_SetDisplayWindow(uint16_t Xpos, uint16_t Ypos, uint16_t Width, uint16_t Height)
{
	uint16_t   parameter[4];

    /* CASET: Column Address Set */
    /* Column addr set, 4 args, no delay: XSTART = Xpos, XEND = (Xpos + Width - 1) */
    parameter [0] = (uint8_t)(Xpos >> 8U);
    parameter [1] = (uint8_t)(Xpos & 0xFFU);
    parameter [2] = (uint8_t)((Xpos + Width - 1U) >> 8U);
    parameter [3] = (uint8_t)((Xpos + Width - 1U) & 0xFFU);
    ST7789V_WriteReg(ST7789V_CASET, parameter, 4);

    /* RASET: Row Address Set */
    /* Row addr set, 4 args, no delay: YSTART = Ypos, YEND = (Ypos + Height - 1) */
    parameter [0] = (uint8_t)(Ypos >> 8U);
    parameter [1] = (uint8_t)(Ypos & 0xFFU);
    parameter [2] = (uint8_t)((Ypos + Height - 1U) >> 8U);
    parameter [3] = (uint8_t)((Ypos + Height - 1U) & 0xFFU);
    ST7789V_WriteReg(ST7789V_RASET, parameter, 4);

    ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t *)NULL, 0);

}
/**
  * @brief  Draw vertical line.
  * @param  RGBCode: Specifies the RGB color in RGB565 format
  * @param  Xpos:     specifies the X position.
  * @param  Ypos:     specifies the Y position.
  * @param  Length:   specifies the Line length.
  * @retval None
  */
void ST7789V_DrawHLine(uint16_t RGBCode, uint16_t Xpos, uint16_t Ypos, uint16_t Length)
{
	uint16_t counter = 0;

	/* Set Cursor */
	ST7789V_SetCursor(Xpos, Ypos);

	/* Prepare to write to LCD RAM */
	ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t*)NULL, 0);   /* RAM write data command */

	/* Sent a complete line */
	for(counter = 0; counter < Length; counter++)
	{
#ifdef _8BITS_
		LCD_IO_WriteData(RGBCode>>8);
		LCD_IO_WriteData(RGBCode&0xff);
#endif
#ifdef _16_BITS_
		LCD_IO_WriteData(RGBCode);
#endif
  }
}

/**
  * @brief  Draw vertical line.
  * @param  RGBCode: Specifies the RGB color
  * @param  Xpos:     specifies the X position.
  * @param  Ypos:     specifies the Y position.
  * @param  Length:   specifies the Line length.
  * @retval None
  */
void ST7789V_DrawVLine(uint16_t RGBCode, uint16_t Xpos, uint16_t Ypos, uint16_t Length)
{
	uint16_t counter = 0;

	/* Set Cursor */
	ST7789V_SetCursor(Xpos, Ypos);

	/* Prepare to write to LCD RAM */
	ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t*)NULL, 0);   /* RAM write data command */

	/* Fill a complete vertical line */
	for(counter = 0; counter < Length; counter++)
	{
		ST7789V_WritePixel(Xpos, Ypos + counter, RGBCode);
	}
}

/**
  * @brief  Displays a bitmap picture.
  * @param  BmpAddress: Bmp picture address.
  * @param  Xpos: Bmp X position in the LCD
  * @param  Ypos: Bmp Y position in the LCD
  * @retval None
  */
void ST7789V_DrawBitmap(uint16_t Xpos, uint16_t Ypos, uint8_t *pbmp)
{
	uint32_t index = 0, size = 0;
	uint32_t posY;
	uint32_t nb_line = 0;
	uint16_t Xsize = WindowsXend - WindowsXstart + 1;
	uint16_t Ysize = WindowsYend - WindowsYstart + 1;

	/* Read bitmap size */
	size = *(volatile uint16_t *) (pbmp + 2);
	size |= (*(volatile uint16_t *) (pbmp + 4)) << 16;
	/* Get bitmap data address offset */
	index = *(volatile uint16_t *) (pbmp + 10);
	index |= (*(volatile uint16_t *) (pbmp + 12)) << 16;
	size = (size - index)/2;
	pbmp += index;

	for (posY = (Ypos + Ysize); posY > Ypos; posY--)  /* In BMP files the line order is inverted */
	{
		/* Set Cursor */
		ST7789V_SetCursor(Xpos, posY - 1);

		/* Draw one line of the picture */
		ST7789V_DrawRGBHLine(Xpos, posY - 1, Xsize, (pbmp + (nb_line * Xsize * 2)));
		nb_line++;
	}
}

/**
  * @brief  Displays picture.
  * @param  pdata: picture address.
  * @param  Xpos: Image X position in the LCD
  * @param  Ypos: Image Y position in the LCD
  * @param  Xsize: Image X size in the LCD
  * @param  Ysize: Image Y size in the LCD
  * @retval None
  */
void ST7789V_DrawRGBImage(uint16_t Xpos, uint16_t Ypos, uint16_t Xsize, uint16_t Ysize, uint8_t *pdata)
{
	uint32_t posY;
	uint32_t nb_line = 0;

	for (posY = Ypos; posY < (Ypos + Ysize); posY ++)
	{
		/* Set Cursor */
		ST7789V_SetCursor(Xpos, posY);

		/* Draw one line of the picture */
		ST7789V_DrawRGBHLine(Xpos, posY, Xsize, (pdata + (nb_line * Xsize * 2)));
		nb_line++;
	}
}


/******************************************************************************
                            Static Functions
*******************************************************************************/

/**
  * @brief  Read pixel from LCD RAM in RGB888 format
  * @param  Xpos: specifies the X position.
  * @param  Ypos: specifies the Y position.
  * @retval Each RGB pixel color components in a structure
  */
static ST7789H2_Rgb888 ST7789V_ReadPixel_rgb888(uint16_t Xpos, uint16_t Ypos)
{
	ST7789H2_Rgb888 rgb888;
	uint16_t         rgb888_part1, rgb888_part2;

	/* In LCD RAM, pixels are 24 bits packed and read with 16 bits access
	 * Here is the pixels components arrangement in memory :
	 *       bits:  15 14 13 12 11 10 09 08 | 07 06 05 04 03 02 01 00
	 * address 0 :     red pixel 0    X  X  |   green pixel 0   X  X
	 * address 1 :    blue pixel 0    X  X  |     red pixel 1   X  X
	 * address 2 :   green pixel 1    X  X  |    blue pixel 1   X  X
	 */

	/* Set Cursor */
	ST7789V_SetCursor(Xpos, Ypos);
	/* Prepare to read LCD RAM */
	ST7789V_WriteReg(ST7789V_READ_RAM, (uint16_t*)NULL, 0);   /* RAM read data command */
	/* Dummy read */
	LCD_IO_ReadData();
	/* Read first part of the RGB888 data */
	rgb888_part1 = LCD_IO_ReadData();
	/* Read first part of the RGB888 data */
	rgb888_part2 = LCD_IO_ReadData();

	/* red component */
	rgb888.red   = (rgb888_part1 & 0xFC00) >> 8;
	/* green component */
	rgb888.green = (rgb888_part1 & 0x00FC) >> 0;
	/* blue component */
	rgb888.blue  = (rgb888_part2 & 0xFC00) >> 8;

	return rgb888;
}


/**
  * @brief  Displays a single picture line.
  * @param  pdata: picture address.
  * @param  Xpos: Image X position in the LCD
  * @param  Ypos: Image Y position in the LCD
  * @param  Xsize: Image X size in the LCD
  * @retval None
  */
static void ST7789V_DrawRGBHLine(uint16_t Xpos, uint16_t Ypos, uint16_t Xsize, uint8_t *pdata)
{
	uint32_t i = 0;
	uint32_t posX;
	uint16_t *rgb565 = (uint16_t*)pdata;

	/* Prepare to write to LCD RAM */
	ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t*)NULL, 0);   /* RAM write data command */

	for (posX = Xpos; posX < (Xsize + Xpos); posX++)
	{
		if ((posX >= WindowsXstart) && (Ypos >= WindowsYstart) &&     /* Check we are in the defined window */
				(posX <= WindowsXend) && (Ypos <= WindowsYend))
		{
			if (posX != (Xsize + Xpos))     /* When writing last pixel when size is odd, the third part is not written */
			{
#ifdef _8BITS_
				LCD_IO_WriteData(rgb565[i]>>8);
				LCD_IO_WriteData(rgb565[i]&0xff);
#endif

#ifdef _16_BITS_
				LCD_IO_WriteData(rgb565[i]);
#endif
			}
			i++;
		}
	}
}

/**
  * @}
  */

void Set_Background_Screen(uint16_t rgb)
{
	//volatile uint16_t swap_rgb = (rgb&0xff)<<8;
	//swap_rgb |= (rgb >>8)&0xff;
	uint32_t i;
	ST7789V_SetDisplayWindow(0,0,ST7789V_LCD_PIXEL_WIDTH,ST7789V_LCD_PIXEL_HEIGHT);
	/* Prepare to write to LCD RAM */
	ST7789V_WriteReg(ST7789V_WRITE_RAM, (uint16_t*)NULL, 0);   /* RAM write data command */

	// RGB in 565 format
	volatile uint32_t frame = ReadFrameData(INVERSION,rgb);

	for(i=0;i<(ST7789V_LCD_PIXEL_WIDTH*ST7789V_LCD_PIXEL_HEIGHT);i++)
	{
#ifdef _16_BITS_

#ifdef _DATA_16_BTS_
		LCD_IO_WriteData(rgb);//ac4a
#endif

#ifdef _DATA_18_BTS_
		LCD_IO_WriteData(GetR(rgb)<<10 |  GetG(rgb)<<4 |  GetB(rgb)>>2);
		LCD_IO_WriteData(GetB(rgb)<<14);
#endif

#endif

#ifdef _8BITS_
#ifdef _DATA_18_BTS_
		LCD_IO_WriteData(GetB(rgb));
		LCD_IO_WriteData(GetG(rgb));
		LCD_IO_WriteData(GetR(rgb));
#endif

#ifdef _DATA_16_BTS_
		LCD_IO_WriteData(rgb>>8);//ac4a
		LCD_IO_WriteData(rgb&0xff);//dummy
#endif

#endif
		}

#if 1
	volatile uint16_t cmd_parameters[8] ;
	cmd_parameters[0] = 0x66;
	ST7789V_WriteReg(ST7789V_COLOR_MODE,(uint16_t *)&cmd_parameters[0], 1);

	ST7789V_WriteReg(ST7789V_READ_RAM,(uint16_t *)NULL,  0);

	for(int i =0; i<8;i++)
	{
		cmd_parameters[i] = ST7789V_ReadReg(ST7789V_LCD_ID);
	}
	uint32_t read = cmd_parameters[1] <<24 |cmd_parameters[2] <<16 |cmd_parameters[3];
	// frame = ReadFrameData666(INVERSION,rgb);
	if(frame == read)
	{

	}
	else
	{

	}

#ifdef _DATA_16_BTS_
    //  EPF 00 Frame Data R5 - R1 0|  G5-G0 | B5-B1 |0
	cmd_parameters[0] = 0x55;
	ST7789V_WriteReg(ST7789V_COLOR_MODE,(uint16_t *)&cmd_parameters[0], 1);
#endif

#endif
	//for(int x=0;x<(ST7789V_LCD_PIXEL_WIDTH);x++)
	//{
	//   for(int y =0; y <ST7789V_LCD_PIXEL_HEIGHT;y++)
	//   {
	//     ST7789V_WritePixel(x,y,rgb);
	//
	//
	//   }
	//
	//}

}
/**
  * @}
  */
void LCD_Command(uint16_t command, uint16_t * parameters, uint16_t data_interface, uint16_t number_parameters)
{
	uint32_t i;

	if(data_interface == DATA_8BITS)
	{
		if(number_parameters > 0)
		{
			*LCD_REG = (command<<8);
			for (i=0; i<number_parameters; i++)
			{
				*LCD_RAM =(parameters[i]<<8);
			}
		}
		else
		{
			*LCD_REG = (command<<8);
		}
	}
	else if(data_interface == DATA_16BITS)
	{
		if(number_parameters > 0)
		{
			for (i=0; i<number_parameters; i++)
			{
				*LCD_RAM =(parameters[i]);
			}
			*LCD_REG = (command);
		}
		else
		{
			*LCD_REG = (command);
		}
	}
}
/**
  * @brief  Writes to the selected LCD register.
  * @param  Command: command value (or register address as named in st7789v doc).
  * @param  Parameters: pointer on parameters value (if command uses one or several parameters).
  * @param  NbParameters: number of command parameters (0 if no parameter)
  * @retval None
  */
void ST7789V_WriteReg(uint16_t Command, uint16_t *Parameters, uint8_t NbParameters)
{
	uint8_t   i;

	/* Send command */
	LCD_IO_WriteReg(Command);

	/* Send command's parameters if any */
	for (i=0; i<NbParameters; i++)
	{
		LCD_IO_WriteData(Parameters[i]);
	}
}

/**
 * @brief Backlight control. Set the PWM configuration.
 *      PWMF[7:0] : Set the PWM frequency in system clock. Default after reset value 0.
 *      PWM[7:0] : Set the PWM duty cycle. Default after reset value 0.
 *      NOTE: If DBC enable (0xD0] A0 = 1, these parameter will be ignored
 *      NOTE: PWM always 0 if PWM[7:0] = 00h
 *      C[3] : PWM configuration. Default after reset value 0.
 *          0 PWM controlled by host
 *          1 PWM controlled by DBC
 *      C[0] : PWM enable. Default after reset value 0.
 *          0 PWM disable
 *          1 PWM enable
 *      D[7:0] : DBC manual brightness. Default after reset value 0.
 *          00 Dimmest
 *          FF brightest
 *      E[7:0] : DBC minimum brightness. Default after reset value 0.
 *          00 Dimmest
 *          FF brightest
 *      F[3:0] : Brightness prescaler. Default after reset value 0. This parameter is valid when
 *          Transition Effect enable (0xD0) A5 = 1
 * @param level 0 to 100%
 */
void ST7789V_backlight(uint8_t level)
{
	level =(uint8_t) (level * 2.55);

    /* Send command */
	LCD_IO_WriteReg(ST7789V_WBRIGHTNESS);
	LCD_IO_WriteData(level);
}

#ifdef _8BITS_
/**
  * @}
  */
void LCD_IO_WriteReg(uint16_t Reg)
{
	/* Write 16-bits Index, then Write Reg */
	// FMC_BANK4_WriteReg(Reg);
	*LCD_REG = (Reg<<8);
	__DSB();
}



void LCD_IO_WriteData(uint16_t RegValue)
{
	/* Write 16-bits Reg */
	*LCD_RAM = (RegValue<<8);//SWAP 39CF
	__DSB();
}
#endif

/**
      * @brief  Writes several data on LCD data register.
      * @param  Data: pointer on data to be written
      * @param  Size: data amount in 16bits short unit
      * @retval None
      */
void LCD_IO_WriteMultipleData(uint16_t* pData, uint32_t Size)
{
	uint32_t  i;

	for (i = 0; i < Size; i++)
	{

#ifdef _16_BITS_
#ifdef _DATA_18_BTS_
		LCD_IO_WriteData(GetR(pData[i]) <<10|  GetG(pData[i]) <<2) ;
		LCD_IO_WriteData(GetB(pData[i])<<10);
#endif

#ifdef _DATA_16_BTS_
		LCD_IO_WriteData(pData[i]);
#endif

#endif

#ifdef _8BITS_
#ifdef _DATA_18_BTS_
		LCD_IO_WriteData(GetR(pData[i]));
		LCD_IO_WriteData(GetG(pData[i]));
		LCD_IO_WriteData(GetB(pData[i]));
#endif

#ifdef _DATA_16_BTS_
		/* Write 16-bits Reg */
		//  r     g     b   5  6  5
		LCD_IO_WriteData(pData[i]>>8);
		LCD_IO_WriteData(pData[i]&0xff);
#endif
#endif
	}
}


void LCD_WriteMultipleData(uint16_t* pData, uint32_t Size, uint32_t line)
{
    uint32_t  i;
    uint16_t *lcd_dat = LCD_RAM;

    for (i = 0; i < Size; i++)
    {


#ifdef _DATA_16_BTS_
        /* Write 16-bits Reg */
        //  r     g     b   5  6  5
        *(lcd_dat+i+(Size*line)) = (pData[i]&0xff00);
        *(lcd_dat+i+1+(Size*line))= pData[i]<<8;
        __DSB();
#endif

    }
}

//#endif

uint16_t V_RGB565(uint8_t reverse, uint32_t color24)
{
	uint8_t r = (color24 >> 16) & 0xFF;
    uint8_t g = (color24 >> 8) & 0xFF;
    uint8_t b = color24 & 0xFF;
    if (reverse) return ((b / 8) << 11) | ((g / 4) << 5) | (r / 8);
    else return ((r / 8) << 11) | ((g / 4) << 5) | (b / 8);
}
uint32_t V_RGB666(uint8_t reverse, uint32_t color24)
{
	uint32_t rgb666;
    uint8_t r = (color24 >> 16) & 0xFF;
    uint8_t g = (color24 >> 8) & 0xFF;
    uint8_t b = color24 & 0xFF;
    if (reverse) rgb666 = ((b &0x3f) << 12) | ((g &0x3f) << 6) | (r&0x3f);
    else rgb666 = ((r &0x3f) << 12) | ((g&0x3f) << 6) | (b &0x3f);

    return rgb666;
}


uint32_t ReadFrameData(uint8_t reverse, uint16_t bgr)
{
	uint32_t frame_data=0;
	uint32_t frame_data_inv =0;
	//565
	// b5 b5 b3 b2 b1 |g6 g5 g4 g3 g2 g1 | r5 r4 r3 r2 r1
	uint8_t r = (bgr & 0x1f) ;
	uint8_t g = ((bgr & 0x007e0) >>5);
	uint8_t b = (bgr & 0xF800)>>11;

	frame_data_inv = b <<26 | g<<18 | r<<10;

	frame_data = r <<12 | g<<6 | (b<<1);

	if(reverse)
		return frame_data_inv;
	else
		return frame_data;
}

uint32_t ReadFrameData666(uint8_t reverse, uint16_t bgr)
{
	uint32_t frame_data=0;
	uint32_t frame_data_inv =0;

	// b6 b5 b4 b3 b2 b1 |g6 g5 g4 g3 g2 g1 | r6 r5 r4 r3 r2 r1
	uint8_t r = (bgr & 0x3f) ;
	uint8_t g = ((bgr & 0x00fc0) >>6);
	uint8_t b = (bgr & 0x3F000)>>12;

	frame_data_inv = r <<26 | g<<18 | b<<10;

	frame_data = b <<26 | g<<18 | (r<<10);

	if(reverse)
		return frame_data_inv;
	else
		return frame_data;
}

uint8_t GetR(uint16_t rgb)
{
	//565
	uint16_t r = (rgb&0xf800) >>8;
	return r ;
}

uint8_t GetG(uint16_t rgb)
{
	//565
	uint16_t g = (rgb&0x7e0) >>3;
	return g ;
}

uint8_t GetB(uint16_t rgb)
{
	//565
	uint16_t b = (rgb&0x001f) <<2 ;
	return b ;
}

/**
  * @}
  */


/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
