/****************************************************************************
              Copyright (c) 2020 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : bluetooth.c
   Author             : Alejandro Mendoza
   Version            : V0.0.0.1
   Origination Date   : 03/23/2020
   Description        : C Source File.
   Environment        : Independant
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

		General File Description
-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0              04/04/2011         Name Last_Name

   Notes:              - Initial Version
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1              31/03/2011         Name Last_Name

   Notes:              - Second Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifdef __cplusplus

extern "C"{

#endif

/* Includes ---------------------------------------------------------------*/
#include "../../../Core/Maximatecc/Inc/Common.h"

#if (SDK_SERVICE_BLE == PS_ENABLE)

extern   pvt_variable pv;
BLE_t    ble;
Proto_t  proto_ble;
uint16_t g_inst_id = 1;
uint8_t  Config_mem[1024];
uint8_t  ble_rtimer_flag = 0;
uint8_t  ble_stimer_flag = 0;
uint8_t  ble_read_req = 0;
uint8_t  ble_read_retransmit = 0;
uint8_t  ble_read_retransmit_count = 0;
SemaphoreHandle_t xSemaphore = NULL;
uint8_t  l_BLE_DataPkt[40];

#if 0
/**
  * @brief Initialize the BLE with sending init packet
  * @param None
  * @retval None
  */
void Bluetooth_Init()
{
	//volatile GPIO_PinState status;
	const TickType_t xMaxBlockTime = pdMS_TO_TICKS(100);
	/* Send a Initial Packet until we receive the ACK */
	while(1)
	{
		/* Select the BLE: Chip Select low	*/
		BLE_CS_LOW();

		/* Send the initialize packet to BLE module */
		/* to start the BLE stack initialization    */
		SPI_Send((uint8_t *)BLE_Name,strlen((char *)BLE_Name));

		BLE_CS_HIGH();
#if 1
		/* Wait for receiving the BLE Wakeup Interrupt */
		if( xSemaphoreTake( xSemaphore, xMaxBlockTime ) == pdTRUE )
		{
			/* Processed the interrupt, Again Enable */
			HAL_NVIC_EnableIRQ(EXTI9_5_IRQn);
			break;
		}
#else
		/* Poll the GPIO (PF6) BLE_WAKEUP */
		/* wait until the PF6 GPIO pin not being high */
		while(itr < 50)
		{
			/* Read the WAKEUp Pin */
			status = HAL_GPIO_ReadPin(GPIOF,GPIO_PIN_6);

			/* Receive ACK from BLE chip */
			if (1 == status)
				break;

			itr++;

			HAL_Delay(200);
		}

		/* received ACK, break the loop */
		if (1 == status)
			break;
#endif
	}
}
#endif

uint8_t blesdkpkt[2000];
/** @brief Send the SDK packet
 *
 *  This function sends the SDK packet to the BLE
 *
 *  @param  buff  : Packet
 *          size  : Packet length
 *
 *  @return none
 *
 */
uint8_t sendSDKPkt(uint8_t *buff, uint16_t size)
{
    if (TRUE != pv.blestart_com)
    {
    	return 0;
    }

	/* Construct the packet */
	blesdkpkt[PKT_MODE_INDEX] = TXMODE;
	blesdkpkt[PKT_DADR_INDEX] = DEST_ADDRESS;
	blesdkpkt[PKT_TADR_INDEX] = TOOL_ADDRESS;

	blesdkpkt[PKT_SIZE_INDEX] = size + 2;

	/* Copy the packet to the BLE packet */
	memcpy(&blesdkpkt[PKT_SIZE_INDEX+1], (uint8_t *)buff, size);

	/* Update the checksum */
	blesdkpkt[CMD_PKT_HEADER_SIZE + size + 2] =
			CalCheckSum(&blesdkpkt[PKT_SIZE_INDEX+1], size + 2);

	/* Send the data Packet */
    TakeSPIBusLock();
    Send_BLE_PKT((uint8_t *)&blesdkpkt[0], CMD_PKT_HEADER_SIZE + size + 3);
	GiveSPIBusLock();

	return 1;
}

#if 0
/**
  * @brief Wait for receiving Initialization ACK from the BLE Module
  * @param ACKRSPdata  -  Receiving ACK Message  
  *        No_OfRetry  -  Number of retries
  * @retval None
  */
uint8_t Waitfor_BLE_RSP(uint8_t *ACKRSPdata, uint16_t No_OfRetry)
{
	uint8_t Retry_cnt = 0;
	
	do
	{
		/* Call SPI read function to read command/data from BLE module */
		if(HAL_SPI_Receive(pv.hspi2, &ble.Rx_Buff[0], strlen((const char *)ACKRSPdata),
				strlen((const char *)ACKRSPdata)*2) != HAL_OK)
		{
			Error_Handler();
			return Fail;
		}

		/* Compare the received ACK data */
		if(strcmp((const char *)ACKRSPdata,(const char *)&ble.Rx_Buff[0]) == 0)
		{
			return Success;
		}
		else
		{
			/* wait for ACK/NACK timeout*/
			HAL_Delay(100);

			/*increment retry count*/
			Retry_cnt ++;

			/* retry the same data */
			SPI_Send((uint8_t *)BLE_Name,strlen((char *)BLE_Name));
		}
	
	}while(Retry_cnt < No_OfRetry);

	return Fail;
}
#endif

/**
  * @brief : SPI_Send
  * BLE Data will be Send through SPI
  * @param None
  * @note
  * @retval True or False
  */
void SPI_Send(uint8_t *Data, uint8_t Len)
{
	HAL_StatusTypeDef spiBLETransmitStatus;

	while (pv.hspi2->State != HAL_SPI_STATE_READY)
	{
		HAL_Delay(1);
	}

	/* Make 5 attemtps to write the data */
	for(uint8_t i = 0; i < 5; i++)
	{
		spiBLETransmitStatus = HAL_SPI_Transmit(pv.hspi2, Data, Len,Len*50);

		if(spiBLETransmitStatus == HAL_BUSY)
		{
			HAL_Delay(5);
        }
		else
		{
			break;
		}
	}
}


/**
  * @brief : Sending data to the PC
  * @param len       : data packet size
  *        Cdata       : Data buffer
  * @note
  * @retval None
  */
void Send_BLE_PKT(uint8_t *Cdata, uint8_t len)
{
    /*	Select the BLE: Chip Select low	*/
	BLE_CS_LOW();

	SPI_Send(Cdata, len);

    /* Deselect the BLE: Chip Select high */
	BLE_CS_HIGH();
}

/**
  * @brief : Sending Config data to the PC
  * @param sizeofpkt : data packet size
  *        Fun_Code  : Function code
  *        CByte     : Command code
  *        Instance_ID : Packet Instance ID
  *        Cdata       : Data buffer
  * @note
  * @retval True or False
  */
void Send_BLE_Data(uint8_t sizeofpkt, 
                             uint8_t Fun_Code,
                             uint8_t CByte,
                             uint8_t Instance_ID,
                             uint8_t *Cdata)
{

	uint8_t index = 0;

	l_BLE_DataPkt[index++] = sizeofpkt;
	l_BLE_DataPkt[index++] = Fun_Code;
	l_BLE_DataPkt[index++] = CByte;
	l_BLE_DataPkt[index++] = Instance_ID;
	
	for(uint8_t i=0;i < sizeofpkt; i++)
		l_BLE_DataPkt[index++] = *(Cdata + i);

    /*	Select the BLE: Chip Select low	*/
	BLE_CS_LOW();
	
	SPI_Send(&l_BLE_DataPkt[0], index);

    /* Deselect the BLE: Chip Select high */
	BLE_CS_HIGH();

}

/**
  * @brief : Sending Config data to the PC
  * @param sizeofpkt : data packet size
  *        Fun_Code  : Function code
  *        CByte     : Command code
  *        Instance_ID : Packet Instance ID
  *        Cdata       : Data buffer
  * @note
  * @retval True or False
  */
void Send_BLE_Name(uint8_t sizeofpkt,
                             uint8_t Fun_Code,
                             uint8_t CByte,
                             uint8_t Instance_ID,
                             uint8_t *Cdata)
{

	uint8_t index = 0;

	for(uint8_t i=0;i < sizeofpkt; i++)
		l_BLE_DataPkt[index++] = *(Cdata + i);

	strncat((char *)&l_BLE_DataPkt, "#1234#", 8);
	index += 8;

    /*	Select the BLE: Chip Select low	*/
	BLE_CS_LOW();

	SPI_Send(&l_BLE_DataPkt[0], index);

    /* Deselect the BLE: Chip Select high */
	BLE_CS_HIGH();

}

/**
  * @brief : Send the Response packet to PC
  * @param sizeofpkt : data packet size
  *        Fun_Code  : Function code
  *        CByte     : Command code
  *        Instance_ID : Packet Instance ID
  * @note
  * @retval none
  */
void Send_BLE_Packet_Rsp(uint8_t sizeofpkt, uint8_t Fun_Code,uint8_t CByte,
				uint8_t Instance_ID)
{
	uint8_t l_BLE_RspPkt[8];
	
	memset(&l_BLE_RspPkt[0], 000, sizeof(l_BLE_RspPkt));

	l_BLE_RspPkt[4] = sizeofpkt;
	l_BLE_RspPkt[5] = Fun_Code;
	l_BLE_RspPkt[6] = CByte;
	l_BLE_RspPkt[7] = Instance_ID;

	/* Select the BLE: Chip Select low	*/
	BLE_CS_LOW();
	
	SPI_Send(&l_BLE_RspPkt[0], RSP_PKT_SIZE);

	/* Deselect the BLE: Chip Select high */
	BLE_CS_HIGH();
}

/**
  * @brief : Process the password verification packet
  * @param Data : Received data packet
  * @note
  * @retval None
  */
void VerifyPassword(Proto_t  *pkt)
{
	/* Check the Password packet size */
	if(pkt->Pkt_size > PASS_PKT_SIZE)
		return;

	/* Check the instance ID */
	if(PASSWORD_INSTANCE_ID != pkt->instance_ID)
		return;

	/* Validate command byte */
	if(CMD_PC_WRITE !=  pkt->command_Byte)
		return;

	/*validate received passcode*/
	if(strncmp((const char *)&pkt->Data[0],(const char *)BLE_DEV_PASSCODE, 4) == 0)
	{
		/*valid passcode*/
		/*send ack passcode*/
		Send_BLE_Data(strlen((char const *)BLE_APP_PASSCODE),
						FUNC_ECU_PC_PASS_RESP,
						CMD_PC_WRITE,
						pkt->instance_ID,
						(uint8_t *)BLE_APP_PASSCODE);

		/*set passcode validate flag*/
		ble.passcode = SET;
	}
	else
	{
		/*invalid passcode*/
		/*send 0's in passcode field*/
		Send_BLE_Data(strlen((char const *)BLE_NAK_PASSCODE),
						FUNC_ECU_PC_PASS_RESP,
						CMD_PC_WRITE,
						pkt->instance_ID,
						(uint8_t *)BLE_NAK_PASSCODE);
	}
}


/**
  * @brief : Process the received write data from PC
  * @param Data : Received data packet
  * @note
  * @retval None
  */
void BLE_Write_Data(Proto_t  *pkt)
{
	/* Check Password verification completed */
	if(SET != ble.passcode)
		return;

	/*  Check for the Zero Packet size */
	if( 0 == pkt->Pkt_size)
	{
		/* This is last packet, write reqest is completed */
		/* Send the ACK to PC BLE APP */
		Send_BLE_Packet_Rsp(0, CMD_DATA_RX_RSP, CMD_PC_WRITE, pkt->instance_ID);
		
		g_inst_id = 1;
		ble_rtimer_flag = 0;
		return;
	}

	/* Check for the Packet size */
	if(DATA_PKT_SIZE != pkt->Pkt_size)
		return;

	/* Check for the Write cmd Request */
	if (CMD_PC_WRITE != pkt->command_Byte)
		return;

	/* Check the intstance ID */
	if(pkt->instance_ID != g_inst_id)
		return;

	/* Check for the RAM exceed size */
	if ((g_inst_id*8) >= CONFIG_MEM_RAM_SIZE)
		return;

	/* Write data into the RAM */
	memcpy(Config_mem + (g_inst_id * 8), &pkt->Data, DATA_PKT_SIZE);

	ble_rtimer_flag = 1;
	g_inst_id++;

	/* Send the ACK to PC BLE APP */
	Send_BLE_Packet_Rsp(0, CMD_DATA_RX_RSP, CMD_PC_WRITE, pkt->instance_ID);
}


/**
  * @brief : Process the received read data request from PC
  * @param Data : Received data packet
  * @note
  * @retval None
  */
void BLE_Read_Data(Proto_t  *pkt)
{
	/* Check Password verification completed */
	if(SET != ble.passcode)
		return;

	/* Check for the read request or Response */
	/* Read request packet */
	if(CMD_PC_WRITE == pkt->command_Byte)
	{
		/* Check the Password packet size */
		if(0 != pkt->Pkt_size)
			return;

		/* Check the instance ID */
		if (1 != pkt->instance_ID)
			return;

		/* Start with first instance ID */
		g_inst_id = 1;
#if 0
		/* Send ACK to the PC Application*/
		Send_BLE_Packet_Rsp(0, pkt->function_code,pkt->command_Byte, pkt->instance_ID);

		/* Give some delay */
		vTaskDelay(3000);
#endif
		/* */
		memset(Config_mem + (g_inst_id * 8), 0x11, 0x8);

		/* Send the data Packet */
		Send_BLE_Data(DATA_PKT_SIZE,
						B_Data_Read_Pkt_Req,
						CMD_PC_READ,
						g_inst_id,
						Config_mem + (g_inst_id * 8));

		g_inst_id ++;

		/* Enable read request flag */
		ble_read_req = 1;
	}
	else
	{
		/* Read Response packet */
		if(CMD_PC_READ == pkt->command_Byte)
		{
			/* Check whether Read cmd packet is received */
			if (1 != ble_read_req)
				return;

			/* Check the Password packet size */
			if(0 != pkt->Pkt_size)
				return;

			/* Check the instance ID */
			if (g_inst_id != pkt->instance_ID)
				return;

			/* Check the read command */
			if (CMD_PC_READ != pkt->command_Byte)
				return;

			/* Last Packet */
			if ((g_inst_id * 8) == CONFIG_MEM_RAM_SIZE)
			{
				/* Reset the instance ID */
				g_inst_id = 1;
				
				/* Send the Next set of data */
				Send_BLE_Data(0,B_Data_Read_Pkt_Req,
								CMD_PC_READ,
								g_inst_id,
								Config_mem + (g_inst_id * 8));
				return;
			}

			g_inst_id++;

			/* Send the Next set of data */
			Send_BLE_Data(DATA_PKT_SIZE,
							B_Data_Read_Pkt_Req,
							CMD_PC_READ,
							g_inst_id,
							Config_mem + (g_inst_id * 8));

			ble_read_retransmit = 1;
		}
	}
}

/**
  * @brief : Process the received packet
  * @param Data : Received data packet
  * @note
  * @retval True or False
  */
uint8_t proto_Data_process(uint8_t *Data)
{
	Proto_t        *RX_Ble_Data;

	RX_Ble_Data = (Proto_t *) Data;

	/* Now check the function code */
	switch(RX_Ble_Data->function_code)
	{
		/* Received Password Validate Request from PC */
		case B_Password_Validate_Req:

			/* Verify the password */
			VerifyPassword(RX_Ble_Data);
			break;

		/* Received Write data packet from PC */
		case B_Data_Write_Pkt_Req:
           
			/* Process the received Write data packet */
			BLE_Write_Data(RX_Ble_Data);
			break;

		/* Received Read data Packet Request from PC */
		case B_Data_Read_Pkt_Req_Resp:

			/* Process the read data request */
			BLE_Read_Data(RX_Ble_Data);
			break;

		default:

		/* Don't do anything */
			break;
	}

	return TRUE;
}

/**
  * @brief : IS_SPI_Data_Available
  BLE Data will be read through SPI
  * @param None
  * @note
  * @retval True or False
  */
uint8_t IS_SPI_Data_Available()
{
	/* Receive timer the flag */
	/* ble_rtimer_flag > 0 is something received */
	if (ble_rtimer_flag > 0)
		ble_rtimer_flag++;

	/* Receive timeout reset g_inst_id */
	if (RECV_TIMEOUT == ble_rtimer_flag)
	{
		ble_rtimer_flag = 0;
		g_inst_id = 1;
	}

	/* Clear the buffer */
	memset(&ble.Rx_Buff[0], 0x00, sizeof(ble.Rx_Buff));
      
    /* Select the BLE: Chip Select low	*/
	BLE_CS_LOW();

	if(HAL_SPI_Receive(pv.hspi2, &ble.Rx_Buff[0], 12, 12*2) != HAL_OK)
	{
		//Error_Handler();
		/* Select the BLE: Chip Select high */
		BLE_CS_HIGH();
		return FALSE;
	}
    
    /* Select the BLE: Chip Select high */
	BLE_CS_HIGH();

	proto_Data_process(&ble.Rx_Buff[0]);

	return TRUE;
}

#if 0
/**
  * @brief  External Interrupt Callback function for the BLE Wake Up pin
  * @param  Triggered Interrupt pin
  * @retval None
  */
void EXTI9_5_IRQHandler()
{
	BaseType_t xHigherPriorityTaskWoken = pdFALSE;

	/* USER CODE BEGIN EXTI4_IRQn 0 */
	__HAL_GPIO_EXTI_CLEAR_IT(GPIO_PIN_6);
	__HAL_GPIO_EXTI_CLEAR_FLAG(GPIO_PIN_6);

	if (NULL != xSemaphore)
	{
		/* Unblock the task by releasing the semaphore. */
		xSemaphoreGiveFromISR( xSemaphore, &xHigherPriorityTaskWoken );

		HAL_NVIC_DisableIRQ(EXTI9_5_IRQn);
	}
	
	/* If xHigherPriorityTaskWoken is now set to pdTRUE then a context switch
	 * should be performed to ensure the interrupt returns directly to the highest
	 * priority task.  The macro used for this purpose is dependent on the port in
	 * use and may be called portEND_SWITCHING_ISR(). */
	portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
	//NVIC_SystemReset();
}
#endif

#if 0
/**
  * @brief  Initialize the BLE Wake Up Interrupt pin
  * @param  None
  * @retval None
  */
void BLEWakeupExternal_interrupt_init()
{
	GPIO_InitTypeDef GPIO_InitStruct={0};

	/* Make the BLE_WAKEUP Pin as interrupt */
	GPIO_InitStruct.Pin = GPIO_PIN_6;	//BLE_WAKEUP_GPIOF_PIN_6
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;

	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

	HAL_NVIC_SetPriority(EXTI9_5_IRQn, 5,0);
	HAL_NVIC_EnableIRQ(EXTI9_5_IRQn);
}
#endif

#if 0
/**
  * @brief  BLE Main thread function
  * @param  None
  * @retval None
  */
uint8_t Bluetooth_task()
{
	GPIO_InitTypeDef GPIO_InitStruct1= {0};
	const TickType_t xMaxBlockTime = pdMS_TO_TICKS( 200 );

	/* We are using the semaphore for synchronisation so we create a binary
	 * semaphore rather than a mutex.  We must make sure that the interrupt
	 * does not attempt to use the semaphore before it is created! */
	xSemaphore = xSemaphoreCreateBinary();

#if 1
	/* Initialize the BLE Wakeup pin as interrupt */
	BLEWakeupExternal_interrupt_init();
#else

	/* Make the BLE_WAKEUP Pin to input */
	GPIO_InitStruct1.Pin = GPIO_PIN_6;
	GPIO_InitStruct1.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct1.Pull = GPIO_NOPULL;
	GPIO_InitStruct1.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct1);
#endif
	
	/* Give 2 Seconds delay for the BLE chip to up */
	vTaskDelay(2000);

	/* Make the CS pin to Output */
	GPIO_InitStruct1.Pin = GPIO_PIN_7;
	GPIO_InitStruct1.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct1.Pull = GPIO_NOPULL;
	GPIO_InitStruct1.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct1);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_7, GPIO_PIN_SET);

	/*Config BLE module*/
	Bluetooth_Init();

	for(;;)
	{
		/* Wait for receiving the BLE Wakeup Interrupt */
		if( xSemaphoreTake( xSemaphore, xMaxBlockTime ) == pdTRUE )
		{
			/* Processed the interrupt, Again Enable */
			HAL_NVIC_EnableIRQ(EXTI9_5_IRQn);

			/*check for any data availbale*/
			IS_SPI_Data_Available();
		}
		else
		{
			/* Timedout, Check for the retransmission */
			/* Retransmit the Read Packet       */
			/* Since we did not receive ACK     */
			if (1 == ble_read_retransmit)
			{
				ble_read_retransmit_count ++;
				if (ble_read_retransmit_count > MAX_RETRANSMIT_COUNT)
				{
					g_inst_id = 1;
					ble_read_retransmit = 0;
					ble_read_retransmit_count = 0;
				}
				else
				{
					/* Send the Next set of data */
					Send_BLE_Data(DATA_PKT_SIZE,
									B_Data_Read_Pkt_Req,
									CMD_PC_READ,
									g_inst_id,
									Config_mem + (g_inst_id * 8));
				}
			}
		}
	}
}
#endif

#endif	//SDK_SERVICE_BLE

#ifdef __cplusplus

}

#endif

/**************** (C) COPYRIGHT 2020 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
