/*
* j1939_j1939.h - contains defines for all services
*
* Copyright (c) 2014-2021 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_j1939.h 38942 2021-11-26 14:32:18Z boe $
*-------------------------------------------------------------------
*
*
*/

/**
* \brief defines for all services
*
* \file j1939_j1939.h - contains defines for all services
*
* This header inludes defines for all services of the J1939 library.
* It can be included instead of header files of each service.
*
*/

#ifndef J1939_J1939_H
#define J1939_J1939_H 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define UNSIGNED24_T    UNSIGNED32


#include <co_datatype.h>
#include <co_common.h>
#include <co_cob.h>
#include <co_commtask.h>
#include <co_drv.h>
#include <co_timer.h>

#include <../../../../Core/Maximatecc/user_modify_files/j1939_datatype.h>
#include <j1939_indication.h>


#include <j1939_slots.h>

#ifdef J1939_DYNAMIC_PGN
#include <j1939_dynpgn.h>
#endif /* J1939_DYNAMIC_PGN */

#if defined(J1939_DIAGNOSTIC_SUPPORTED) || defined(J1939_DIAGNOSTIC_TOOL_SUPPORTED)
#include <j1939_diagnostic.h>
#endif /*  J1939_DIAGNOSTIC_SUPPORTED */

#ifdef J1939_NMEA2000_SUPPORTED
# include <j1939_nmea2000.h>
#endif /* J1939_NMEA2000_SUPPORTED */

#ifdef J1939_SDG_SUPPORTED
# include <j1939_sdg.h>
#endif /* J1939_SDG_SUPPORTED */

#ifdef J1939_RV_C_SUPPORTED
# include <j1939_rvc.h>
#endif /* J1939_RV_C_SUPPORTED */

#ifdef J1939_CAM11_CAM21_SUPPORTED
# include <j1939_cam11_cam21.h>
#endif /* J1939_CAM11_21_SUPPORTED */


/* actual J1939 stack software version */
#ifdef CANOPEN_SUPPORTED
#include <co_canopen.h>
#endif /* CANOPEN_SUPPORTED */
#ifndef J1939_STACK_VERSION
# define J1939_STACK_VERSION	0x30701ul
#endif /* CO_STACK_VERSION */


/* SPN lock macros */
#ifndef CO_OS_LOCK_OD
# define CO_OS_LOCK_OD
#endif /* CO_OS_LOCK_OD */
#ifndef CO_OS_UNLOCK_OD
# define CO_OS_UNLOCK_OD
#endif /* CO_OS_LOCK_OD */

#define DP_NMEA2000				0x10000ul	/* datapage NMEA2000 */
#define DP_ISO					0x30000ul	/* datapage ISO */

#define PGN_MULTI_PG			0x2500ul

#define PGN_FD_TP_CMD			0x4d00ul
#define PGN_FD_TP_DATA			0x4e00ul

#define PGN_TP_CMD				60416ul
#define PGN_TP_DATA				60160ul

#define PGN_ACKNOWLEDGMENT		0xe800ul
#define PGN_REQUEST_PGN			0xea00ul
#define PGN_ADDR_CLAIM_REQ		0xee00ul

#define PGN_NMEA2000			(DP_NMEA2000 | 0xfed0ul)

#define PGN_PROPRIETARY_A		0xef00ul
#define PGN_DST_SPECIFIC		0xf000ul
#define PGN_PDU_FORMAT1_START	0xf000ul /*  + group extension */
#define PGN_PDU_FORMAT1_END		0xff00ul /*  + group extension */
#define PGN_PDU_FORMAT2_START	0x0000ul /*  + destination address */
#define PGN_PDU_FORMAT2_END		0xef00ul /*  + destination address */
#define PGN_PROPRIETARY_B_START	0xff00ul
#define PGN_PROPRIETARY_B_END	0xfffful



typedef struct {
	UNSIGNED16	structVersion;
	UNSIGNED16	receivePgnCnt[CO_MULTI_LINE_CNT];
	UNSIGNED16	transmitPgnCnt[CO_MULTI_LINE_CNT];
	UNSIGNED8	sdoServerCnt[CO_MULTI_LINE_CNT];
	UNSIGNED8	sdoClientCnt[CO_MULTI_LINE_CNT];
} J1939_SERVICE_INIT_VAL_T;


/* function prototypes */

/**
* \brief j1939_stackInit - init of J1939 stack 
*
* This function is normally generated by the J1939 Device Designer
* and responsible for the intialization of the J1939 stack.
*
* \return RET_T
*/
EXTERN_DECL void j1939drvHardwareInit(void);

EXTERN_DECL RET_T j1939drvTimerSetup(UNSIGNED32 timerInterval);

EXTERN_DECL RET_T j1939drvCanInit(CO_LINE_DECL, UNSIGNED16 bitRate);
EXTERN_DECL RET_T j1939drvCanEnable(CO_LINE_DECL);
EXTERN_DECL RET_T j1939drvCanDisable(CO_LINE_DECL);
#ifdef J1939_CAN_DEINIT
EXTERN_DECL RET_T j1939drvCanDeinit(CO_LINE_DECL);
#endif /* J1939_CAN_DEINIT */

EXTERN_DECL RET_T j1939StackInit(void);
EXTERN_DECL RET_T j1939StackInit_common(const J1939_SERVICE_INIT_VAL_T *initVars);
EXTERN_DECL RET_T j1939StackInit_line(CO_LINE_DECL, const J1939_CONFIG_T *pConfig);

EXTERN_DECL BOOL_T j1939CommTask(CO_LINE_DECL);

EXTERN_DECL RET_T j1939AddressClaimingStart(CO_LINE_DECL, UNSIGNED8 newAddress);
EXTERN_DECL RET_T j1939SendPgn(CO_LINE_DECL, UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939SendPgnAddr(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED8 destNode);
EXTERN_DECL RET_T j1939SendCmdt(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED8 dstNode);
EXTERN_DECL RET_T j1939SendBam(CO_LINE_DECL, UNSIGNED32 pgn);

#ifdef J1939_RV_C_SUPPORTED
EXTERN_DECL RET_T j1939RequestPgn(CO_LINE_DECL, UNSIGNED32 pgn,
			UNSIGNED8 destination, UNSIGNED8  instance, UNSIGNED8 instanceBank);
#else /* J1939_RV_C_SUPPORTED */
EXTERN_DECL RET_T j1939RequestPgn(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED8 destination);
#endif /* J1939_RV_C_SUPPORTED */

EXTERN_DECL RET_T j1939ChangeDestinationAddress(CO_LINE_DECL, UNSIGNED32 pgn,
				UNSIGNED8 dstAddress);

EXTERN_DECL RET_T j1939ProprietaryTransmit_A(CO_LINE_DECL, UNSIGNED8 destNode);
EXTERN_DECL RET_T j1939ProprietaryTransmit_A_Length(CO_LINE_DECL, UNSIGNED8 destNode, UNSIGNED16  length);
EXTERN_DECL RET_T j1939ProprietaryTransmit_B(CO_LINE_DECL, UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939ProprietaryTransmit_B_Length(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED16 length);

EXTERN_DECL RET_T j1939PgnDisable(CO_LINE_DECL, DIRECTION_T direction, UNSIGNED32 pgn);
EXTERN_DECL RET_T j1939PgnEnable(CO_LINE_DECL, DIRECTION_T direction, UNSIGNED32 pgn);

EXTERN_DECL CO_CAN_TIMESTAMP_T j1939PgnGetTimestamp(CO_LINE_DECL, UNSIGNED32 pgn);


EXTERN_DECL RET_T j1939SpnGet_u64(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED64 *pValue);
EXTERN_DECL RET_T j1939SpnGet_u32(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED32 *pValue);
EXTERN_DECL RET_T j1939SpnGet_u16(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED16 *pValue);
EXTERN_DECL RET_T j1939SpnGet_u8(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED8 *pValue);
EXTERN_DECL RET_T j1939SpnGet_vs(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, VIS_STRING *pValue, UNSIGNED16 maxBuf);

EXTERN_DECL RET_T j1939SpnPut_u64(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED64 value);
EXTERN_DECL RET_T j1939SpnPut_u32(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED32 value);
EXTERN_DECL RET_T j1939SpnPut_u16(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED16 value);
EXTERN_DECL RET_T j1939SpnPut_u8(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, UNSIGNED8 value);
EXTERN_DECL RET_T j1939SpnPut_vs(CO_LINE_DECL, UNSIGNED32 pgn, UNSIGNED32 spn,
				DIRECTION_T direction, const VIS_STRING value, UNSIGNED16 size);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* J1939_J1939_H */
