﻿/*
* j1939_rvc.h - contains defines for rv-c extension
*
* Copyright (c) 2021 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_rvc.h 37253 2021-07-22 09:15:17Z boe $
*-------------------------------------------------------------------
*
*
*
*/

/**
* \brief data types
*
* \file j1939_rvc.h - contains defines for rv-c
*
*/

#ifndef J1939_RVC_H
#define J1939_RVC_H 1

#define J1939_PGN_DM_RV			0x1fecau
#define J1939_PGN_PRODUCT_ID	0xfeebu

/* datatypes */

typedef enum {
	DM_RV_OPSTATE_OFF_STDBY = 0,	/**< disabled and not operating */
	DM_RV_OPSTATE_OFF_ACTIVE = 4,	/**< disabled but is running */
	DM_RV_OPSTATE_ON_STDBY = 1,		/**< not operating, but will accept commands */
	DM_RV_OPSTATE_ON_ACTIVE = 5		/**< operating, accept commands */
} DM_RV_OPSTATE_T;


typedef struct {
	UNSIGNED8	trBuf[8];			/**< transmit buffer */
	CO_TIMER_T	timer;				/**< timer struct */
} DM_RV_DATA_T;


/** \brief function pointer to receive RV-C Diagnostic message
 *
 * This function is called if an RV-C message was received
 *
 * \param node
 * 			src node
 * \param operatingState
 * 			operating state
 * \param yellowLampState
 * 			yello lamp state
 * \param redLampState, 
 * 			red lamp state
 * \param dsa
 * 			DSA
 * \param spn
 * 			SPN
 * \param fmi
 * 			FMI
 * \param ocCnt
 * 			occurence count
 * \param bankSelect
 * 			bank select
 *
 * \return
 *              new address (1..240, 255)
 */
typedef void (* J1939_EVENT_RVC_DM_RECEIVED_T)(CO_LINE_TYPE, UNSIGNED8,
		DM_RV_OPSTATE_T, UNSIGNED8, UNSIGNED8, UNSIGNED8, UNSIGNED32,
		UNSIGNED8, UNSIGNED8, UNSIGNED8);




extern RET_T j1939Rvc_DM_RV(CO_LINE_DECL, DM_RV_DATA_T *dataBuffer,
		DM_RV_OPSTATE_T	operatingState, UNSIGNED8 yellowLampState,
		UNSIGNED8 redLampState, UNSIGNED8 dsa, UNSIGNED32 spn, UNSIGNED8 fmi,
		UNSIGNED8 ocCnt, UNSIGNED8 bankSelect, UNSIGNED32 transRate);
extern RET_T j1939EventRegister_RECEIVE_RVC_DM(J1939_EVENT_RVC_DM_RECEIVED_T);

#endif /* J1939_RVC_H */

